#include "uCalc.h"
#include <cstdio>
#include <string>
#include <cmath>
#include <iostream>
#include "UserCallbacks.h"

using namespace std;

void MyErrorHandler(uCalcPtr uCalcHandle)
{
   auto uc = uCalc(uCalcHandle);
   cout << endl;
   cout << "uCalc Error (this message box is part of the demo)" << endl << endl;
   cout << "Error Number: " << uc.ErrorNumber() << endl;
   cout << "Bad expression: " << uc.ErrorExpression() << endl;
   cout << "Error Handler message: " << uc.ErrorMessage() << endl;
   cout << "Offending symbol: " << uc.ErrorSymbol() << endl;
   cout << "Error Location: " << uc.ErrorLocation() << endl;
   cout << "See MyErrorHandler() in UserCallbacks.cpp for error handler source code." << endl;
}

std::string UserFormat;

void MyNumericFormat(uCalcPtr ExprPartPtr) {
   auto uc = uCalc::Callback(ExprPartPtr);
   char FormattedResult[50];
   sprintf(FormattedResult, UserFormat.c_str(), stod(uc.ArgStr(1)));

   if (UserFormat != "")   
      uc.ReturnStr(FormattedResult);
   else
      uc.ReturnStr(uc.ArgStr(1));
}


void MyArea(uCalcPtr ExprPartPtr)
{
   auto uc = uCalc::Callback(ExprPartPtr);
   auto MyLength = uc.Arg(1); // Same as uCalc.Arg(Expr, 1);   
   auto MyWidth = uc.Arg(2);         

   if(MyLength < 0) uc.ErrorRaiseMessage("Length cannot be negative");
   if(MyWidth  < 0) uc.ErrorRaiseMessage("Width cannot be negative");

   uc.Return(MyLength * MyWidth);
}

/* +++

long double NonNativeFunc(double a, long& b, byte c)
{
return a + b + c;
}
*/


void MyIIF_Numeric(uCalcPtr ExprPartPtr)
{
   auto uc = uCalc::Callback(ExprPartPtr);
   auto Condition = uc.Arg(1);
   auto TruePart = uc.ArgExpr(2);
   auto FalsePart = uc.ArgExpr(3);

   if(Condition != 0)
      uc.Return(TruePart.Evaluate());
   else 
      uc.Return(FalsePart.Evaluate());
}


void MyIIF_String(uCalcPtr ExprPartPtr)
{
   auto uc = uCalc::Callback(ExprPartPtr);   
   auto Condition = uc.Arg(1);
   auto TruePart = uc.ArgExpr(2);
   auto FalsePart = uc.ArgExpr(3);

   if(Condition != 0)
      uc.ReturnStr(TruePart.EvaluateStr());
   else 
      uc.ReturnStr(FalsePart.EvaluateStr());
}

/* +++

void Native_MsgBox(DWORD Expr)
{
// The 2nd and 3rd args are swapped so that MsgBox is uniform with the demo for VB and other compilers
ucReturn(Expr, MessageBox(0, ucParamStr(Expr, 1), ucParamStr(Expr, 3), ucParam(Expr, 2)));
}


long MyMsgBox(long Prompt, long Buttons, long Title)
{
return MessageBox(0, ucEvaluateStr(Prompt), ucEvaluateStr(Title), Buttons);
}


long nn_MsgBox(CString& Prompt, long Buttons, CString& Title)
{
return MessageBox(0, Prompt, Title, Buttons);
}



void MyMsgBox(uCalcPtr ExprPartPtr)
{
   auto uc = uCalc::Callback(ExprPartPtr); // Args 2 & 3 inverted because it's based on VB MsgBox syntax
   uc.Return(MessageBoxA(0, uc.ArgStr(1).c_str(), uc.ArgStr(3).c_str(), uc.ArgInt32(2)));   
}
*/


void MyAverage(uCalcPtr ExprPartPtr)
{
   auto uc = uCalc::Callback(ExprPartPtr); 
   double Total = 0;

   for(int x=1; x <= uc.ArgCount(); x++)
      Total = Total + uc.Arg(x);        

   uc.Return(Total / uc.ArgCount());
}


void MyLeft(uCalcPtr ExprPartPtr)
{
   auto uc = uCalc::Callback(ExprPartPtr);
   uc.ReturnStr(uc.ArgStr(1).substr(0, uc.ArgInt32(2)));
}


void StringRepeat(uCalcPtr ExprPartPtr)
{   
   auto uc = uCalc::Callback(ExprPartPtr); 
   string TotalString = "";

   for(int x = 1; x <= uc.Arg(2); x++)
      TotalString = TotalString + uc.ArgStr(1);

   uc.ReturnStr(TotalString);
}


void ucSum(uCalcPtr ExprPartPtr)
{
   auto uc = uCalc::Callback(ExprPartPtr); 
   double Total = 0;

   auto Expression = uc.ArgExpr(1);
   auto Start = uc.Arg(2);
   auto Finish = uc.Arg(3);
   auto sStep = uc.Arg(4);
   auto VarHandle = uc.ArgObj(5);

   for (double x = Start; x <= Finish; x += sStep) 
   {
      VarHandle.SetVariableValue(x);
      Total = Total + Expression.Evaluate();
      x = x + sStep;
   }

   uc.Return(Total);   
}


void ucSolve(uCalcPtr ExprPartPtr)
{
   auto uc = uCalc::Callback(ExprPartPtr); 
   int Iterations = 0;
   double fa, fb, Value = 0, tmp;

   auto Expression = uc.ArgExpr(1);
   auto a = uc.Arg(2);
   auto b = uc.Arg(3);
   auto Variable = uc.ArgObj(4);         

   Variable.SetVariableValue(a);
   fa = Expression.Evaluate();

   Variable.SetVariableValue(b);
   fb = Expression.Evaluate();

   if(fb < fa) {tmp = a; a = b; b = tmp;}  // swap a, b

   while(fabs(b - a) > 0.000000000000001)
   {
      Variable.SetVariableValue((a + b) / 2);

      Value = Expression.Evaluate();

      if(Value == 0) {a = (a + b) / 2; break;}

      if(Value < 0) a = (a + b) / 2; else b = (a + b) / 2;

      Iterations = Iterations + 1;
      if(Iterations == 100) break;
   }

   if(fabs(Value) > 0.0000000001) uc.ErrorRaiseMessage("Solution not found");
   uc.Return(a);
}
