//#include "stdafx.h"
#include "uCalcNET.h"
#include "Form1.h"

static void MyErrorHandler(uCalcPtr uCalcHandle)
{
	auto uc = uCalc(uCalcHandle);
	static bool DisplayAgain = true;
	
	if (DisplayAgain) {	   
		auto Option =  MessageBox::Show(
			      "uCalc Error (this message box is part of the demo)\n\n"
                + "Error Number: " + Convert::ToString(uc.ErrorNumber()) + "  [" /* + Enum::GetName(ErrorNumberEnum::typeid, uc.ErrorNumber()) */ + "]\n"
                + "Bad expression: " + uc.ErrorExpression() + "\n"
                + "Error Handler message: " + uc.ErrorMessage() + "\n"
                + "Offending symbol: " + uc.ErrorSymbol() + "\n"
                + "Error Location: " + Convert::ToString(uc.ErrorLocation()) + "\n\n"
                + "See MyErrorHandler(), in MyFunctions.vb, for error handler source code.\n\n"
                + "Show this message next time?", "uCalc Error (this message box is part of the demo)", MessageBoxButtons::YesNoCancel);
		if (Option == System::Windows::Forms::DialogResult::No) DisplayAgain = false;
   }
}

static void MyNumericFormat(uCalcPtr ExprPartPtr) {
	auto uc = uCalc::Callback(ExprPartPtr);
	String ^ ResultAsString = uc.ArgStr(1);

	try {
		String ^ FormattedResult = String::Format(Form1::NumericFormat, Convert::ToDouble(ResultAsString));
		uc.ReturnStr(FormattedResult);
	} catch(...) { // +++ For special values like -1.#IND, -1.#INF, etc..
		uc.ReturnStr(ResultAsString);
	}	
}


static void MyArea(uCalcPtr ExprPartPtr)
{
	auto uc = uCalc::Callback(ExprPartPtr);
	auto MyLength = uc.Arg(1);
	auto MyWidth = uc.Arg(2);			

	if(MyLength < 0) uc.ErrorRaiseMessage("Length cannot be negative");
	if(MyWidth  < 0) uc.ErrorRaiseMessage("Width cannot be negative");

	uc.Return(MyLength * MyWidth);
}

/* +++

long double _stdcall NonNativeFunc(double a, long& b, byte c)
{
return a + b + c;
}
*/


static void MyIIF_Numeric(uCalcPtr ExprPartPtr)
{
	auto uc = uCalc::Callback(ExprPartPtr);
	auto Condition = uc.Arg(1);
	auto TruePart = uc.ArgExpr(2);
	auto FalsePart = uc.ArgExpr(3);

	if(Condition != 0)
		uc.Return(TruePart.Evaluate());
	else 
		uc.Return(FalsePart.Evaluate());
}


static void MyIIF_String(uCalcPtr ExprPartPtr)
{
	auto uc = uCalc::Callback(ExprPartPtr);
	auto Condition = uc.Arg(1);
	auto TruePart = uc.ArgExpr(2);
	auto FalsePart = uc.ArgExpr(3);

	if(Condition != 0)
		uc.ReturnStr(TruePart.EvaluateStr());
	else 
		uc.ReturnStr(FalsePart.EvaluateStr());
}




static void MyMsgBox(uCalcPtr ExprPartPtr)
{
	auto uc = uCalc::Callback(ExprPartPtr); // Args 2 & 3 inverted because it's based on VB MsgBox syntax
	uc.Return((int)MessageBox::Show(uc.ArgStr(1), uc.ArgStr(3), (MessageBoxButtons)(int)uc.Arg(2)));
}



static void MyAverage(uCalcPtr ExprPartPtr)
{
	auto uc = uCalc::Callback(ExprPartPtr);
	double Total = 0;

	for(int x=1; x <= uc.ArgCount(); x++)
		Total = Total + uc.Arg(x);        

	uc.Return(Total / uc.ArgCount());
}


static void MyLeft(uCalcPtr ExprPartPtr)
{
	auto uc = uCalc::Callback(ExprPartPtr);
	uc.ReturnStr(uc.ArgStr(1)->Substring(0, (int)uc.Arg(2)));
}


static void StringRepeat(uCalcPtr ExprPartPtr)
{   
	auto uc = uCalc::Callback(ExprPartPtr);
	String ^ TotalString("");

	for(int x = 1; x <= uc.Arg(2); x++)
		TotalString = TotalString + uc.ArgStr(1);

	uc.ReturnStr(TotalString);
}


static void ucSum(uCalcPtr ExprPartPtr)
{
	auto uc = uCalc::Callback(ExprPartPtr);
	double Total = 0;

	auto Expression = uc.ArgExpr(1);
	auto Start = uc.Arg(2);
	auto Finish = uc.Arg(3);
	auto sStep = uc.Arg(4);
	auto VarHandle = uc.ArgObj(5);

	for (double x = Start; x <= Finish; x += sStep) 
	{
		VarHandle.SetVariableValue(x);
		Total = Total + Expression.Evaluate();
		x = x + sStep;
	}

	uc.Return(Total);   
}

#define ABS(x) (x > 0 ? x : -x)
static void ucSolve(uCalcPtr ExprPartPtr)
{
	auto uc = uCalc::Callback(ExprPartPtr);
	auto Expression = uc.ArgExpr(1);
	auto a = uc.Arg(2);
	auto b = uc.Arg(3);
	auto Variable = uc.ArgObj(4);

	int Iterations = 0;
	double fa, fb, Value = 0, tmp;

	Variable.SetVariableValue(a);
	fa = Expression.Evaluate();

	Variable.SetVariableValue(b);
	fb = Expression.Evaluate();

	if(fb < fa) {tmp = a; a = b; b = tmp;}  // swap a, b

	while(ABS(b - a) > 0.000000000000001)
	{
		Variable.SetVariableValue((a + b) / 2);

		Value = Expression.Evaluate();

		if(Value == 0) {a = (a + b) / 2; break;}

		if(Value < 0) a = (a + b) / 2; else b = (a + b) / 2;

		Iterations = Iterations + 1;
		if(Iterations == 100) break;
	}

	if(ABS(Value) > 0.0000000001) uc.ErrorRaiseMessage("Solution not found");
	uc.Return(a);
}
