using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;

/// <summary>S: Data types that come with uCalc</summary>
/// <remarks>These are built-in data types that come with uCalc</remarks>
public enum BuiltInTypesEnum : int {
   Type_Unspecified,
   Type_Auto,
   /// <summary> Boolean -- </summary>
   Type_Boolean,
   /// <summary> Int8 +++ Rename these to things like Type_Int8</summary>
   Type_Integer_8,
   /// <summary> Byte</summary>
   Type_Integer_8u,
   /// <summary> Int16</summary>
   Type_Integer_16,
   /// <summary> Int16u</summary>
   Type_Integer_16u,
   /// <summary> Int32</summary>
   Type_Integer_32,
   /// <summary> Int32u</summary>
   Type_Integer_32u,
   /// <summary> Int64</summary>
   Type_Integer_64,
   /// <summary> Int64u</summary>
   Type_Integer_64u,
   /// <summary> Int128</summary>
   Type_Integer_128,
   /// <summary> Int128u</summary>
   Type_Integer_128u,
   /// <summary> Single</summary>
   Type_Float_Single,
   /// <summary> Double</summary>
   Type_Float_Double,
   /// <summary> Extended</summary>
   Type_Float_Extended,
   Type_Size_t,
   Type_Complex,
   Type_Decimal,
   Type_BigNum,
   Type_Date,
   Type_String,
   /// <summary> LPCSTR</summary>
   Type_String_LPCSTR,
   /// <summary> FixedStr</summary>
   Type_String_Fixed,
   /// <summary> WideStr</summary>
   Type_String_Wide,
   /// <summary> StackList</summary>
   Type_List_Stack,
   /// <summary> TableList</summary>
   Type_List_Table,
   /// <summary> SortedList</summary>
   Type_List_Sorted,
   Type_Null,
   Type_Void,
   Type_Any,
   Type_Pointer,
   Type_Omni,
   Type_UserDefined,
   Type_DataType,
   Type_Namespace,
   Type_IntPtr,
   Type_ADDR,
   Type_uCalcPtr,
   Type_Item,
   Type_ParsedExpr,
   Type_uCalcCallback,
   Type_Object,
   Type_Intrinsic
};

/// <summary>S: Data type constructors</summary>
public enum DataTypeConstruction : int {
   TypeConstr_ByteSize,
   TypeConstr_AllocArray,
   TypeConstr_AllocScalar,
   TypeConstr_AllocByPtr,
   TypeConstr_ReleaseArray,
   TypeConstr_ReleaseScalar,
   TypeConstr_ResetScalar,
   TypeConstr_ResetArray,
   TypeConstr_SetArrayValue,
   TypeConstr_SetScalarValue,
   TypeConstr_SwapScalarValues
};

/// <summary>S: Error hander responses</summary>
public enum ErrorHandlerResponseEnum : int {
   /// <summary> Immediately stops parsing</summary>
   ErrorAbort,
   /// <summary> Continues parsing (after the error is resolved)</summary>
   ErrorResume,
   /// <summary> Raises the same error again for the next error handler to take care of</summary>
   ErrorReRaise
};

/// <summary>S: Error numbers that uCalc may return</summary>
public enum ErrorNumberEnum : int {
   /// <summary> No error</summary>
   Err_None,
   /// <summary> Dynamically defined user error</summary>
   Err_Dynamically_Defined,
   /// <summary> Syntax error</summary>
   Err_Syntax_Error,
   /// <summary> Undefined identifier</summary>
   Err_Undefined_Identifier,
   /// <summary> File not found</summary>
   Err_FileNotFound,
   /// <summary> Unrecognized token</summary>
   Err_Unrecognized_Token,
   /// <summary> Unrecognized command</summary>
   Err_Unrecognized_Command,
   /// <summary> Data type mismatch</summary>
   Err_Datatype_Mismatch,
   /// <summary> Invalid number of arguments</summary>
   Err_Invalid_Argument_Count,
   /// <summary> Invalid definition</summary>
   Err_Invalid_Definition,
   /// <summary> Code block delimiter error</summary>
   Err_CodeBlock_Error,
   /// <summary> Undefined callback</summary>
   Err_Undefined_Callback,
   /// <summary> Error message already defined</summary>
   Err_ErrorMessageAlreadyDefined,
   /// <summary> This item cannot be modified</summary>
   Err_ItemCannotBeModified,
   /// <summary> Unrecognized class member</summary>
   Err_Unrecognized_Class_Member,
   /// <summary> Unrecognized namespace member</summary>
   Err_Unrecognized_Namespace_Member,
   /// <summary> Unbalanced quote</summary>
   Err_Unbalanced_Quote,
   /// <summary> Value cannot be assigned here</summary>
   Err_ValueCannotBeAssigned,
   /// <summary> Array or list bound exceeded</summary>
   Err_Array_Bounds_Exceeded,
   /// <summary> Division by 0</summary>
   Err_Float_Division_By_Zero,
   /// <summary> Inexact result</summary>
   Err_Float_Inexact_Result,
   /// <summary> Invalid operation</summary>
   Err_Float_Invalid_Operation,
   /// <summary> Floating point overflow</summary>
   Err_Float_Overflow,
   /// <summary> Floating point underflow</summary>
   Err_Float_Underflow,
   Err_Customized1 = 100,
   Err_Customized2,
   Err_Customized3,
   Err_Customized4,
   Err_Customized5,
   Err_Customized6,
   Err_Customized7,
   Err_Customized8,
   Err_Customized9,
   Err_Customized10,
   Err_Customized11,
   Err_Customized12,
   Err_Customized13,
   Err_Customized14,
   Err_Customized15,
   Err_Customized16,
   Err_Customized17,
   Err_Customized18,
   Err_Customized19,
   Err_Customized20
};

/// <summary>S: Operator grouping associativity</summary>
public enum GroupingEnum : int {
   GroupLeftToRight,
   GroupRightToLeft
};

/// <summary>S: Properties that a uCalc item may have</summary>
/// <remarks>
///Items defined with uCalc typically have a combination of properties.  A variable defined
///with DefineVariable for instance will have the ItemIsEnum.Variable property set.  An operator
///will have the ItemIsEnum.Operator property set.  If it's an infix operator, in addition to
///having the ItemIsEnum.Operator property set, ItemIsEnum.Infix will also be set.
///property will also be set.
///
///The IsProperty function can determine if a given uCalc.Item has a certain property.
///
///You can use the GetItemOfProperty function to cycle through the list of all items that share
///certain properties, or to disambiguate between items that share the same name but have
///different roperties (for instance the + operator with the Prefix property vs another one
///with the Infix property).
///
///Some members are keywords in VB.NET.  To avoid naming conflict, the following members have been
///renamed in the VB.NET header: Function_, Operator_, Optional_, ByVal_, ByRef_, Class_, and NameSpace_.
///</remarks>
public enum ItemIsEnum : int {
   AnyItem,
   LeftToRight,
   RightToLeft,
   DataType,
   Array,
   Class,
   NameSpace,
   Variable,
   LocalVariable,
   Function,
   Operator,
   FunctionOrOperator,
   Prefix,
   Infix,
   Postfix,
   AddressOwner,
   ByVal,
   ByRef,
   ByCopy,
   ByExpr,
   ByHandle,
   Pointer,
   Optional,
   VariableNumberOfArgs,
   RegEx,
   LiteralItem,
   Locked,
   Transform,
   InTable,
   ItemDef,
   ErrorHandler,
   StopNow,
   ApiCall,
   CaseSensitive,
   QuotedText,
   ChangeMulti,
   ExpandArg,
   IgnoreStatementSep,
   IgnoreCodeBlock,
   SyntaxStop,
   SkipEvalExpand,
   ExprUnit,
   EagerArg,
   SkipSyntax,
   WhiteSpaceCounts,
   IgnoreQuote,
   UpToEndOrSeparator,
   UpToUserStop,
   EvalStrOutput,
   SkipOutput,
   DefaultSettings,
   NotFound,
   MiscItem
};

/// <summary>S: Licensing options for uCalc products</summary>
public enum LicenseTypeEnum : int {
   License_Demo,
   License_FMP_Free,
   License_FMP_Standard_32,
   License_FMP_Standard_64,
   License_FMP_32_and_64,
   License_LanguageBuilder,
   License_GeneralParser,
   License_Interpreter,
   License_ConsoleCalc,
   License_WindowsCalc,
   License_StringLib,
   License_Transform,
   License_TransformLib,
   License_Search,
   License_SearchLib,
   License_Library1,
   License_Library2,
   License_Beta,
   License_Debugger,
   License_uCalc_All,
   License_Developer,
   License_EndUser
};

/// <summary>S: Types of tokens</summary>
public enum TokenEnum : int {
   None,
   /// <summary> A token defined without any specified property is generic (default)</summary>
   Generic,
   /// <summary> Represents the absence of any character (token not found)</summary>
   Empty,
   /// <summary> A token that behaves as alphanuric (works with word boundaries)</summary>
   AlphaNumeric,
   /// <summary> A token that serves as argument separator (typically a comma)</summary>
   ArgSeparator,
   /// <summary> A token that starts a code block; typically characters like {, [, or (</summary>
   CodeBlock,
   /// <summary> The closing token that matches a CodeBlock token, such as }, ], or )</summary>
   CodeBlockMatch,
   /// <summary> A token that separates an object from its member, typically a . (a dot)</summary>
   MemberAccess,
   /// <summary> A token that represents a literal value, typically a quoted string</summary>
   Literal,
   /// <summary> A token that represents a statement separator, typically a ; (semicolon)</summary>
   StatementSep,
   /// <summary> A token that represents whitespace, typically tab or space characters</summary>
   WhiteSpace,
   /// <summary> A token that is not delimited by word boundaries (+, -, *, etc), consecutive characters are reduced until they match a value that exists in the symbol table or until 1 character</summary>
   Reducible,
   /// <summary> Non-character that represents the end of a stream of text</summary>
   Eof,
   CurrentThread, 
   LocalVar,
   StaticVar,
   /// <summary> Item in parenthesis (or quotes) following this token will be defined while the expression is parsed</summary>
   DefineNow = StaticVar,
   TempDef,
   /// <summary> The expression in parathesis (or quotes) following the token is evaluated and inserted in place of the token in the string</summary>
   EvalInsert,
   EvalDuringParse,
   /// <summary> Works like EvalInsert, except the expression is expanded instead of evaluated</summary>
   ExpandInsert,
   /// <summary> The contents of the quoted file name following this token is inserted into the string</summary>
   FileInclude,
   CurrentSpace,
   DefSpace,
   EvalSpace
};

/// <summary>S: Boolean True or False</summary>
public enum TrueFalseEnum : int {
   /// <summary> For compilers in which False is a keyword use False_ instead</summary>
   False,
   /// <summary> For compilers in which True is a keyword use True_ instead</summary>
   True
};

/// <summary>S: Floating point environment exceptions</summary>
public enum FPEnvExceptionEnum : int {
   /// <summary> Floating point division by zero, or operation resulting in infinity, occured (pole error)</summary>
   DivisionByZero = 8 ,
   /// <summary> The result is not an exact floating point number</summary>
   Inexact = 1 ,
   /// <summary> Argument passed to a function or operator was invalid (domain error)</summary>
   Invalid = 16 ,
   /// <summary> The result is too large to be represented as a floating point value of the return type (overflow range error)</summary>
   Overflow = 4 ,
   /// <summary> The result is too small to be represented as a floating point value of the return type, causing loss of precision (underflow range error)</summary>
   Underflow = 2 ,
   /// <summary> Represents selection of all the floating point exceptions: DivisionByZero, Inexact, Invalid, Overflow, and Underflow (Bitwise ORed)</summary>
   AllExceptions = 31 
};

/// <summary>S: Floating point rouding options</summary>
public enum FPEnvRoundingEnum : int {
   /// <summary> Rounds downward (towards negative infinity)</summary>
   Round_Down = 512 ,
   /// <summary> Rounds towards nearest value</summary>
   Round_ToNearest = 0 ,
   /// <summary> Rounds towards zero</summary>
   Round_TowardZero = 768 ,
   /// <summary> Rounds upward (towards positive infinity)</summary>
   Round_Up = 256 
};

/// <summary> uCalc main class</summary>
public class uCalc {
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Evaluate")]
   protected static extern  double Evaluate_(IntPtr ExprHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="EvaluateInt32")]
   protected static extern  Int32 EvaluateInt32_(IntPtr ExprHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="EvaluateStr")]
   protected static extern  IntPtr EvaluateStr_(IntPtr ExprHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="EvaluateVoid")]
   protected static extern  IntPtr EvaluateVoid_(IntPtr ExprHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Execute")]
   protected static extern  void Execute_(IntPtr ExprHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReleaseExpr")]
   protected static extern  void ReleaseExpr_(IntPtr ExprHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Handle_Overload_ParsedExpr")]
   protected static extern  IntPtr Handle_Overload_ParsedExpr_(IntPtr ExprHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ElementCount")]
   protected static extern  Int32 ElementCount_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="GetDataType")]
   protected static extern IntPtr GetDataType_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="FunctionAddress")]
   protected static extern  IntPtr FunctionAddress_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="IsProperty")]
   protected static extern Int32 IsProperty_(IntPtr ItemHandle,ItemIsEnum ItemProperty);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Name_Overload_Item")]
   protected static extern  IntPtr Name_Overload_Item_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="NextOverload")]
   protected static extern IntPtr NextOverload_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Precedence_Overload_Item")]
   protected static extern  Int32 Precedence_Overload_Item_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReleaseItem")]
   protected static extern  void ReleaseItem_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Rename")]
   protected static extern  void Rename_(IntPtr ItemHandle, string NewName);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReplacementText")]
   protected static extern  IntPtr ReplacementText_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValue")]
   protected static extern  void SetVariableValue_(IntPtr ItemHandle, double Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValueAny")]
   protected static extern  void SetVariableValueAny_(IntPtr ItemHandle, string Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValueByte")]
   protected static extern  void SetVariableValueByte_(IntPtr ItemHandle, Byte Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValueInt16")]
   protected static extern  void SetVariableValueInt16_(IntPtr ItemHandle, Int16 Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValueInt32")]
   protected static extern  void SetVariableValueInt32_(IntPtr ItemHandle, Int32 Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValueInt64")]
   protected static extern  void SetVariableValueInt64_(IntPtr ItemHandle, Int64 Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValueSng")]
   protected static extern  void SetVariableValueSng_(IntPtr ItemHandle, float Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValueDbl")]
   protected static extern  void SetVariableValueDbl_(IntPtr ItemHandle, double Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValueStr")]
   protected static extern  void SetVariableValueStr_(IntPtr ItemHandle, string Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValuePtr")]
   protected static extern  void SetVariableValuePtr_(IntPtr ItemHandle, IntPtr Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetVariableValueByPtr")]
   protected static extern  void SetVariableValueByPtr_(IntPtr ItemHandle, IntPtr Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Text")]
   protected static extern  IntPtr Text_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="GetuCalc_Overload_Item")]
   protected static extern IntPtr GetuCalc_Overload_Item_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Value")]
   protected static extern  double Value_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValueByte")]
   protected static extern  Byte ValueByte_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValueInt16")]
   protected static extern  Int16 ValueInt16_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValueInt32")]
   protected static extern  Int32 ValueInt32_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValueInt64")]
   protected static extern  Int64 ValueInt64_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValueSng")]
   protected static extern  float ValueSng_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValueDbl")]
   protected static extern  double ValueDbl_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValueStr")]
   protected static extern  IntPtr ValueStr_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValuePtr")]
   protected static extern  IntPtr ValuePtr_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValueAddr")]
   protected static extern  IntPtr ValueAddr_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Handle_Overload_Item")]
   protected static extern  IntPtr Handle_Overload_Item_(IntPtr ItemHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Arg")]
   protected static extern  double Arg_(IntPtr ExprPartHandle, Int32 Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ArgCount")]
   protected static extern  Int32 ArgCount_(IntPtr ExprPartHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ArgObj")]
   protected static extern IntPtr ArgObj_(IntPtr ExprPartHandle, Int32 Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ArgAddr")]
   protected static extern  IntPtr ArgAddr_(IntPtr ExprPartHandle, Int32 Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ArgExpr")]
   protected static extern IntPtr ArgExpr_(IntPtr ExprPartHandle, Int32 Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ArgInt32")]
   protected static extern  Int32 ArgInt32_(IntPtr ExprPartHandle, Int32 Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ArgInt64")]
   protected static extern  Int64 ArgInt64_(IntPtr ExprPartHandle, Int32 Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ArgPtr")]
   protected static extern  IntPtr ArgPtr_(IntPtr ExprPartHandle, Int32 Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ArgStr")]
   protected static extern  IntPtr ArgStr_(IntPtr ExprPartHandle, Int32 Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Return")]
   protected static extern  void Return_(IntPtr ExprPartHandle, double Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReturnInt32")]
   protected static extern  void ReturnInt32_(IntPtr ExprPartHandle, Int32 Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReturnInt64")]
   protected static extern  void ReturnInt64_(IntPtr ExprPartHandle, Int64 Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReturnPtr")]
   protected static extern  void ReturnPtr_(IntPtr ExprPartHandle, IntPtr Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReturnStr")]
   protected static extern  void ReturnStr_(IntPtr ExprPartHandle, string Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ErrorRaise")]
   protected static extern  Int32 ErrorRaise_(IntPtr ExprPartHandle,ErrorNumberEnum ErrNumber);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ErrorRaiseMessage")]
   protected static extern  Int32 ErrorRaiseMessage_(IntPtr ExprPartHandle, string ErrMessage);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="GetuCalc_Overload_Callback")]
   protected static extern IntPtr GetuCalc_Overload_Callback_(IntPtr ExprPartHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Handle_Overload_Callback")]
   protected static extern  IntPtr Handle_Overload_Callback_(IntPtr ExprPartHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="OpItem")]
   protected static extern IntPtr OpItem_(IntPtr ExprPartHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="AllocArray")]
   protected static extern  IntPtr AllocArray_(IntPtr DataTypeHandle, Int32 ArraySize);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="AllocByPtr")]
   protected static extern  IntPtr AllocByPtr_(IntPtr DataTypeHandle, IntPtr ValuePtr);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="AllocScalar")]
   protected static extern  IntPtr AllocScalar_(IntPtr DataTypeHandle,  string Value  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ByteSize")]
   protected static extern  Int32 ByteSize_(IntPtr DataTypeHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Index")]
   protected static extern  BuiltInTypesEnum Index_(IntPtr DataTypeHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="GetItem")]
   protected static extern IntPtr GetItem_(IntPtr DataTypeHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Name_Overload_DataType")]
   protected static extern  IntPtr Name_Overload_DataType_(IntPtr DataTypeHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReleaseArray")]
   protected static extern  void ReleaseArray_(IntPtr DataTypeHandle, IntPtr ValuePtr);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReleaseScalar")]
   protected static extern  void ReleaseScalar_(IntPtr DataTypeHandle, IntPtr ValuePtr);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ResetArray")]
   protected static extern  void ResetArray_(IntPtr DataTypeHandle, IntPtr ValuePtr);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ResetScalar")]
   protected static extern  void ResetScalar_(IntPtr DataTypeHandle, IntPtr ValuePtr);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetArray")]
   protected static extern  void SetArray_(IntPtr DataTypeHandle, IntPtr Destination ,  IntPtr Source ,  Int32 Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetScalar")]
   protected static extern  void SetScalar_(IntPtr DataTypeHandle, IntPtr Destination ,  IntPtr Source);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SwapScalarValues")]
   protected static extern  void SwapScalarValues_(IntPtr DataTypeHandle, IntPtr ValuePtrA ,  IntPtr ValuePtrB);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ToString_Overload_DataType_ByPtr")]
   protected static extern  IntPtr ToString_Overload_DataType_ByPtr_(IntPtr DataTypeHandle, IntPtr ValuePtr);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ToString_Overload_DataType_ByStr")]
   protected static extern  IntPtr ToString_Overload_DataType_ByStr_(IntPtr DataTypeHandle, string Value);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Handle_Overload_DataType")]
   protected static extern  IntPtr Handle_Overload_DataType_(IntPtr DataTypeHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="AddErrorHandler")]
   protected static extern IntPtr AddErrorHandler_(IntPtr uCalcHandle,uCalcDelegate ErrorHandlerAddress ,  Int32 Position  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="CreateuCalc")]
   protected static extern  IntPtr CreateuCalc_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="DataTypeOfIndex")]
   protected static extern IntPtr DataTypeOfIndex_(IntPtr uCalcHandle,BuiltInTypesEnum Index);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Define")]
   protected static extern IntPtr Define_(IntPtr uCalcHandle, string Expression , IntPtr Address   , IntPtr ExistingItem   );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="DefineConstant")]
   protected static extern IntPtr DefineConstant_(IntPtr uCalcHandle, string Definition , IntPtr VariableAddress   );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="DefineDataType")]
   protected static extern IntPtr DefineDataType_(IntPtr uCalcHandle, Int32 TypeSize , IntPtr AllocArrayValue  , IntPtr AllocScalar  , IntPtr AllocScalarByPtr  , IntPtr DeleteArray  , IntPtr DeleteScalar  , IntPtr ResetScalar  , IntPtr ResetArray  , IntPtr SetValueArray  , IntPtr SetValueScalar  , IntPtr SwapValues  , IntPtr ToString );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="DefineFunction")]
   protected static extern IntPtr DefineFunction_(IntPtr uCalcHandle, string Definition , uCalcDelegate FunctionAddress  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="DefineOperator")]
   protected static extern IntPtr DefineOperator_(IntPtr uCalcHandle, string Definition ,  Int32 iPrecedence , GroupingEnum Grouping  , uCalcDelegate FunctionAddress  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="DefineSyntax")]
   protected static extern IntPtr DefineSyntax_(IntPtr uCalcHandle, string Definition);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="DefineTransform")]
   protected static extern IntPtr DefineTransform_(IntPtr uCalcHandle, string FromExpr ,  string ToExpr , IntPtr ItemHandle   );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="DefineVariable")]
   protected static extern IntPtr DefineVariable_(IntPtr uCalcHandle, string Definition , IntPtr VariableAddress   );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ErrorExpression")]
   protected static extern  IntPtr ErrorExpression_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ErrorLocation")]
   protected static extern  Int32 ErrorLocation_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ErrorMessage")]
   protected static extern  IntPtr ErrorMessage_(IntPtr uCalcHandle, ErrorNumberEnum Index  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ErrorNumber")]
   protected static extern  ErrorNumberEnum ErrorNumber_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ErrorResponse")]
   protected static extern  void ErrorResponse_(IntPtr uCalcHandle,ErrorHandlerResponseEnum Response);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ErrorSymbol")]
   protected static extern  IntPtr ErrorSymbol_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ErrorSetExpression")]
   protected static extern  void ErrorSetExpression_(IntPtr uCalcHandle, string Expression);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Eval")]
   protected static extern  double Eval_(IntPtr uCalcHandle, string Expression);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="EvalStr")]
   protected static extern  IntPtr EvalStr_(IntPtr uCalcHandle, string Expression);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Expand")]
   protected static extern  IntPtr Expand_(IntPtr uCalcHandle, string Expression);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ExpandSteps")]
   protected static extern  IntPtr ExpandSteps_(IntPtr uCalcHandle, string Expression ,  string Separator  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="FPEnvClearException")]
   protected static extern  Int32 FPEnvClearException_(IntPtr uCalcHandle,  Int32 Exceptions  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="FPEnvGetExceptionMask")]
   protected static extern  Int32 FPEnvGetExceptionMask_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="FPEnvRaiseException")]
   protected static extern  Int32 FPEnvRaiseException_(IntPtr uCalcHandle, FPEnvExceptionEnum Exception  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="FPEnvGetRounding")]
   protected static extern  FPEnvRoundingEnum FPEnvGetRounding_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="FPEnvSetExceptionMask")]
   protected static extern  void FPEnvSetExceptionMask_(IntPtr uCalcHandle, Int32 Exceptions);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="FPEnvSetExceptionUnMask")]
   protected static extern  void FPEnvSetExceptionUnMask_(IntPtr uCalcHandle, Int32 Exceptions);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="FPEnvSetRounding")]
   protected static extern  Int32 FPEnvSetRounding_(IntPtr uCalcHandle,FPEnvRoundingEnum Direction);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="FPEnvToggleExceptionMask")]
   protected static extern  void FPEnvToggleExceptionMask_(IntPtr uCalcHandle,FPEnvExceptionEnum Exception);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="HandleOf")]
   protected static extern  IntPtr HandleOf_(IntPtr uCalcHandle, string ItemName,   Int64 Properties ,   bool SkipExpansion );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="GetItemOf")]
   protected static extern IntPtr GetItemOf_(IntPtr uCalcHandle, string ItemName ,  Int64 Properties  ,  bool SkipExpansion  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="GetItemOfProperty")]
   protected static extern IntPtr GetItemOfProperty_(IntPtr uCalcHandle,ItemIsEnum Property_ ,  Int32 nth ,  string DataTypeName  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="License")]
   protected static extern  Int32 License_(IntPtr uCalcHandle,LicenseTypeEnum LicenseType,   Int32 L1 ,   Int32 L2 ,   Int32 L3 ,   Int32 L4 ,   Int32 L5 );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="NamespaceChild")]
   protected static extern IntPtr NamespaceChild_(IntPtr uCalcHandle,  Int32 nth );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="NamespaceChildCount")]
   protected static extern  Int32 NamespaceChildCount_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="NamespaceParent")]
   protected static extern IntPtr NamespaceParent_(IntPtr uCalcHandle,  Int32 Level );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="NamespaceSymbol")]
   protected static extern IntPtr NamespaceSymbol_(IntPtr uCalcHandle, Int32 nth);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="NamespaceSymbolCount")]
   protected static extern  Int32 NamespaceSymbolCount_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Parse")]
   protected static extern IntPtr Parse_(IntPtr uCalcHandle, string Expression ,  string DataType  );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Precedence_Overload_uCalc")]
   protected static extern  Int32 Precedence_Overload_uCalc_(IntPtr uCalcHandle, string Operator1,   string Operator2 );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Properties")]
   protected static extern  Int64 Properties_(IntPtr uCalcHandle,ItemIsEnum p1,  ItemIsEnum p2 ,  ItemIsEnum p3 ,  ItemIsEnum p4 );
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ReleaseuCalc")]
   protected static extern  void ReleaseuCalc_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ResetOutput")]
   protected static extern  void ResetOutput_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Default")]
   protected static extern IntPtr Default_(IntPtr uCalcHandle);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetOutput_Overload_uCalc_Callback")]
   protected static extern IntPtr SetOutput_Overload_uCalc_Callback_(IntPtr uCalcHandle,uCalcDelegate FunctionAddress ,  string Properties);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="SetOutput_Overload_uCalc_NoCallback")]
   protected static extern IntPtr SetOutput_Overload_uCalc_NoCallback_(IntPtr uCalcHandle, string Definition);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="ValueAt")]
   protected static extern  IntPtr ValueAt_(IntPtr uCalcHandle, IntPtr ValuePtr,  string DataType);
   [DllImport(uCalcDLL, CharSet=CharSet.Ansi, CallingConvention=CallingConvention.Cdecl, EntryPoint="Handle_Overload_uCalc")]
   protected static extern  IntPtr Handle_Overload_uCalc_(IntPtr uCalcHandle);
   
#if WIN64 
   public const string uCalcDLL = "uCalc64.dll";
#else
   public const string uCalcDLL ="uCalc32.dll";
#endif
   public delegate void uCalcDelegate(IntPtr Expr); 
   /// <summary> Handle for uCalc instance</summary>
   public IntPtr uCalcHandle = default(IntPtr);
   
   /// <summary> Class for evaluating an already parsed expression or executing parsed code</summary>
   public class ParsedExpr {
      public IntPtr ExprHandle;
      
      /// <summary> Rapidly evaluates a floating point expression that was parsed with Parse or passed By Expression</summary>
      /// <returns> The result of the evaluated floating point expression</returns>
      /// <remarks><para>Evaluate can be used instead of Eval for faster results in time-critical loops where the same expression
      /// is repeatedly re-evaluated, with the only change in each iteration typically being the value(s) of one or
      /// more variables.  Evaluate works in pair with Parse, the relatively slower part, which is called first before
      /// going into the time-critical loop.  The ParsedExpr object returned by Parse is what calls the Evaluate function.
      /// </para><para>
      /// Evaluate can also be called by a ParsedExpr object that was passed to a callback By Expression.
      /// </para><para>
      /// The expression must be one who&#39;s evaluation returns a floating point numeric value.  If the expression evaluates
      /// to a non-numeric value (such as a string) or non-scalar numeric value (such as a complex number), or if a numeric
      /// type other than the default was explicitely specified with Parse, the value returned will be invalid.  However,
      /// an error will not be raised.
      /// </para></remarks>
      public  double Evaluate() { return Evaluate_(ExprHandle);       }
      
      /// <summary> Rapidly evaluates expression that was parsed as integer with Parse or passed By Expression</summary>
      /// <returns> The result of the evaluated integer expression</returns>
      /// <remarks><para>
      /// This is similar to Evaluate, but applies only to integer expressions.
      /// </para></remarks>
      public  Int32 EvaluateInt32() { return EvaluateInt32_(ExprHandle);       }
      
      /// <summary> Rapidly evaluates an expression of any data type that was parsed with Parse or passed By Expression returning a string</summary>
      /// <returns> The result of an evaluated expression of any data type, returned as a string</returns>
      /// <remarks><para>EvaluateStr is similar to Evaluate.  However, EvaluateStr returns a value as a string.  Unlike Evaluate, which can only
      /// return a floating point value, EvaluateStr can return a value of any data type.  Non-string values are converted to a string
      /// before being returned.
      /// </para></remarks>
      public  string EvaluateStr() { return Marshal.PtrToStringAnsi(EvaluateStr_(ExprHandle));       } 
      
      /// <summary> Rapidly evaluates an expression that was parsed with Parse or passed By Expression and returns a IntPtr to the result</summary>
      /// <returns> Returns a IntPtr to the result instead of the result itself</returns>
      /// <remarks><para>Use this when you are interested in a IntPtr to the result instead of the result itself.  For instance, given an integer
      /// result, you might use a IntPtr in order to retrieve the hi byte and lo byte values.
      /// </para></remarks>
      public  IntPtr EvaluateVoid() { return EvaluateVoid_(ExprHandle);       }
      
      /// <summary> Rapidly executes code that was parsed with Parse or passed By Expression for which a result is not required</summary>
      /// <remarks><para>This can be used in cases where the reutrn value is not needed, such as statements that represent code to be executed
      /// rather than a math expression.
      /// </para></remarks>
      public  void Execute() {  Execute_(ExprHandle);       }
      
      /// <summary> (Use Release instead)</summary>
      /// <remarks><para>Use Release</para></remarks>
      public  void ReleaseExpr() {  ReleaseExpr_(ExprHandle);       }
      
      /// <summary> Returns the handle of the parsed expression</summary>
      /// <remarks><para>This numeric value represents the IntPtr that uCalc uses internally for the ParsedExpr object.
      /// If you are iterating through a series of objects, such as with NextOverload, you may check Handle()
      /// to see when it becomes NULL (or 0) to know when to stop.</para></remarks>
      public IntPtr  Handle() { return Handle_Overload_ParsedExpr_(ExprHandle);       }
      
      /// <summary> Releases an expression that was parsed with Parse</summary>
      /// <remarks><para>Whenever an expression is parsed with Parse, the parsed expression remains in memory until released.
      /// When the parsed expression is no longer needed it should be released with Release to preserve memory.
      /// Note: An expression parsed with Parse must explicitely be released with Release unless the uCalc
      /// object it is in is released.  A parsed expression will not be released automatically when it simply
      /// goes out of scope.
      /// </para></remarks>
      public void Release()  { ReleaseExpr(); ExprHandle = default(IntPtr);       }
      
      public ParsedExpr() { ExprHandle = default(IntPtr);       }
      public ParsedExpr( IntPtr ExpressionHandle) { ExprHandle = ExpressionHandle;       }
      public ParsedExpr( ParsedExpr ParsedExprInstance) { ExprHandle = ParsedExprInstance.ExprHandle;       }
   }
   
   /// <summary> Class for handling uCalc items</summary>
   public class Item {
      
      /// <summary> Handle for a uCalc.Item</summary>
      public IntPtr ItemHandle = default(IntPtr);
      
      /// <summary> Returns the number of elements in a uCalc.Item</summary>
      /// <remarks><para>Depending on the kind of object, this may represent the number of elements in an array, the number of parameters in a function,
      /// the number of operands in an operator, the number of parameters in a syntax construct, the number of parts in a transform,
      /// the number of parts in a compound token, or the value explicitlely set in the Count property when calling Define.
      /// </para><para>
      /// When determining the number of elements, optional parameters in a function count as an element.  If the function
      /// has a variable number of arguments, then count is -1.
      /// </para><para>
      /// The value returned for a syntax construct may not seem obvious, especially if it has optional and/or alternative
      /// parts or some other complexity.
      /// </para><para>
      /// +++ the number of symbols in a uCalc
      /// instance,</para></remarks>
      public  Int32 ElementCount() { return ElementCount_(ItemHandle);       }
      
      /// <summary> Returns the data type of a uCalc.Item</summary>
      /// <remarks><para>The returned data type object can be used to determine the type name, size, etc.</para></remarks>
      public DataType GetDataType() { return new DataType(GetDataType_(ItemHandle));       }
      
      /// <summary> Returns the callback address of a defined function (or operator)</summary>
      /// <remarks><para>The returned value is the actual IntPtr address of the function as used by uCalc internally, and not
      /// a delegate, even in .NET.  As such, in .NET this value can be used in the more general Define function but
      /// not functions like DefineFunction or DefineneOperator.</para></remarks>
      public  IntPtr FunctionAddress() { return FunctionAddress_(ItemHandle);       }
      
      /// <summary> Determines if a uCalc.Item has the given properties</summary>
      /// <param name="ItemProperty"> The property you want to check for</param>
      /// <returns> Returns True if the uCalc.Item matches the given property, or False otherwise.</returns>
      /// <remarks><para>This helps you determine whether the uCalc.Item is a function, operator, variable, data type,
      /// literal value, etc, or has certain other special properties.  Note: Some items may have multiple
      /// properties.  For instance the Function property may be set to true for arrays.
      /// </para></remarks>
      public  bool IsProperty(ItemIsEnum ItemProperty) { return IsProperty_(ItemHandle,ItemProperty) != 0;       }
      
      /// <summary> Returns the name of the uCalc.Item</summary>
      /// <remarks><para>Items such as functions, operators, variables, etc generally are given a name when they are defined.
      /// That name can be retried with this function.</para></remarks>
      public string  Name() { return Marshal.PtrToStringAnsi(Name_Overload_Item_(ItemHandle));       } 
      
      /// <summary> Returns next uCalc.Item object that shares the same name</summary>
      /// <returns> Returns next uCalc.Item object that shares the same name, or a NULL (or 0) if there is none</returns>
      /// <remarks><para>If several functions, operators, parameters, etc are overloaded -- that is they are defined with different
      /// parameters but share the same name --, or if they (or variables) overshadow each other, this function returns
      /// the next uCalc.Item object that shares the same name.</para></remarks>
      public Item NextOverload() { return new Item(NextOverload_(ItemHandle));       }
      
      /// <summary> Returns the precedence level of the uCalc.Item</summary>
      /// <remarks><para>Operators, and some syntax constructs are defined with a precedence level.  When defining a new operator you
      /// may sometimes want to give it the same level as a pre-existing operator.</para></remarks>
      public Int32  Precedence() { return Precedence_Overload_Item_(ItemHandle);       }
      
      /// <summary> (Use Release instead)</summary>
      /// <remarks><para>Use Release</para></remarks>
      public  void ReleaseItem() {  ReleaseItem_(ItemHandle);       }
      
      /// <summary> Renames an item</summary>
      /// <param name="NewName"> The new name of the item</param>
      /// <remarks><para>
      /// Any defined item that has a name (such as a variable, constant, function, operator, etc) can be renamed with this function.
      /// </para></remarks>
      public  void Rename( string NewName) {  Rename_(ItemHandle,NewName);       }
      
      /// <summary> Returns the replacement part of a syntax construct defined with DefineSyntax</summary>
      /// <returns> Returns the part to the right of &quot;::=&quot; in a syntax construct defined with DefineSyntax</returns>
      /// <remarks><para>This allows you to inspect the part to the right of &quot;::=&quot; in a syntax construct defined with DefineSyntax.</para></remarks>
      public  string ReplacementText() { return Marshal.PtrToStringAnsi(ReplacementText_(ItemHandle));       } 
      
      /// <summary> Sets the value of a floating point variable +++ maybe SetValue instead of SetVariableValue</summary>
      /// <param name="Value"> Double precision floating point value that the variable will be set to</param>
      /// <remarks><para>When a floating point variable is defined with DefineVariable, you can later set a value for this variable using
      /// SetVariableValue.  Note: If your compiler supports pointers, you can define a uCalc variable that is directly attached
      /// to the address of a variable in your source code, and not have to use SetVariableValue.</para></remarks>
      public  void SetVariableValue( double Value) {  SetVariableValue_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a variable of any data type by passing a string</summary>
      /// <param name="Value"> String containing the value that will be converted and stored in the variable</param>
      /// <remarks><para>This lets you set the value of a variable defined with DefineVariable (or Define) regardless of the variable&#39;s data type.
      /// The value is passed as a string, and internally converted to the appropriate data type.  Note: Because a conversion takes
      /// place with SetVariableValueAny, it is typically slower than calling SetVariableValue for floating point, or one of the
      /// other versions of SetVariableValue specific to a data type.</para></remarks>
      public  void SetVariableValueAny( string Value) {  SetVariableValueAny_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a Byte variable</summary>
      /// <param name="Value"> Byte value that the variable will be set to</param>
      /// <remarks><para>This lets you set the value of a Byte variable defined with DefineVariable (or Define)</para></remarks>
      public  void SetVariableValueByte( Byte Value) {  SetVariableValueByte_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a 16-bit integer variable</summary>
      /// <param name="Value"> 16-bit integer value that the variable will be set to</param>
      /// <remarks><para>This lets you set the value of a 16-bit integer variable defined with DefineVariable (or Define).  This works for
      /// both signed and unsigned 16-bit integers.</para></remarks>
      public  void SetVariableValueInt16( Int16 Value) {  SetVariableValueInt16_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a 32-bit integer variable</summary>
      /// <param name="Value"> 32-bit integer value that the variable will be set to</param>
      /// <remarks><para>This lets you set the value of a 32-bit integer variable defined with DefineVariable (or Define).  This works for
      /// both signed and unsigned 32-bit integers.</para></remarks>
      public  void SetVariableValueInt32( Int32 Value) {  SetVariableValueInt32_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a 64-bit integer variable</summary>
      /// <param name="Value"> 64-bit integer value that the variable will be set to</param>
      /// <remarks><para>This lets you set the value of a 64-bit integer variable defined with DefineVariable (or Define).  This works for
      /// both signed and unsigned 64-bit integers.</para></remarks>
      public  void SetVariableValueInt64( Int64 Value) {  SetVariableValueInt64_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a single precision floating point variable</summary>
      /// <param name="Value"> Single precision value that the variable will be set to</param>
      /// <remarks><para>This lets you set the value of a single precision variable defined with DefineVariable (or Define).</para></remarks>
      public  void SetVariableValueSng( float Value) {  SetVariableValueSng_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a double precision floating point variable</summary>
      /// <param name="Value"> Double precision floating point value that the variable will be set to</param>
      /// <remarks><para>SetVariableValueDbl is synonymous with SetVariableValue</para></remarks>
      public  void SetVariableValueDbl( double Value) {  SetVariableValueDbl_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a string variable</summary>
      /// <param name="Value"> String value that the variable will be set to</param>
      /// <remarks><para>This lets you set the value of a string variable defined with DefineVariable (or Define).</para></remarks>
      public  void SetVariableValueStr( string Value) {  SetVariableValueStr_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a IntPtr variable</summary>
      /// <param name="Value"> IntPtr that the variable will be set to</param>
      /// <remarks><para>This lets you set the value of a IntPtr variable defined with DefineVariable (or Define).  Note: This does not
      /// change the IntPtr of the variable.  It simply stores a IntPtr value in the variable.</para></remarks>
      public  void SetVariableValuePtr( IntPtr Value) {  SetVariableValuePtr_(ItemHandle,Value);       }
      
      /// <summary> Sets the value of a variable to a value who&#39;s IntPtr is passed</summary>
      /// <param name="Value"> IntPtr to the value that the variable will be set to</param>
      /// <remarks><para>This lets you set the value of a variable by passing the IntPtr to the value you want to set the variable to.</para></remarks>
      public  void SetVariableValueByPtr( IntPtr Value) {  SetVariableValueByPtr_(ItemHandle,Value);       }
      
      /// <summary> Returns the text data associated with a uCalc.Item</summary>
      /// <remarks><para>Certain items, such as a syntax construct defined with DefineSyntax have a text property that can be retrieved
      /// with a call to this function.</para></remarks>
      public  string Text() { return Marshal.PtrToStringAnsi(Text_(ItemHandle));       } 
      
      /// <summary> Returns the uCalc object that the uCalc.Item belongs to</summary>
      /// <remarks><para>A uCalc.Item is defined in the context of a uCalc object.  You can later use this function to determine
      /// the uCalc object the defined item belongs to.</para></remarks>
      public uCalc  GetuCalc() { return new uCalc(GetuCalc_Overload_Item_(ItemHandle));       }
      
      /// <summary> Returns the value (default double precision) associated with a uCalc.Item</summary>
      /// <remarks><para>Use this to retrieve the Double precision value associated with a variable.</para></remarks>
      public  double Value() { return Value_(ItemHandle);       }
      
      /// <summary> Returns the value (default double precision) associated with a uCalc.Item</summary>
      /// <remarks><para>Use this to retrieve the Byte value associated with a variable.</para></remarks>
      public  Byte ValueByte() { return ValueByte_(ItemHandle);       }
      
      /// <summary> Returns the 16-bit integer value associated with a uCalc.Item</summary>
      /// <remarks><para>Use this to retrieve the Int16 value associated with a variable.</para></remarks>
      public  Int16 ValueInt16() { return ValueInt16_(ItemHandle);       }
      
      /// <summary> Returns the 32-bit integer value associated with a uCalc.Item</summary>
      /// <remarks><para>Use this to retrieve the Int32 value associated with a variable.</para></remarks>
      public  Int32 ValueInt32() { return ValueInt32_(ItemHandle);       }
      
      /// <summary> Returns the 64-bit integer value associated with a uCalc.Item</summary>
      /// <remarks><para>Use this to retrieve the Int64 value associated with a variable.</para></remarks>
      public  Int64 ValueInt64() { return ValueInt64_(ItemHandle);       }
      
      /// <summary> Returns the Double precision value associated with a uCalc.Item</summary>
      /// <remarks><para>Use this to retrieve the Single precision value associated with a variable.</para></remarks>
      public  float ValueSng() { return ValueSng_(ItemHandle);       }
      
      /// <summary> Returns the Double precision value associated with a uCalc.Item</summary>
      /// <remarks><para>Synonymous with Value</para></remarks>
      public  double ValueDbl() { return ValueDbl_(ItemHandle);       }
      
      /// <summary> Returns the String value associated with a uCalc.Item</summary>
      /// <remarks><para>Use this to retrieve the String value associated with a variable.
      /// </para><para>
      /// ValueStr has a special property beyond the Value function for other data types in that it can return the value
      /// of an item regardless of what data type it has.  The value is returned as a string.
      /// </para></remarks>
      public  string ValueStr() { return Marshal.PtrToStringAnsi(ValueStr_(ItemHandle));       } 
      
      /// <summary> Returns the IntPtr value associated with a uCalc.Item</summary>
      /// <remarks><para>Use this to retrieve the IntPtr value associated with a variable.
      /// Note: If the uCalc.Item is a Variable of type IntPtr, then ValuePtr returns a IntPtr value
      /// that is stored in that variable, whereas ValueAddr returns the IntPtr of the variable itself.  In other words
      /// ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any value you may
      /// have set as the address the variable points to.</para></remarks>
      public  IntPtr ValuePtr() { return ValuePtr_(ItemHandle);       }
      
      /// <summary> Returns the IntPtr address of the value associated with a uCalc.Item</summary>
      /// <remarks><para>Use this to retrieve the IntPtr address of the value associated with a variable (instead of
      /// the value itself).  Note: If the uCalc.Item is a Variable of type IntPtr, then ValuePtr returns a IntPtr value
      /// that is stored in that variable, whereas ValueAddr returns the IntPtr of the variable itself.  In other words
      /// ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any given address
      /// you may have stored as the value of the variable.</para></remarks>
      public  IntPtr ValueAddr() { return ValueAddr_(ItemHandle);       }
      
      /// <summary> Returns the handle of the current uCalc.Item object</summary>
      public IntPtr  Handle() { return Handle_Overload_Item_(ItemHandle);       }
      
      /// <summary> Releases an Item defined with Define or other related function</summary>
      /// <remarks><para>When a function, operator, variable, or other item is defined with such functions as
      /// Define, DefineFunction, DefineOperator, DefineVariable, these items remain in memory, until they are
      /// released with this function.  Items should generally be released when they are no longer needed, in order
      /// to preserve memory.
      /// </para><para>
      /// Note: A uCalc Item object must explicitely be released with Release unless the entire uCalc
      /// object it is a part of is is released.  An Item object will not be released automatically when it
      /// simply goes out of scope.
      /// </para></remarks>
      public void Release()  { ReleaseItem(); ItemHandle = default(IntPtr);       }
      
      public Item() { ItemHandle = default(IntPtr);       }
      public Item( IntPtr PassedItemHandle) { ItemHandle = PassedItemHandle;       }
      public Item( Item ItemHandleInstance) { ItemHandle = ItemHandleInstance.ItemHandle;       }
   }
   
   /// <summary> Class for retrieving arguments passed to a callback, returning a value, and more</summary>
   public class Callback {
      
      /// <summary> Handle of the expression part the current callback instance is refering to</summary>
      public IntPtr ExprPartHandle = default(IntPtr);
      
      /// <summary> Returns the value of an argument passed to a uCalc.Callback function</summary>
      /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
      /// <remarks><para>When the name of a callback function is specified in DefineFunction or DefineOperator, that
      /// callback function will receive arguments uCalc passes to it through the Arg.  Arg handles values
      /// of the default Double floating point data type.  For other data types, ArgInt32, ArgStr, etc instead.
      /// </para><para>
      /// Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be
      /// 1 (not 0), and the second and third will be 2 and 3 respectively.
      /// </para><para>
      /// If you are defining a binary operator, 1 is the index for the left operand, and 2 for the right operand.
      /// For a unary operator, the index is 1.
      /// </para></remarks>
      public  double Arg( Int32 Index) { return Arg_(ExprPartHandle,Index);       }
      
      /// <summary> Returns the number of arguments being passed to a uCalc.Callback function</summary>
      /// <returns> 0 if no arg is passed; or a positive integer representing the number of args being passed.</returns>
      /// <remarks><para>This is useful for functions that have a variable number of arguments</para></remarks>
      public  Int32 ArgCount() { return ArgCount_(ExprPartHandle);       }
      
      /// <summary> Returns an object instance of the argument being passed to a uCalc.Callback function</summary>
      /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
      /// <returns> The returned object can be used (where relevent) just like any other uCalc.Item</returns>
      /// <remarks><para>Each argument is a uCalc.Item, just like anything else defined with Define.  With this object,
      /// you can do such things as determine the argument&#39;s data type, determine which instance of uCalc
      /// it belongs to, retrieve the IntPtr of the value of the argument, etc.</para></remarks>
      public Item ArgObj( Int32 Index) { return new Item(ArgObj_(ExprPartHandle,Index));       }
      
      /// <summary> Returns the address of the argument value that is being passed</summary>
      /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
      /// <remarks><para>In addition to Arg which returns arguments that are of the default Double floating point numeric type, there
      /// are specialized functions for other datatypes such as ArgInt32, ArgInt64, etc.  However, if the argument is of
      /// a data type that does not have a specialized function, you can retrieve the address of the argument value as
      /// a IntPtr, and use that to retrieve the value.</para></remarks>
      public  IntPtr ArgAddr( Int32 Index) { return ArgAddr_(ExprPartHandle,Index);       }
      
      /// <summary> Returns the object instance of the expression that&#39;s passed as an argument to a uCalc.Callback function+++Example</summary>
      /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
      /// <returns> Returns parsed expression object with which you can call functions like Evaluate or Execute</returns>
      /// <remarks><para>uCalc allows you to not only pass arguments by value or by reference.  But it also lets you pass an argument
      /// by expression.  When an expression is passed By Expression, the expresison is not evaluated before being passed.
      /// Instead a handle to the parsed expression is passed, allowing the callback to perform the evaluation.</para></remarks>
      public ParsedExpr ArgExpr( Int32 Index) { return new ParsedExpr(ArgExpr_(ExprPartHandle,Index));       }
      
      /// <summary> Returns the Int32 integer argument passed to a uCalc.Callback function</summary>
      /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
      /// <remarks><para>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can
      /// recieve it through ArgInt32.  If the argument is of a different but compatible numeric type, a conversion takes
      /// place first before being passed.</para></remarks>
      public  Int32 ArgInt32( Int32 Index) { return ArgInt32_(ExprPartHandle,Index);       }
      
      /// <summary> Returns the Int64 integer argument being passed to a uCalc.Callback function</summary>
      /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
      /// <remarks><para>If a function parameter (or operator operand) was explicitely defined as Int64, then the callback function can
      /// recieve it through ArgInt64.  If the argument is of a different but compatible numeric type, a conversion takes
      /// place first before being passed.</para></remarks>
      public  Int64 ArgInt64( Int32 Index) { return ArgInt64_(ExprPartHandle,Index);       }
      
      /// <summary> Returns a IntPtr value being passed as an argument to a uCalc.Callback function</summary>
      /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
      /// <remarks><para>If a function parameter (or operator operand) was explicitely defined as a IntPtr, then the callback function can
      /// recieve it through ArgPtr.  Note that ArgPtr returns any address value that is being sent, whereas ArgAddr returns the
      /// address of the value being passed.</para></remarks>
      public  IntPtr ArgPtr( Int32 Index) { return ArgPtr_(ExprPartHandle,Index);       }
      
      /// <summary> Returns the value being passed as argument in to a uCalc.Callback function in form of a string</summary>
      /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
      /// <remarks><para>In addition to returning an argument that was passed as a string, ArgStr has the special property of returning an argument that
      /// was passed as any data type.  If the argument was not a string, ArgStr converts it to a string before returning its value.</para></remarks>
      public  string ArgStr( Int32 Index) { return Marshal.PtrToStringAnsi(ArgStr_(ExprPartHandle,Index));       } 
      
      /// <summary> Sets the return value (default floating point) of a uCalc.Callback function</summary>
      /// <param name="Value"> Floating point value that your callback returns</param>
      /// <remarks><para>When you define a callback function with DefineFunction and you want to return a floating point number,
      /// do it with Return.
      /// </para><para>
      /// Note: For compilers in which Return is a reserved keyword, use ReturnDbl instead.
      /// </para></remarks>
      public  void Return( double Value) {  Return_(ExprPartHandle,Value);       }
      
      /// <summary> Sets the return value of a uCalc.Callback function as an Int32 integer</summary>
      /// <param name="Value"> Integer (32-bit) that your callback returns</param>
      /// <remarks><para>When you define a callback function with DefineFunction and you want to return a Int32,
      /// do it with ReturnInt32.</para></remarks>
      public  void ReturnInt32( Int32 Value) {  ReturnInt32_(ExprPartHandle,Value);       }
      
      /// <summary> Sets the return value of a uCalc.Callback function as an Int64 integer</summary>
      /// <param name="Value"> Integer (64-bit) that your callback returns</param>
      /// <remarks><para>When you define a callback function with DefineFunction and you want to return a Int64,
      /// do it with ReturnInt64.</para></remarks>
      public  void ReturnInt64( Int64 Value) {  ReturnInt64_(ExprPartHandle,Value);       }
      
      /// <summary> Sets the return value of a uCalc.Callback function as a IntPtr</summary>
      /// <param name="Value"> IntPtr value that your callback returns</param>
      /// <remarks><para>When you define a callback function with DefineFunction that returns a IntPtr value,
      /// do it with ReturnPtr.</para></remarks>
      public  void ReturnPtr( IntPtr Value) {  ReturnPtr_(ExprPartHandle,Value);       }
      
      /// <summary> Sets the return value of a uCalc.Callback function as a String</summary>
      /// <param name="Value"> String value that your callback returns</param>
      /// <remarks><para>When you define a callback function with DefineFunction that returns a string,
      /// do it with ReturnStr.</para></remarks>
      public  void ReturnStr( string Value) {  ReturnStr_(ExprPartHandle,Value);       }
      
      /// <summary> Raises an error in a uCalc.Callback function</summary>
      /// <param name="ErrNumber"> Index number of error to raise</param>
      /// <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
      /// <remarks><para>Your callback can raise an error by calling this funciton with one of the values of the
      /// ErrorNumberEnum enumerator.  End-user function definitions can also raise an error by calling Error().
      /// The return value is determined by error handlers that you may have added.  ErrorAbort (value of 0)
      /// is returned by default if there are no error handlers.</para></remarks>
      public  Int32 ErrorRaise(ErrorNumberEnum ErrNumber) { return ErrorRaise_(ExprPartHandle,ErrNumber);       }
      
      /// <summary> Raises an error with a user-defined message in a uCalc.Callback function</summary>
      /// <param name="ErrMessage"> Text of custom error message to return</param>
      /// <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
      /// <remarks><para>Your callback can raise a custom error message (one not listed as a ErrorNumberEnum enumerator)
      /// by simply passing the text of the message to this function.</para></remarks>
      public  Int32 ErrorRaiseMessage( string ErrMessage) { return ErrorRaiseMessage_(ExprPartHandle,ErrMessage);       }
      
      /// <summary> Returns the uCalc instance the current expression part belongs to</summary>
      /// <remarks><para>A callback is designed to work with all instances of uCalc.  However, at times you may need to know
      /// which particular instance of uCalc the expression part being passed belongs to.</para></remarks>
      public uCalc  GetuCalc() { return new uCalc(GetuCalc_Overload_Callback_(ExprPartHandle));       }
      
      /// <summary> Returns the handle of the sub expression passed to the callback</summary>
      /// <remarks><para>This IntPtr value is the argument received by a callback function associated with a function or operator
      /// defined with DefineFunction or DefineOperator.  Once a uCalc.Callback object is construct with this handle, this
      /// handle typically no longer needs to be refered to directly.  However, it&#39;s still avaible should you need it.
      /// uCalc interfaces with this handle underneath the hood of the uCalc.Callback object.
      /// </para></remarks>
      public IntPtr  Handle() { return Handle_Overload_Callback_(ExprPartHandle);       }
      
      /// <summary> Returns the function or operator Item of the current expression part</summary>
      /// <remarks><para>If several functions and/or operators are mapped to the same callback function, you can use this to
      /// let you determine which one is currently being called, based on properties you can check.</para></remarks>
      public Item OpItem() { return new Item(OpItem_(ExprPartHandle));       }
      
      public Callback() { ExprPartHandle = default(IntPtr);       }
      public Callback( IntPtr ExpressionHandle) { ExprPartHandle = ExpressionHandle;       }
      public Callback( Callback ExprInstance) { ExprPartHandle = ExprInstance.ExprPartHandle;       }
   }
   
   /// <summary> Class for working with uCalc data types</summary>
   public class DataType {
      
      /// <summary> Handle for uCalc Data Type</summary>
      public IntPtr DataTypeHandle = default(IntPtr);
      
      /// <summary> Allocates an array of the given data type</summary>
      /// <param name="ArraySize"> Size - in umber of elements - of the array you&#39;re allocating</param>
      /// <returns> IntPtr of the array that was allocated</returns>
      /// <remarks><para>This allocates an array of the given type.  Elements of the array are set to the default value
      /// of that type.</para></remarks>
      public  IntPtr AllocArray( Int32 ArraySize) { return AllocArray_(DataTypeHandle,ArraySize);       }
      
      /// <summary> Allocates a scalar of the given type and copies data from the area indicated by the IntPtr</summary>
      /// <param name="ValuePtr"> IntPtr to the data that is copied to the scalar being defined</param>
      /// <returns> IntPtr of the scalar that was allocated</returns>
      public  IntPtr AllocByPtr( IntPtr ValuePtr) { return AllocByPtr_(DataTypeHandle,ValuePtr);       }
      
      /// <summary> Allocates a scalar of the given data type</summary>
      /// <param name="Value"> Expression who&#39;s evaluated value will be copied to the allocated scalar</param>
      /// <returns> IntPtr of the scalar that was allocated</returns>
      public  IntPtr AllocScalar( string Value = "") { return AllocScalar_(DataTypeHandle,Value);       }
      
      /// <summary> Returns the size (in bytes) of one unit of data for this data type</summary>
      public  Int32 ByteSize() { return ByteSize_(DataTypeHandle);       }
      
      /// <summary> Returns the index of the data type</summary>
      /// <remarks><para>The return value is a BuiltInTypesEnum enumerator value that represents the data type of the
      /// current object.  For instance, if the data type is a string, the return value would be Type_String.
      /// </para></remarks>
      public  BuiltInTypesEnum Index() { return Index_(DataTypeHandle);       }
      
      /// <summary> Returns the uCalc.Item handle of the data type</summary>
      /// <remarks><para>Each data type object can also be represented as a uCalc object by using this return value.</para></remarks>
      public Item GetItem() { return new Item(GetItem_(DataTypeHandle));       }
      
      /// <summary> Returns the name of the data type</summary>
      public string  Name() { return Marshal.PtrToStringAnsi(Name_Overload_DataType_(DataTypeHandle));       } 
      
      /// <summary> Releases an array that was allocated with AllocArray</summary>
      /// <param name="ValuePtr"> IntPtr of uCalc array to be released from memory</param>
      /// <remarks><para>This should only be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
      public  void ReleaseArray( IntPtr ValuePtr) {  ReleaseArray_(DataTypeHandle,ValuePtr);       }
      
      /// <summary> Releases a scalar that was allocated with AllocScalar</summary>
      /// <param name="ValuePtr"> IntPtr of uCalc scalar to be released from memory</param>
      /// <remarks><para>This should only be used for scalars that were explicitly allocated with uCalc&#39;s AllocScalar.</para></remarks>
      public  void ReleaseScalar( IntPtr ValuePtr) {  ReleaseScalar_(DataTypeHandle,ValuePtr);       }
      
      /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
      /// <param name="ValuePtr"> IntPtr of the uCalc array to be reset</param>
      /// <remarks><para>Resets all the elements of the array to the default value for the given data type.  This should only
      /// be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
      public  void ResetArray( IntPtr ValuePtr) {  ResetArray_(DataTypeHandle,ValuePtr);       }
      
      /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
      /// <param name="ValuePtr"> IntPtr of the uCalc scalar to be reset</param>
      public  void ResetScalar( IntPtr ValuePtr) {  ResetScalar_(DataTypeHandle,ValuePtr);       }
      
      /// <summary> Sets the value of an array at a given memory location that was allocated by AllocArray</summary>
      /// <param name="Destination"> IntPtr of array (base address) who&#39;s element value is to be set</param>
      /// <param name="Source"> IntPtr to a value of the same data type that will be copied to the element</param>
      /// <param name="Index"> Index of the array element who&#39;s value needs to be set</param>
      public  void SetArray( IntPtr Destination ,  IntPtr Source ,  Int32 Index) {  SetArray_(DataTypeHandle,Destination , Source , Index);       }
      
      /// <summary> Sets the value of a scalar at a given memory location that was allocated by AllocScalar</summary>
      /// <param name="Destination"> IntPtr of scalar who&#39;se value is to be set</param>
      /// <param name="Source"> IntPtr to a value of the same data type that will be copied to the element</param>
      public  void SetScalar( IntPtr Destination ,  IntPtr Source) {  SetScalar_(DataTypeHandle,Destination , Source);       }
      
      /// <summary> swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</summary>
      /// <param name="ValuePtrA"> IntPtr of variable 1</param>
      /// <param name="ValuePtrB"> IntPtr of variable 2</param>
      public  void SwapScalarValues( IntPtr ValuePtrA ,  IntPtr ValuePtrB) {  SwapScalarValues_(DataTypeHandle,ValuePtrA , ValuePtrB);       }
      
      /// <summary> Retuns a string representing the value at the given address formatted for the given data type</summary>
      /// <param name="ValuePtr"> IntPtr of the data who&#39;s value you want to display</param>
      /// <remarks><para>If you have a IntPtr to data of the given type, you can display this data as a string with this function.  Special formatting
      /// associated with a data type is used, if such formatting was defined.  For instance, if you&#39;re dealing with a bool, then
      /// you might get: True, Yes, On, etc, depending on how the formatting was configured with SetOutput.</para></remarks>
      public string  ToString( IntPtr ValuePtr) { return Marshal.PtrToStringAnsi(ToString_Overload_DataType_ByPtr_(DataTypeHandle,ValuePtr));       } 
      
      /// <summary> Retuns a formatted string of the given data type representing the value that was passed as text</summary>
      /// <param name="Value"> String representing data of any type that is to be represented as data of the given type</param>
      /// <remarks><para>If you pass a string, it will be converted to the given data type, and return using the formatting
      /// for that data type.</para></remarks>
      public string  ToString( string Value) { return Marshal.PtrToStringAnsi(ToString_Overload_DataType_ByStr_(DataTypeHandle,Value));       } 
      
      /// <summary> Returns the handle of the data type object</summary>
      /// <remarks><para>Generally for internal use</para></remarks>
      public IntPtr  Handle() { return Handle_Overload_DataType_(DataTypeHandle);       }
      public DataType() { DataTypeHandle = default(IntPtr);       }
      public DataType( IntPtr PassedDataTypeHandle) { DataTypeHandle = PassedDataTypeHandle;       }
      public DataType( DataType DataTypeHandleInstance) { DataTypeHandle = DataTypeHandleInstance.DataTypeHandle;       }
   }
   
   /// <summary> Adds an error handler</summary>
   /// <param name="ErrorHandlerAddress"> Address of the callback function containing the error handler code</param>
   /// <param name="Position"> Position in which to insert the error handler</param>
   /// <returns> The error handler can later be removed using this return object</returns>
   /// <remarks><para>Use this to set a a callback function that is called whenever there is an error.  Your callback can
   /// do things like alert the user of the error, correct the error and resume, keep a log of errors, etc.
   /// </para><para>
   /// By default, if there are multiple error handlers, they are called starting with the most recently
   /// defined one first, ending with the first one to be defined last.  In order to define a handler in a different
   /// order, specify the Position value of where it should be inserted; 0 so that it is called first, -1 so it
   /// is called last, or a positive integer indicated where it should be inserted.  If the Postion value is out
   /// of bounds, it is adjusted into the maximum acceptable value.
   /// </para><para>
   /// If you set ErrorResponse to ErrorHandlerResponseEnum.ErrorResume, subsequent error handler callbacks will
   /// be bypassed and uCalc will resume operations as though no error had taken place.  Note that the condition
   /// that caused the error must be corrected before resuming, otherwise uCalc may get stuck in an infinite loop.
   /// </para><para>
   /// Some functions, such as ErrorExpression, ErrorLocation, and ErrorSymbol produce a meaningful result only
   /// during the parsing stage.  They do not cause other problems beyond meaningless data during the evaluation stage.
   /// </para><para>
   /// For Floating Point Environment errors, such as: Err_Float_Division_By_Zero, Err_Float_Inexact_Result,
   /// Err_Float_Invalid_Operation, Err_Float_Overflow, and Err_Float_Underflow, the uCalc error handler callbacks
   /// are called only if the corresponding environment bits are not masked.
   /// </para><para>
   /// </para></remarks>
   public Item AddErrorHandler(uCalcDelegate ErrorHandlerAddress , Int32 Position  = 0) { return new Item(AddErrorHandler_(uCalcHandle,ErrorHandlerAddress ,Position));    }
   
   /// <summary> Creates a new uCalc instance</summary>
   /// <remarks><para>This is not meant to be called directly.</para></remarks>
   public  IntPtr CreateuCalc() { return CreateuCalc_(uCalcHandle);    }
   
   /// <summary> Returns a data type object</summary>
   /// <param name="Index"> Index associated with the data type</param>
   /// <remarks><para>Each instance of uCalc has its own separate handle &amp; data type object for any given data type.  For
   /// instance, GetItemOf(&quot;Int64&quot;) in one instance of uCalc does not share the same handle as GetItemOf(&quot;Int64&quot;) in
   /// another instance.  This allows the same data type to have a different name and different formatting in
   /// different instances.  However, they share the same data type index across instances.  Given the index
   /// of a data type, you can directly obtain the proper data type object for a particular uCalc instance,
   /// faster than obtaining it by name as text with GetItemOf.</para></remarks>
   public DataType DataTypeOfIndex(BuiltInTypesEnum Index) { return new DataType(DataTypeOfIndex_(uCalcHandle,Index));    }
   
   /// <summary> Defines a uCalc.Item object (a constunction, function, operator, variable, etc)</summary>
   /// <param name="Expression"> Text containing item to be defined.  Expression must start with a keyword such as Function, Operator, Variable, Token, Syntax, etc., followed by a colon, before the expression</param>
   /// <param name="Address"> Address a function or variable is attached to</param>
   /// <param name="ExistingItem"> Address of existing item this definition is associated with</param>
   /// <remarks><para>Functions, operators, variables, constants, tokens, syntax constructs, etc., can be defined with
   /// this method.  Although there are specialized methods that do the same things, such as DefineFunction,
   /// DefineOperator, DefineVariable, DefineConstant, and DefineSyntax, this method may in same cases offer
   /// more flexibility.
   /// </para><para>
   /// The expression must be preceded by a keyword and colon, such as &quot;Variable:&quot;.  For a full list of keywords,
   /// see Definition Keywords.
   /// </para><para>
   /// Sections are separated by ~~ (two tilda signs).
   /// </para></remarks>
   public Item Define( string Expression ,IntPtr Address  = default(IntPtr),IntPtr ExistingItem  = default(IntPtr)) { return new Item(Define_(uCalcHandle,Expression ,Address,ExistingItem));    }
   
   /// <summary> Defines a constant</summary>
   /// <param name="Definition"> Text containing constant to be defined</param>
   /// <param name="VariableAddress"> Address the constant is attached to</param>
   /// <remarks><para>
   /// A constant definition is similar to that of a variable defined with DefineVariable, except that the end-user is not allowed to modify the value of a constant.
   /// You are, however, allowed to perform lower-level operations such as renaming a constant, or deleting it and re-defining a new constant or variable with the same
   /// name but different value.  If you attach the constant to the address of a variable in your host program, the host program may change the value.  However, when
   /// the end-user attempts to assign a value to the constant, an error is raised.
   /// </para><para>
   /// DefineConstant(&quot;x&quot;) is the same as: Define(&quot;Lock ~~ Variable: x&quot;)
   /// </para></remarks>
   public Item DefineConstant( string Definition ,IntPtr VariableAddress  = default(IntPtr)) { return new Item(DefineConstant_(uCalcHandle,Definition ,VariableAddress));    }
   
   /// <summary> Defines a data type</summary>
   /// <param name="TypeSize"> Number of bytes 1 scalar value of this type takes</param>
   /// <param name="AllocArrayValue"> Address of a function that will allocate an array of this type</param>
   /// <param name="AllocScalar"> Address of a function that will allocate a scalar of this type.  If a string is passed, this function must convert it to this type and store the value, otherwise the function must store a default value for this type.</param>
   /// <param name="AllocScalarByPtr"> Address of a function that takes a IntPtr to a value of this type, allocates a scalar and stores the copied value in the allocated space.</param>
   /// <param name="DeleteArray"> Address of a function that frees a memory block that was allocated with AllocValueArray</param>
   /// <param name="DeleteScalar"> Address of a function that frees a memory block that was allocated by AllocScalar or AllocScalarByPtr.</param>
   /// <param name="ResetScalar"> Address of a function that resets an allocated scalar back to the default value of this type.</param>
   /// <param name="ResetArray"> Address of a function that resets array elements back to the default value of this type.</param>
   /// <param name="SetValueArray"> Address of a function that stores an array element value.</param>
   /// <param name="SetValueScalar"> Address of a function that stores a scalar value.</param>
   /// <param name="SwapValues"> Address of a function that swaps two values of this type.</param>
   /// <param name="ToString"> Address of a function that converts this data type to a string.</param>
   /// <remarks><para>
   /// This allows you to define a data type that functions at the same level as the intrinsic types that come with uCalc.
   /// </para></remarks>
   public Item DefineDataType( Int32 TypeSize , IntPtr AllocArrayValue  , IntPtr AllocScalar  , IntPtr AllocScalarByPtr  , IntPtr DeleteArray  , IntPtr DeleteScalar  , IntPtr ResetScalar  , IntPtr ResetArray  , IntPtr SetValueArray  , IntPtr SetValueScalar  , IntPtr SwapValues  , IntPtr ToString ) { return new Item(DefineDataType_(uCalcHandle,TypeSize , AllocArrayValue , AllocScalar , AllocScalarByPtr , DeleteArray , DeleteScalar , ResetScalar , ResetArray , SetValueArray , SetValueScalar , SwapValues , ToString));    }
   
   /// <summary> Defines a function</summary>
   /// <param name="Definition"> Text containing function to be defined</param>
   /// <param name="FunctionAddress"> Address of callback function this function is attached to</param>
   /// <remarks><para>
   /// A function definition can either be self-contained, or it may be attached to the address of a callback function in your
   /// source code.
   /// </para><para>
   /// DefineFunction(&quot;f(x) = ...&quot;) is the same as: Define(&quot;Function: ...&quot;)
   /// </para></remarks>
   public Item DefineFunction( string Definition ,uCalcDelegate FunctionAddress = null) { return new Item(DefineFunction_(uCalcHandle,Definition ,FunctionAddress));    }
   
   /// <summary> Defines an operator</summary>
   /// <param name="Definition"> Text containing operator to be defined</param>
   /// <param name="iPrecedence"> Precedence level of the operator.</param>
   /// <param name="Grouping"> Operator associativity; arithmetic operators are typically grouped left-to-right; special operators, such as the equal sign, =, assignment operator are right-to-left</param>
   /// <param name="FunctionAddress"> Address of callback function this operator is attached to</param>
   /// <remarks><para>
   /// Many details for defining an operator are similar to those of defining a function.  In additionn, all operators must be
   /// defined with a precedence level.  You may use the Precedence function to obtain precedence levels of existing operators,
   /// and select a precedence level in comparison to those operators.  By default operators are grouped left to right.  But you
   /// may also set it to right to left.  This means that when adjacent operators have the same precedence level, the operands on
   /// the right will be evaluated first.
   /// </para><para>
   /// DefineOperator(&quot;...&quot;) is the same as: Define(&quot;Operator: ...&quot;)
   /// More later+++
   /// </para></remarks>
   public Item DefineOperator( string Definition ,  Int32 iPrecedence ,GroupingEnum Grouping = GroupingEnum.GroupLeftToRight,uCalcDelegate FunctionAddress = null) { return new Item(DefineOperator_(uCalcHandle,Definition , iPrecedence ,Grouping,FunctionAddress));    }
   
   /// <summary> Defines a syntax construct</summary>
   /// <param name="Definition"> Text containing syntax construct</param>
   public Item DefineSyntax( string Definition) { return new Item(DefineSyntax_(uCalcHandle,Definition));    }
   
   /// <summary> Defines a transform</summary>
   /// <param name="FromExpr"> Pattern representing the text that must be transformed</param>
   /// <param name="ToExpr"> The text that the matching test is transformed to</param>
   /// <param name="ItemHandle"> Handle</param>
   public Item DefineTransform( string FromExpr ,  string ToExpr ,IntPtr ItemHandle  = default(IntPtr)) { return new Item(DefineTransform_(uCalcHandle,FromExpr , ToExpr ,ItemHandle));    }
   
   /// <summary> Defines a variable</summary>
   /// <param name="Definition"> Text containing variable to be defined</param>
   /// <param name="VariableAddress"> Address of variable in host program this uCalc variable is attached to</param>
   /// <remarks><para>
   /// A variable consists of ...
   /// </para><para>
   /// DefineVariable(&quot;...&quot;) is the same as: Define(&quot;Variable: x&quot;)
   /// </para></remarks>
   public Item DefineVariable( string Definition ,IntPtr VariableAddress  = default(IntPtr)) { return new Item(DefineVariable_(uCalcHandle,Definition ,VariableAddress));    }
   
   /// <summary> Returns the expression associated with an error that was raised</summary>
   public  string ErrorExpression() { return Marshal.PtrToStringAnsi(ErrorExpression_(uCalcHandle));    } 
   
   /// <summary> Returns the location of an error that was raised</summary>
   public  Int32 ErrorLocation() { return ErrorLocation_(uCalcHandle);    }
   
   /// <summary> Returns a message associated with the error that was raised</summary>
   /// <param name="Index"> Index of error message to return</param>
   /// <remarks><para>If an error was raised, this returns a message describing the recent error.  If the Index argument is specified,
   /// then the error message associated with this Index is returned.  A value must be chosen from ErrorNumberEnum.
   /// </para><para>
   /// Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
   /// </para></remarks>
   public  string ErrorMessage(ErrorNumberEnum Index = ErrorNumberEnum.Err_None) { return Marshal.PtrToStringAnsi(ErrorMessage_(uCalcHandle,Index));    } 
   
   /// <summary> Returns the error number of the error that was raised</summary>
   /// <remarks><para>
   /// Returns the error number of the most recently raised error.
   /// </para><para>
   /// Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
   /// </para></remarks>
   public  ErrorNumberEnum ErrorNumber() { return ErrorNumber_(uCalcHandle);    }
   
   /// <summary> Sets the error response for an error handler</summary>
   /// <param name="Response"> Indicates what should happen now that an error is raised</param>
   /// <remarks><para>When an error is raised, you have the option of aborting the rest of the option (which is the default option),
   /// or you may correct the problem, and then resume the operation.  Or you may perform an action, and than raise the
   /// error again for a subsequent error handle to take care of.
   /// </para><para>
   /// This method can only be called from within a callback of an error handler routine definined with AddErrorHandler.
   /// The options to use can be found at ErrorHandlerResponseEnum.
   /// </para></remarks>
   public  void ErrorResponse(ErrorHandlerResponseEnum Response) {  ErrorResponse_(uCalcHandle,Response);    }
   
   /// <summary> Returns the symbol associated with the error that is raised</summary>
   public  string ErrorSymbol() { return Marshal.PtrToStringAnsi(ErrorSymbol_(uCalcHandle));    } 
   
   /// <summary> Sets a new expression after an error is raised</summary>
   /// <param name="Expression"> Error expression</param>
   public  void ErrorSetExpression( string Expression) {  ErrorSetExpression_(uCalcHandle,Expression);    }
   
   /// <summary> Parses and evaluates an expression in one step</summary>
   /// <param name="Expression"> Expression to be evaluated</param>
   /// <remarks><para>This takes a math expression and returns a numeric result.</para></remarks>
   public  double Eval( string Expression) { return Eval_(uCalcHandle,Expression);    }
   
   /// <summary> Parses and evaluates an expression of any data type in one step, returning the result as a string</summary>
   /// <param name="Expression"> Expression to be evaluated</param>
   /// <remarks><para>This is similar to Eval, however, it can return non-numeric results, such as strings.</para></remarks>
   public  string EvalStr( string Expression) { return Marshal.PtrToStringAnsi(EvalStr_(uCalcHandle,Expression));    } 
   
   /// <summary> Expands an expression</summary>
   /// <param name="Expression"> Expression to be expanded</param>
   public  string Expand( string Expression) { return Marshal.PtrToStringAnsi(Expand_(uCalcHandle,Expression));    } 
   
   /// <summary> Returns each step leading to an expression expansion</summary>
   /// <param name="Expression"> Expression to be expanded</param>
   /// <param name="Separator"> Separator used between steps</param>
   public  string ExpandSteps( string Expression , string Separator = "") { return Marshal.PtrToStringAnsi(ExpandSteps_(uCalcHandle,Expression ,Separator));    } 
   
   /// <summary> Clears floating point exception(s) that were raised</summary>
   /// <param name="Exceptions"> Exceptions</param>
   public  Int32 FPEnvClearException( Int32 Exceptions  = 0) { return FPEnvClearException_(uCalcHandle,Exceptions);    }
   
   /// <summary> Returns the floating point environment error mask</summary>
   public  Int32 FPEnvGetExceptionMask() { return FPEnvGetExceptionMask_(uCalcHandle);    }
   
   /// <summary> Raises a floating point exception in a uCalc.Callback function</summary>
   /// <param name="Exception"> Bitmask value consisting of one or more exception flags to raise</param>
   /// <returns> Returns a 0 if all the specified execptions were raised successfully, otherwise a non-zero value is returned.</returns>
   /// <remarks><para>This attempts to raise the exception(s) specfied by the Exception parameter.  This is based on the C++
   /// feraiseexcept() function.  If you are using C++, you may use the pre-defined macro values of 8 ,
   /// 1 , 16 , 4 , 2 , 31 .  If you are using another compiler, you may
   /// use the matching members of the FPEnvExceptionEnum enumerator.
   /// </para></remarks>
   public  Int32 FPEnvRaiseException(FPEnvExceptionEnum Exception = FPEnvExceptionEnum.Invalid) { return FPEnvRaiseException_(uCalcHandle,Exception);    }
   
   /// <summary> Gets the current floating point environment rounding option</summary>
   public  FPEnvRoundingEnum FPEnvGetRounding() { return FPEnvGetRounding_(uCalcHandle);    }
   
   /// <summary> Sets the current floating point environment mask</summary>
   public  void FPEnvSetExceptionMask( Int32 Exceptions) {  FPEnvSetExceptionMask_(uCalcHandle,Exceptions);    }
   
   /// <summary> Unmasks the current floating point environment mask</summary>
   public  void FPEnvSetExceptionUnMask( Int32 Exceptions) {  FPEnvSetExceptionUnMask_(uCalcHandle,Exceptions);    }
   
   /// <summary> Sets the floating point environment rounding option</summary>
   public  Int32 FPEnvSetRounding(FPEnvRoundingEnum Direction) { return FPEnvSetRounding_(uCalcHandle,Direction);    }
   
   /// <summary> Toggles the floating point environment mask</summary>
   public  void FPEnvToggleExceptionMask(FPEnvExceptionEnum Exception) {  FPEnvToggleExceptionMask_(uCalcHandle,Exception);    }
   
   /// <summary> Returns the handle of a uCalc.Item given a name</summary>
   public  IntPtr HandleOf( string ItemName,  Int64 Properties = 0,  bool SkipExpansion = false) { return HandleOf_(uCalcHandle,ItemName, Properties, SkipExpansion);    }
   
   /// <summary> Returns the uCalc.Item object associated with the given name</summary>
   /// <param name="ItemName"> Name of uCalc.Item that you want to get</param>
   /// <param name="Properties"> Properties to help pinpoint the right Item, in case several items share the same name</param>
   /// <param name="SkipExpansion"> Some items are define as a syntax construct alias of another item.  By default the symbol is expanded first.  However, use this option if the alias is the object you&#39;re actually after.</param>
   /// <remarks><para>
   /// If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
   /// </para><para>
   /// In case multiple items share the same name, you can narrow it down using properties.  For instance, the + operator has a
   /// binary and unary versions.  You can specify Prefix if you wanted the unary one.
   /// </para><para>
   /// Note: A ItemIsEnum member should not be passed directly as the second argument.  Instead Properties, with one or more
   /// ItemIsEnum members should be passed.
   /// </para></remarks>
   public Item GetItemOf( string ItemName , Int64 Properties  = 0, bool SkipExpansion = false) { return new Item(GetItemOf_(uCalcHandle,ItemName ,Properties,SkipExpansion));    }
   
   /// <summary> Returns the nth uCalc.Item object that matches a property</summary>
   /// <param name="Property_"> Property of the uCalc.Item object you&#39;re after</param>
   /// <param name="nth"> The nth object</param>
   /// <param name="DataTypeName"> The data type of the object</param>
   /// <remarks><para>
   /// This can be used to cycle through defined items.
   /// If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
   /// </para></remarks>
   public Item GetItemOfProperty(ItemIsEnum Property_ ,  Int32 nth , string DataTypeName = "") { return new Item(GetItemOfProperty_(uCalcHandle,Property_ , nth ,DataTypeName));    }
   
   /// <summary> Sets the license code</summary>
   public  Int32 License(LicenseTypeEnum LicenseType,  Int32 L1 = 0,  Int32 L2 = 0,  Int32 L3 = 0,  Int32 L4 = 0,  Int32 L5 = 0) { return License_(uCalcHandle,LicenseType, L1, L2, L3, L4, L5);    }
   
   /// <summary> ???</summary>
   public Item NamespaceChild( Int32 nth = 0) { return new Item(NamespaceChild_(uCalcHandle,nth));    }
   
   /// <summary> ???</summary>
   public  Int32 NamespaceChildCount() { return NamespaceChildCount_(uCalcHandle);    }
   
   /// <summary> ???</summary>
   public Item NamespaceParent( Int32 Level = 0) { return new Item(NamespaceParent_(uCalcHandle,Level));    }
   
   /// <summary> ???</summary>
   public Item NamespaceSymbol( Int32 nth) { return new Item(NamespaceSymbol_(uCalcHandle,nth));    }
   
   /// <summary> ???</summary>
   public  Int32 NamespaceSymbolCount() { return NamespaceSymbolCount_(uCalcHandle);    }
   
   /// <summary> Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
   /// <param name="Expression"> Expression to be parsed</param>
   /// <param name="DataType"> Data type of return value</param>
   public ParsedExpr Parse( string Expression , string DataType = "") { return new ParsedExpr(Parse_(uCalcHandle,Expression ,DataType));    }
   
   /// <summary> Returns the precedence level of an operator or the level between two operators</summary>
   public Int32  Precedence( string Operator1,  string Operator2 = "") { return Precedence_Overload_uCalc_(uCalcHandle,Operator1, Operator2);    }
   
   /// <summary> Combines properties into a numeric value that can be passed to GetItemOf</summary>
   /// <remarks><para>
   /// ItemIsEnum properties are numbered sequentially (0, 1, 2, 3, 4, ...).  However the Properties argument
   /// of the GetItemOf function requires properties in the 2^x sequence (1, 2, 4, 8, 16, ...).  So you
   /// must pass one or more ItemIsEnum to Properties before passing it to GetItemOf.
   /// </para></remarks>
   public  Int64 Properties(ItemIsEnum p1, ItemIsEnum p2 = ItemIsEnum.AnyItem, ItemIsEnum p3 = ItemIsEnum.AnyItem, ItemIsEnum p4 = ItemIsEnum.AnyItem) { return Properties_(uCalcHandle,p1, p2, p3, p4);    }
   
   /// <summary> (Use Release instead)</summary>
   /// <remarks><para>Use Release</para></remarks>
   public  void ReleaseuCalc() {  ReleaseuCalc_(uCalcHandle);    }
   
   /// <summary> Removes any output decoration</summary>
   public  void ResetOutput() {  ResetOutput_(uCalcHandle);    }
   
   /// <summary> Returns default settings</summary>
   /// <remarks><para>The Item returned contains the default settings for the current uCalc instance.  To define new settings,
   /// use Define(&quot;Default ~~ ...&quot;).</para></remarks>
   public Item Default() { return new Item(Default_(uCalcHandle));    }
   
   /// <summary> Allows a callback to decorate the output of EvalStr</summary>
   /// <param name="FunctionAddress"> Address of callback function that formats the output</param>
   /// <param name="Properties"> Name of datatype whose output will be formated and/or ranking among other output formats; leave blank to affect all data types and use default ranking.</param>
   public Item  SetOutput(uCalcDelegate FunctionAddress ,  string Properties) { return new Item(SetOutput_Overload_uCalc_Callback_(uCalcHandle,FunctionAddress , Properties));    }
   
   /// <summary> Formats the appearance of results returned by EvalStr or EvaluateStr</summary>
   /// <param name="Definition"> Format definition for the output</param>
   /// <remarks><para>
   /// This function lets you modify the output returned by EvalStr or EvaluateStr.  The definition may contain one or more
   /// of the following:
   /// </para><para>
   /// Datatype:  If you specify a data type, then the modification will affect only the evaluation result of expressions
   /// that evaluate to the specified data type.
   /// </para><para>
   /// InsertAt:  By default, if you define multiple outputs, then the one defined most recently is applied first, followed by
   /// the other outputs in reverse order in which they were defined, ending with the output that was defined first.  You may
   /// specify a different insertion point than the default with this parameter.
   /// </para><para>
   /// Expression:  This represents what the modified output should look like.  The expression consists of an identifier
   /// (any temporary variable name), followed by an equal sign, followed by a string expression representing the value
   /// that will be returned.  The variable holds the original value that would have been returned.
   /// </para></remarks>
   public Item  SetOutput( string Definition) { return new Item(SetOutput_Overload_uCalc_NoCallback_(uCalcHandle,Definition));    }
   
   /// <summary> Returns the value located at the specified IntPtr address</summary>
   public  string ValueAt( IntPtr ValuePtr,  string DataType) { return Marshal.PtrToStringAnsi(ValueAt_(uCalcHandle,ValuePtr, DataType));    } 
   
   /// <summary> Returns the handle of the uCalc object</summary>
   /// <remarks><para>Generally for internal use</para></remarks>
   public IntPtr  Handle() { return Handle_Overload_uCalc_(uCalcHandle);    }
   
   /// <summary> Releases a uCalc instance</summary>
   /// <remarks><para>When a uCalc instance is no longer needed, it can be released with this function.  This releases all items
   /// associated with the uCalc instance.
   /// </para><para>
   /// Note: The uCalc object named uc, which is instantiated at startup (except in C#) should not be explicitely released.
   /// In C#, if there is only 1 uCalc object left, do not explicitely release it.
   /// </para><para>
   /// Note: A uCalc object must explicitely be released with Release for it to be released.  A uCalc object is not
   /// automatically released from memory when the object goes out of scope.</para></remarks>
   public void Release()  { ReleaseuCalc(); uCalcHandle = default(IntPtr);    }
   
   /// 
   public void Reset()  { ReleaseuCalc(); uCalcHandle = CreateuCalc();    }
   
   public uCalc() { uCalcHandle = default(IntPtr); uCalcHandle = CreateuCalc();    }
   public uCalc( uCalc uCalcInstance) { uCalcHandle = uCalcInstance.uCalcHandle;    }
   public uCalc( IntPtr InstHandle) { uCalcHandle = InstHandle;    }
   public uCalc(LicenseTypeEnum LicenseType,  Int32 L1 = 0,  Int32 L2 = 0,  Int32 L3 = 0,  Int32 L4 = 0,  Int32 L5 = 0) { License(LicenseType, L1, L2, L3, L4, L5); uCalcHandle = CreateuCalc();    }
   List<uCalcDelegate> Addresses = new List<uCalcDelegate>();
   public uCalcDelegate PinAddr(uCalcDelegate Address) { Addresses.Add(Address); return Address;    }
} // uCalc

