unit uCalcDlp;

interface
{$IFDEF WIN64}
  const uCalcDLL = 'uCalc64.dll';
{$ELSE}
  const uCalcDLL = 'uCalc32.dll';
{$ENDIF}
type


/// <summary>S: Data types that come with uCalc</summary>
/// <remarks>These are built-in data types that come with uCalc</remarks>
BuiltInTypesEnum = (
   Type_Unspecified,
   Type_Auto,
   /// <summary> Boolean -- </summary>
   Type_Boolean,
   /// <summary> Int8 +++ Rename these to things like Type_Int8</summary>
   Type_Integer_8,
   /// <summary> Byte</summary>
   Type_Integer_8u,
   /// <summary> Int16</summary>
   Type_Integer_16,
   /// <summary> Int16u</summary>
   Type_Integer_16u,
   /// <summary> Int32</summary>
   Type_Integer_32,
   /// <summary> Int32u</summary>
   Type_Integer_32u,
   /// <summary> Int64</summary>
   Type_Integer_64,
   /// <summary> Int64u</summary>
   Type_Integer_64u,
   /// <summary> Int128</summary>
   Type_Integer_128,
   /// <summary> Int128u</summary>
   Type_Integer_128u,
   /// <summary> Single</summary>
   Type_Float_Single,
   /// <summary> Double</summary>
   Type_Float_Double,
   /// <summary> Extended</summary>
   Type_Float_Extended,
   Type_Size_t,
   Type_Complex,
   Type_Decimal,
   Type_BigNum,
   Type_Date,
   Type_String,
   /// <summary> LPCSTR</summary>
   Type_String_LPCSTR,
   /// <summary> FixedStr</summary>
   Type_String_Fixed,
   /// <summary> WideStr</summary>
   Type_String_Wide,
   /// <summary> StackList</summary>
   Type_List_Stack,
   /// <summary> TableList</summary>
   Type_List_Table,
   /// <summary> SortedList</summary>
   Type_List_Sorted,
   Type_Null,
   Type_Void,
   Type_Any,
   Type_Pointer,
   Type_Omni,
   Type_UserDefined,
   Type_DataType,
   Type_Namespace,
   Type_IntPtr,
   Type_ADDR,
   Type_uCalcPtr,
   Type_Item,
   Type_ParsedExpr,
   Type_uCalcCallback,
   Type_Object,
   Type_Intrinsic
);

/// <summary>S: Data type constructors</summary>
DataTypeConstruction = (
   TypeConstr_ByteSize,
   TypeConstr_AllocArray,
   TypeConstr_AllocScalar,
   TypeConstr_AllocByPtr,
   TypeConstr_ReleaseArray,
   TypeConstr_ReleaseScalar,
   TypeConstr_ResetScalar,
   TypeConstr_ResetArray,
   TypeConstr_SetArrayValue,
   TypeConstr_SetScalarValue,
   TypeConstr_SwapScalarValues
);

/// <summary>S: Error hander responses</summary>
ErrorHandlerResponseEnum = (
   /// <summary> Immediately stops parsing</summary>
   ErrorAbort,
   /// <summary> Continues parsing (after the error is resolved)</summary>
   ErrorResume,
   /// <summary> Raises the same error again for the next error handler to take care of</summary>
   ErrorReRaise
);

/// <summary>S: Error numbers that uCalc may return</summary>
ErrorNumberEnum = (
   /// <summary> No error</summary>
   Err_None,
   /// <summary> Dynamically defined user error</summary>
   Err_Dynamically_Defined,
   /// <summary> Syntax error</summary>
   Err_Syntax_Error,
   /// <summary> Undefined identifier</summary>
   Err_Undefined_Identifier,
   /// <summary> File not found</summary>
   Err_FileNotFound,
   /// <summary> Unrecognized token</summary>
   Err_Unrecognized_Token,
   /// <summary> Unrecognized command</summary>
   Err_Unrecognized_Command,
   /// <summary> Data type mismatch</summary>
   Err_Datatype_Mismatch,
   /// <summary> Invalid number of arguments</summary>
   Err_Invalid_Argument_Count,
   /// <summary> Invalid definition</summary>
   Err_Invalid_Definition,
   /// <summary> Code block delimiter error</summary>
   Err_CodeBlock_Error,
   /// <summary> Undefined callback</summary>
   Err_Undefined_Callback,
   /// <summary> Error message already defined</summary>
   Err_ErrorMessageAlreadyDefined,
   /// <summary> This item cannot be modified</summary>
   Err_ItemCannotBeModified,
   /// <summary> Unrecognized Class_ member</summary>
   Err_Unrecognized_Class_Member,
   /// <summary> Unrecognized namespace member</summary>
   Err_Unrecognized_Namespace_Member,
   /// <summary> Unbalanced quote</summary>
   Err_Unbalanced_Quote,
   /// <summary> Value cannot be assigned here</summary>
   Err_ValueCannotBeAssigned,
   /// <summary> Array_ Or_ list bound exceeded</summary>
   Err_Array_Bounds_Exceeded,
   /// <summary> Division by 0</summary>
   Err_Float_Division_By_Zero,
   /// <summary> Inexact result</summary>
   Err_Float_Inexact_Result,
   /// <summary> Invalid operation</summary>
   Err_Float_Invalid_Operation,
   /// <summary> Floating point overflow</summary>
   Err_Float_Overflow,
   /// <summary> Floating point underflow</summary>
   Err_Float_Underflow,
   Err_Customized1 = 100,
   Err_Customized2,
   Err_Customized3,
   Err_Customized4,
   Err_Customized5,
   Err_Customized6,
   Err_Customized7,
   Err_Customized8,
   Err_Customized9,
   Err_Customized10,
   Err_Customized11,
   Err_Customized12,
   Err_Customized13,
   Err_Customized14,
   Err_Customized15,
   Err_Customized16,
   Err_Customized17,
   Err_Customized18,
   Err_Customized19,
   Err_Customized20
);

/// <summary>S: Operator_ grouping associativity</summary>
GroupingEnum = (
   GroupLeftToRight,
   GroupRightToLeft
);

/// <summary>S: Properties that a uCalc item may have</summary>
/// <remarks>
///Items defined with uCalc typically have a combination of properties.  A variable defined
///with DefineVariable for instance will have the ItemIsEnum.Variable property set.  An Operator_
///will have the ItemIsEnum.Operator_ property set.  If it's an infix Operator_, in addition to
///having the ItemIsEnum.Operator_ property set, ItemIsEnum.Infix will also be set.
///property will also be set.
///
///The IsProperty Function_ can determine if a given uCalc.Item has a certain property.
///
///You can use the GetItemOfProperty Function_ to cycle through the list of all items that share
///certain properties, Or_ to disambiguate between items that share the same name but have
///different roperties (for instance the + Operator_ with the Prefix property vs another one
///with the Infix property).
///
///Some members are keywords in VB.NET.  To avoid naming conflict, the following members have been
///renamed in the VB.NET header: Function_, Operator_, Optional_, ByVal_, ByRef_, Class_, And_ NameSpace_.
///</remarks>
ItemIsEnum = (
   AnyItem,
   LeftToRight,
   RightToLeft,
   DataType_,
   Array_,
   Class_,
   NameSpace,
   Variable,
   LocalVariable,
   Function_,
   Operator_,
   FunctionOrOperator,
   Prefix,
   Infix,
   Postfix,
   AddressOwner,
   ByVal,
   ByRef,
   ByCopy,
   ByExpr,
   ByHandle,
   Pointer_,
   Optional,
   VariableNumberOfArgs,
   RegEx,
   LiteralItem,
   Locked,
   Transform,
   InTable,
   ItemDef,
   ErrorHandler,
   StopNow,
   ApiCall,
   CaseSensitive,
   QuotedText,
   ChangeMulti,
   ExpandArg,
   IgnoreStatementSep,
   IgnoreCodeBlock,
   SyntaxStop,
   SkipEvalExpand,
   ExprUnit,
   EagerArg,
   SkipSyntax,
   WhiteSpaceCounts,
   IgnoreQuote,
   UpToEndOrSeparator,
   UpToUserStop,
   EvalStrOutput,
   SkipOutput,
   DefaultSettings,
   NotFound,
   MiscItem
);

/// <summary>S: Licensing options for uCalc products</summary>
LicenseTypeEnum = (
   License_Demo,
   License_FMP_Free,
   License_FMP_Standard_32,
   License_FMP_Standard_64,
   License_FMP_32_and_64,
   License_LanguageBuilder,
   License_GeneralParser,
   License_Interpreter,
   License_ConsoleCalc,
   License_WindowsCalc,
   License_StringLib,
   License_Transform,
   License_TransformLib,
   License_Search,
   License_SearchLib,
   License_Library1,
   License_Library2,
   License_Beta,
   License_Debugger,
   License_uCalc_All,
   License_Developer,
   License_EndUser
);

/// <summary>S: Types of tokens</summary>
TokenEnum = (
   None,
   /// <summary> A token defined without any specified property is generic (default)</summary>
   Generic,
   /// <summary> Represents the absence of any character (token not found)</summary>
   Empty,
   /// <summary> A token that behaves as alphanuric (works with word boundaries)</summary>
   AlphaNumeric,
   /// <summary> A token that serves as argument separator (typically a comma)</summary>
   ArgSeparator,
   /// <summary> A token that starts a code block; typically characters like {, [, Or_ (</summary>
   CodeBlock,
   /// <summary> The closing token that matches a CodeBlock token, such as }, ], Or_ )</summary>
   CodeBlockMatch,
   /// <summary> A token that separates an object from its member, typically a . (a dot)</summary>
   MemberAccess,
   /// <summary> A token that represents a literal value, typically a quoted string</summary>
   Literal,
   /// <summary> A token that represents a statement separator, typically a ; (semicolon)</summary>
   StatementSep,
   /// <summary> A token that represents whitespace, typically tab Or_ space characters</summary>
   WhiteSpace,
   /// <summary> A token that is not delimited by word boundaries (+, -, *, etc), consecutive characters are reduced until they match a value that exists in the symbol table Or_ until 1 character</summary>
   Reducible,
   /// <summary> Non-character that represents the end of a stream of text</summary>
   Eof,
   CurrentThread, 
   LocalVar,
   StaticVar,
   /// <summary> Item in parenthesis (Or_ quotes) following this token will be defined while the expression is parsed</summary>
   DefineNow = StaticVar,
   TempDef,
   /// <summary> The expression in parathesis (Or_ quotes) following the token is evaluated And_ inserted in place of the token in the string</summary>
   EvalInsert,
   EvalDuringParse,
   /// <summary> Works like EvalInsert, except the expression is expanded instead of evaluated</summary>
   ExpandInsert,
   /// <summary> The contents of the quoted file name following this token is inserted into the string</summary>
   FileInclude,
   CurrentSpace,
   DefSpace,
   EvalSpace
);

/// <summary>S: Boolean True_ Or_ False_</summary>
TrueFalseEnum = (
   /// <summary> For compilers in which False_ is a keyword use False_ instead</summary>
   False_,
   /// <summary> For compilers in which True_ is a keyword use True_ instead</summary>
   True_
);

/// <summary>S: Floating point environment exceptions</summary>
FPEnvExceptionEnum = (
   /// <summary> Floating point division by zero, Or_ operation resulting in infinity, occured (pole error)</summary>
   DivisionByZero = 8,
   /// <summary> The result is not an exact floating point number</summary>
   Inexact = 1,
   /// <summary> Argument passed to a Function_ Or_ Operator_ was invalid (domain error)</summary>
   Invalid = 16,
   /// <summary> The result is too large to be represented as a floating point value of the return type (overflow range error)</summary>
   Overflow = 4,
   /// <summary> The result is too small to be represented as a floating point value of the return type, causing loss of precision (underflow range error)</summary>
   Underflow = 2,
   /// <summary> Represents selection of all the floating point exceptions: DivisionByZero, Inexact, Invalid, Overflow, And_ Underflow (Bitwise ORed)</summary>
   AllExceptions = 31
);

/// <summary>S: Floating point rouding options</summary>
FPEnvRoundingEnum = (
   /// <summary> Rounds downward (towards negative infinity)</summary>
   Round_Down = 512,
   /// <summary> Rounds towards nearest value</summary>
   Round_ToNearest = 0,
   /// <summary> Rounds towards zero</summary>
   Round_TowardZero = 768,
   /// <summary> Rounds upward (towards positive infinity)</summary>
   Round_Up = 256
);





{DLLImport}function Evaluate__(ExprHandle: System.Pointer):  Double; cdecl; external uCalcDLL name 'Evaluate';
{DLLImport}function EvaluateInt32__(ExprHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'EvaluateInt32';
{DLLImport}function EvaluateStr__(ExprHandle: System.Pointer):  PAnsiChar; cdecl; external uCalcDLL name 'EvaluateStr';
{DLLImport}function EvaluateVoid__(ExprHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'EvaluateVoid';
{DLLImport}procedure Execute__(ExprHandle: System.Pointer); cdecl; external uCalcDLL name 'Execute';
{DLLImport}procedure ReleaseExpr__(ExprHandle: System.Pointer); cdecl; external uCalcDLL name 'ReleaseExpr';
{DLLImport}function Handle_Overload_ParsedExpr__(ExprHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'Handle_Overload_ParsedExpr';
{DLLImport}function ElementCount__(ItemHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'ElementCount';
{DLLImport}function GetDataType__(ItemHandle: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'GetDataType';
{DLLImport}function FunctionAddress__(ItemHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'FunctionAddress';
{DLLImport}function IsProperty__(ItemHandle: System.Pointer;ItemProperty: ItemIsEnum):  Integer; cdecl; external uCalcDLL name 'IsProperty';
{DLLImport}function Name_Overload_Item__(ItemHandle: System.Pointer):  PAnsiChar; cdecl; external uCalcDLL name 'Name_Overload_Item';
{DLLImport}function NextOverload__(ItemHandle: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'NextOverload';
{DLLImport}function Precedence_Overload_Item__(ItemHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'Precedence_Overload_Item';
{DLLImport}procedure ReleaseItem__(ItemHandle: System.Pointer); cdecl; external uCalcDLL name 'ReleaseItem';
{DLLImport}procedure Rename__(ItemHandle: System.Pointer;NewName: AnsiString); cdecl; external uCalcDLL name 'Rename';
{DLLImport}function ReplacementText__(ItemHandle: System.Pointer):  PAnsiChar; cdecl; external uCalcDLL name 'ReplacementText';
{DLLImport}procedure SetVariableValue__(ItemHandle: System.Pointer;Value: Double); cdecl; external uCalcDLL name 'SetVariableValue';
{DLLImport}procedure SetVariableValueAny__(ItemHandle: System.Pointer;Value: AnsiString); cdecl; external uCalcDLL name 'SetVariableValueAny';
{DLLImport}procedure SetVariableValueByte__(ItemHandle: System.Pointer;Value: Byte); cdecl; external uCalcDLL name 'SetVariableValueByte';
{DLLImport}procedure SetVariableValueInt16__(ItemHandle: System.Pointer;Value: Int16); cdecl; external uCalcDLL name 'SetVariableValueInt16';
{DLLImport}procedure SetVariableValueInt32__(ItemHandle: System.Pointer;Value: Int32); cdecl; external uCalcDLL name 'SetVariableValueInt32';
{DLLImport}procedure SetVariableValueInt64__(ItemHandle: System.Pointer;Value: Int64); cdecl; external uCalcDLL name 'SetVariableValueInt64';
{DLLImport}procedure SetVariableValueSng__(ItemHandle: System.Pointer;Value: Single); cdecl; external uCalcDLL name 'SetVariableValueSng';
{DLLImport}procedure SetVariableValueDbl__(ItemHandle: System.Pointer;Value: Double); cdecl; external uCalcDLL name 'SetVariableValueDbl';
{DLLImport}procedure SetVariableValueStr__(ItemHandle: System.Pointer;Value: AnsiString); cdecl; external uCalcDLL name 'SetVariableValueStr';
{DLLImport}procedure SetVariableValuePtr__(ItemHandle: System.Pointer;Value: System.Pointer); cdecl; external uCalcDLL name 'SetVariableValuePtr';
{DLLImport}procedure SetVariableValueByPtr__(ItemHandle: System.Pointer;Value: System.Pointer); cdecl; external uCalcDLL name 'SetVariableValueByPtr';
{DLLImport}function Text__(ItemHandle: System.Pointer):  PAnsiChar; cdecl; external uCalcDLL name 'Text';
{DLLImport}function GetuCalc_Overload_Item__(ItemHandle: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'GetuCalc_Overload_Item';
{DLLImport}function Value__(ItemHandle: System.Pointer):  Double; cdecl; external uCalcDLL name 'Value';
{DLLImport}function ValueByte__(ItemHandle: System.Pointer):  Byte; cdecl; external uCalcDLL name 'ValueByte';
{DLLImport}function ValueInt16__(ItemHandle: System.Pointer):  Int16; cdecl; external uCalcDLL name 'ValueInt16';
{DLLImport}function ValueInt32__(ItemHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'ValueInt32';
{DLLImport}function ValueInt64__(ItemHandle: System.Pointer):  Int64; cdecl; external uCalcDLL name 'ValueInt64';
{DLLImport}function ValueSng__(ItemHandle: System.Pointer):  Single; cdecl; external uCalcDLL name 'ValueSng';
{DLLImport}function ValueDbl__(ItemHandle: System.Pointer):  Double; cdecl; external uCalcDLL name 'ValueDbl';
{DLLImport}function ValueStr__(ItemHandle: System.Pointer):  PAnsiChar; cdecl; external uCalcDLL name 'ValueStr';
{DLLImport}function ValuePtr__(ItemHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'ValuePtr';
{DLLImport}function ValueAddr__(ItemHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'ValueAddr';
{DLLImport}function Handle_Overload_Item__(ItemHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'Handle_Overload_Item';
{DLLImport}function Arg__(ExprPartHandle: System.Pointer;Index: Int32):  Double; cdecl; external uCalcDLL name 'Arg';
{DLLImport}function ArgCount__(ExprPartHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'ArgCount';
{DLLImport}function ArgObj__(ExprPartHandle: System.Pointer;Index: Int32): System.Pointer; cdecl; external uCalcDLL name 'ArgObj';
{DLLImport}function ArgAddr__(ExprPartHandle: System.Pointer;Index: Int32):  System.Pointer; cdecl; external uCalcDLL name 'ArgAddr';
{DLLImport}function ArgExpr__(ExprPartHandle: System.Pointer;Index: Int32): System.Pointer; cdecl; external uCalcDLL name 'ArgExpr';
{DLLImport}function ArgInt32__(ExprPartHandle: System.Pointer;Index: Int32):  Int32; cdecl; external uCalcDLL name 'ArgInt32';
{DLLImport}function ArgInt64__(ExprPartHandle: System.Pointer;Index: Int32):  Int64; cdecl; external uCalcDLL name 'ArgInt64';
{DLLImport}function ArgPtr__(ExprPartHandle: System.Pointer;Index: Int32):  System.Pointer; cdecl; external uCalcDLL name 'ArgPtr';
{DLLImport}function ArgStr__(ExprPartHandle: System.Pointer;Index: Int32):  PAnsiChar; cdecl; external uCalcDLL name 'ArgStr';
{DLLImport}procedure Return__(ExprPartHandle: System.Pointer;Value: Double); cdecl; external uCalcDLL name 'Return';
{DLLImport}procedure ReturnInt32__(ExprPartHandle: System.Pointer;Value: Int32); cdecl; external uCalcDLL name 'ReturnInt32';
{DLLImport}procedure ReturnInt64__(ExprPartHandle: System.Pointer;Value: Int64); cdecl; external uCalcDLL name 'ReturnInt64';
{DLLImport}procedure ReturnPtr__(ExprPartHandle: System.Pointer;Value: System.Pointer); cdecl; external uCalcDLL name 'ReturnPtr';
{DLLImport}procedure ReturnStr__(ExprPartHandle: System.Pointer;Value: AnsiString); cdecl; external uCalcDLL name 'ReturnStr';
{DLLImport}function ErrorRaise__(ExprPartHandle: System.Pointer;ErrNumber: ErrorNumberEnum):  Int32; cdecl; external uCalcDLL name 'ErrorRaise';
{DLLImport}function ErrorRaiseMessage__(ExprPartHandle: System.Pointer;ErrMessage: AnsiString):  Int32; cdecl; external uCalcDLL name 'ErrorRaiseMessage';
{DLLImport}function GetuCalc_Overload_Callback__(ExprPartHandle: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'GetuCalc_Overload_Callback';
{DLLImport}function Handle_Overload_Callback__(ExprPartHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'Handle_Overload_Callback';
{DLLImport}function OpItem__(ExprPartHandle: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'OpItem';
{DLLImport}function AllocArray__(DataTypeHandle: System.Pointer;ArraySize: Int32):  System.Pointer; cdecl; external uCalcDLL name 'AllocArray';
{DLLImport}function AllocByPtr__(DataTypeHandle: System.Pointer;ValuePtr: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'AllocByPtr';
{DLLImport}function AllocScalar__(DataTypeHandle: System.Pointer; Value: AnsiString):  System.Pointer; cdecl; external uCalcDLL name 'AllocScalar';
{DLLImport}function ByteSize__(DataTypeHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'ByteSize';
{DLLImport}function Index__(DataTypeHandle: System.Pointer):  BuiltInTypesEnum; cdecl; external uCalcDLL name 'Index';
{DLLImport}function GetItem__(DataTypeHandle: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'GetItem';
{DLLImport}function Name_Overload_DataType__(DataTypeHandle: System.Pointer):  PAnsiChar; cdecl; external uCalcDLL name 'Name_Overload_DataType';
{DLLImport}procedure ReleaseArray__(DataTypeHandle: System.Pointer;ValuePtr: System.Pointer); cdecl; external uCalcDLL name 'ReleaseArray';
{DLLImport}procedure ReleaseScalar__(DataTypeHandle: System.Pointer;ValuePtr: System.Pointer); cdecl; external uCalcDLL name 'ReleaseScalar';
{DLLImport}procedure ResetArray__(DataTypeHandle: System.Pointer;ValuePtr: System.Pointer); cdecl; external uCalcDLL name 'ResetArray';
{DLLImport}procedure ResetScalar__(DataTypeHandle: System.Pointer;ValuePtr: System.Pointer); cdecl; external uCalcDLL name 'ResetScalar';
{DLLImport}procedure SetArray__(DataTypeHandle: System.Pointer;Destination: System.Pointer ; Source: System.Pointer ; Index: Int32); cdecl; external uCalcDLL name 'SetArray';
{DLLImport}procedure SetScalar__(DataTypeHandle: System.Pointer;Destination: System.Pointer ; Source: System.Pointer); cdecl; external uCalcDLL name 'SetScalar';
{DLLImport}procedure SwapScalarValues__(DataTypeHandle: System.Pointer;ValuePtrA: System.Pointer ; ValuePtrB: System.Pointer); cdecl; external uCalcDLL name 'SwapScalarValues';
{DLLImport}function ToString_Overload_DataType_ByPtr__(DataTypeHandle: System.Pointer;ValuePtr: System.Pointer):  PAnsiChar; cdecl; external uCalcDLL name 'ToString_Overload_DataType_ByPtr';
{DLLImport}function ToString_Overload_DataType_ByStr__(DataTypeHandle: System.Pointer;Value: AnsiString):  PAnsiChar; cdecl; external uCalcDLL name 'ToString_Overload_DataType_ByStr';
{DLLImport}function Handle_Overload_DataType__(DataTypeHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'Handle_Overload_DataType';
{DLLImport}function AddErrorHandler__(uCalcHandle: System.Pointer;ErrorHandlerAddress: System.Pointer ; Position: Int32): System.Pointer; cdecl; external uCalcDLL name 'AddErrorHandler';
{DLLImport}function CreateuCalc__(uCalcHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'CreateuCalc';
{DLLImport}function DataTypeOfIndex__(uCalcHandle: System.Pointer;Index: BuiltInTypesEnum): System.Pointer; cdecl; external uCalcDLL name 'DataTypeOfIndex';
{DLLImport}function Define__(uCalcHandle: System.Pointer;Expression: AnsiString ; Address: System.Pointer  ; ExistingItem: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'Define';
{DLLImport}function DefineConstant__(uCalcHandle: System.Pointer;Definition: AnsiString ; VariableAddress: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'DefineConstant';
{DLLImport}function DefineDataType__(uCalcHandle: System.Pointer;TypeSize: Int32 ; AllocArrayValue: System.Pointer ; AllocScalar: System.Pointer ; AllocScalarByPtr: System.Pointer ; DeleteArray: System.Pointer ; DeleteScalar: System.Pointer ; ResetScalar: System.Pointer ; ResetArray: System.Pointer ; SetValueArray: System.Pointer ; SetValueScalar: System.Pointer ; SwapValues: System.Pointer ; ToString: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'DefineDataType';
{DLLImport}function DefineFunction__(uCalcHandle: System.Pointer;Definition: AnsiString ; FunctionAddress: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'DefineFunction';
{DLLImport}function DefineOperator__(uCalcHandle: System.Pointer;Definition: AnsiString ; iPrecedence: Int32 ; Grouping: GroupingEnum  ; FunctionAddress: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'DefineOperator';
{DLLImport}function DefineSyntax__(uCalcHandle: System.Pointer;Definition: AnsiString): System.Pointer; cdecl; external uCalcDLL name 'DefineSyntax';
{DLLImport}function DefineTransform__(uCalcHandle: System.Pointer;FromExpr: AnsiString ; ToExpr: AnsiString ; ItemHandle: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'DefineTransform';
{DLLImport}function DefineVariable__(uCalcHandle: System.Pointer;Definition: AnsiString ; VariableAddress: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'DefineVariable';
{DLLImport}function ErrorExpression__(uCalcHandle: System.Pointer):  PAnsiChar; cdecl; external uCalcDLL name 'ErrorExpression';
{DLLImport}function ErrorLocation__(uCalcHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'ErrorLocation';
{DLLImport}function ErrorMessage__(uCalcHandle: System.Pointer; Index: ErrorNumberEnum):  PAnsiChar; cdecl; external uCalcDLL name 'ErrorMessage';
{DLLImport}function ErrorNumber__(uCalcHandle: System.Pointer):  ErrorNumberEnum; cdecl; external uCalcDLL name 'ErrorNumber';
{DLLImport}procedure ErrorResponse__(uCalcHandle: System.Pointer;Response: ErrorHandlerResponseEnum); cdecl; external uCalcDLL name 'ErrorResponse';
{DLLImport}function ErrorSymbol__(uCalcHandle: System.Pointer):  PAnsiChar; cdecl; external uCalcDLL name 'ErrorSymbol';
{DLLImport}procedure ErrorSetExpression__(uCalcHandle: System.Pointer;Expression: AnsiString); cdecl; external uCalcDLL name 'ErrorSetExpression';
{DLLImport}function Eval__(uCalcHandle: System.Pointer;Expression: AnsiString):  Double; cdecl; external uCalcDLL name 'Eval';
{DLLImport}function EvalStr__(uCalcHandle: System.Pointer;Expression: AnsiString):  PAnsiChar; cdecl; external uCalcDLL name 'EvalStr';
{DLLImport}function Expand__(uCalcHandle: System.Pointer;Expression: AnsiString):  PAnsiChar; cdecl; external uCalcDLL name 'Expand';
{DLLImport}function ExpandSteps__(uCalcHandle: System.Pointer;Expression: AnsiString ; Separator: AnsiString):  PAnsiChar; cdecl; external uCalcDLL name 'ExpandSteps';
{DLLImport}function FPEnvClearException__(uCalcHandle: System.Pointer; Exceptions: Int32):  Int32; cdecl; external uCalcDLL name 'FPEnvClearException';
{DLLImport}function FPEnvGetExceptionMask__(uCalcHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'FPEnvGetExceptionMask';
{DLLImport}function FPEnvRaiseException__(uCalcHandle: System.Pointer; Exception: FPEnvExceptionEnum):  Int32; cdecl; external uCalcDLL name 'FPEnvRaiseException';
{DLLImport}function FPEnvGetRounding__(uCalcHandle: System.Pointer):  FPEnvRoundingEnum; cdecl; external uCalcDLL name 'FPEnvGetRounding';
{DLLImport}procedure FPEnvSetExceptionMask__(uCalcHandle: System.Pointer;Exceptions: Int32); cdecl; external uCalcDLL name 'FPEnvSetExceptionMask';
{DLLImport}procedure FPEnvSetExceptionUnMask__(uCalcHandle: System.Pointer;Exceptions: Int32); cdecl; external uCalcDLL name 'FPEnvSetExceptionUnMask';
{DLLImport}function FPEnvSetRounding__(uCalcHandle: System.Pointer;Direction: FPEnvRoundingEnum):  Int32; cdecl; external uCalcDLL name 'FPEnvSetRounding';
{DLLImport}procedure FPEnvToggleExceptionMask__(uCalcHandle: System.Pointer;Exception: FPEnvExceptionEnum); cdecl; external uCalcDLL name 'FPEnvToggleExceptionMask';
{DLLImport}function HandleOf__(uCalcHandle: System.Pointer;ItemName: AnsiString;  Properties: Int64 ;  SkipExpansion: Integer):  System.Pointer; cdecl; external uCalcDLL name 'HandleOf';
{DLLImport}function GetItemOf__(uCalcHandle: System.Pointer;ItemName: AnsiString ; Properties: Int64  ; SkipExpansion: Integer): System.Pointer; cdecl; external uCalcDLL name 'GetItemOf';
{DLLImport}function GetItemOfProperty__(uCalcHandle: System.Pointer;Property_: ItemIsEnum ; nth: Int32 ; DataTypeName: AnsiString): System.Pointer; cdecl; external uCalcDLL name 'GetItemOfProperty';
{DLLImport}function License__(uCalcHandle: System.Pointer;LicenseType: LicenseTypeEnum;  L1: Int32 ;  L2: Int32 ;  L3: Int32 ;  L4: Int32 ;  L5: Int32):  Int32; cdecl; external uCalcDLL name 'License';
{DLLImport}function NamespaceChild__(uCalcHandle: System.Pointer; nth: Int32): System.Pointer; cdecl; external uCalcDLL name 'NamespaceChild';
{DLLImport}function NamespaceChildCount__(uCalcHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'NamespaceChildCount';
{DLLImport}function NamespaceParent__(uCalcHandle: System.Pointer; Level: Int32): System.Pointer; cdecl; external uCalcDLL name 'NamespaceParent';
{DLLImport}function NamespaceSymbol__(uCalcHandle: System.Pointer;nth: Int32): System.Pointer; cdecl; external uCalcDLL name 'NamespaceSymbol';
{DLLImport}function NamespaceSymbolCount__(uCalcHandle: System.Pointer):  Int32; cdecl; external uCalcDLL name 'NamespaceSymbolCount';
{DLLImport}function Parse__(uCalcHandle: System.Pointer;Expression: AnsiString ; DataType: AnsiString): System.Pointer; cdecl; external uCalcDLL name 'Parse';
{DLLImport}function Precedence_Overload_uCalc__(uCalcHandle: System.Pointer;Operator1: AnsiString;  Operator2: AnsiString):  Int32; cdecl; external uCalcDLL name 'Precedence_Overload_uCalc';
{DLLImport}function Properties__(uCalcHandle: System.Pointer;p1: ItemIsEnum;  p2: ItemIsEnum ;  p3: ItemIsEnum ;  p4: ItemIsEnum):  Int64; cdecl; external uCalcDLL name 'Properties';
{DLLImport}procedure ReleaseuCalc__(uCalcHandle: System.Pointer); cdecl; external uCalcDLL name 'ReleaseuCalc';
{DLLImport}procedure ResetOutput__(uCalcHandle: System.Pointer); cdecl; external uCalcDLL name 'ResetOutput';
{DLLImport}function Default__(uCalcHandle: System.Pointer): System.Pointer; cdecl; external uCalcDLL name 'Default';
{DLLImport}function SetOutput_Overload_uCalc_Callback__(uCalcHandle: System.Pointer;FunctionAddress: System.Pointer ; Properties: AnsiString): System.Pointer; cdecl; external uCalcDLL name 'SetOutput_Overload_uCalc_Callback';
{DLLImport}function SetOutput_Overload_uCalc_NoCallback__(uCalcHandle: System.Pointer;Definition: AnsiString): System.Pointer; cdecl; external uCalcDLL name 'SetOutput_Overload_uCalc_NoCallback';
{DLLImport}function ValueAt__(uCalcHandle: System.Pointer;ValuePtr: System.Pointer; DataType: AnsiString):  PAnsiChar; cdecl; external uCalcDLL name 'ValueAt';
{DLLImport}function Handle_Overload_uCalc__(uCalcHandle: System.Pointer):  System.Pointer; cdecl; external uCalcDLL name 'Handle_Overload_uCalc';
type


/// <summary> uCalc main class</summary>
uCalc = class public

/// <summary> Handle for uCalc instance</summary>
var uCalcHandle: System.Pointer;
type
ParsedExpr = class;
Item = class;
Callback = class;
DataType = class;


/// <summary> Class for evaluating an already parsed expression or executing parsed code</summary>
ParsedExpr = class public

var ExprHandle: System.Pointer;




    public function Evaluate(): Double; overload;

    public function EvaluateInt32(): Int32; overload;

    public function EvaluateStr(): AnsiString; overload;

    public function EvaluateVoid(): System.Pointer; overload;

    public procedure Execute(); overload;

    public procedure ReleaseExpr(); overload;

    public function Handle(): System.Pointer; overload;

    public procedure Release();

    public constructor Create; overload;
    public constructor Create(ExpressionHandle: System.Pointer); overload;
    public constructor Create(ParsedExprInstance: ParsedExpr); overload;
end; 


/// <summary> Class for handling uCalc items</summary>
Item = class public

/// <summary> Handle for a uCalc.Item</summary>
var ItemHandle: System.Pointer;

    public function ElementCount(): Int32; overload;

    public function GetDataType(): DataType; overload;

    public function FunctionAddress(): System.Pointer; overload;

    public function IsProperty(ItemProperty: ItemIsEnum): Boolean; overload;

    public function Name(): AnsiString; overload;

    public function NextOverload(): Item; overload;

    public function Precedence(): Int32; overload;

    public procedure ReleaseItem(); overload;

    public procedure Rename(NewName: AnsiString); overload;

    public function ReplacementText(): AnsiString; overload;

    public procedure SetVariableValue(Value: Double); overload;

    public procedure SetVariableValueAny(Value: AnsiString); overload;

    public procedure SetVariableValueByte(Value: Byte); overload;

    public procedure SetVariableValueInt16(Value: Int16); overload;

    public procedure SetVariableValueInt32(Value: Int32); overload;

    public procedure SetVariableValueInt64(Value: Int64); overload;

    public procedure SetVariableValueSng(Value: Single); overload;

    public procedure SetVariableValueDbl(Value: Double); overload;

    public procedure SetVariableValueStr(Value: AnsiString); overload;

    public procedure SetVariableValuePtr(Value: System.Pointer); overload;

    public procedure SetVariableValueByPtr(Value: System.Pointer); overload;

    public function Text(): AnsiString; overload;

    public function GetuCalc(): uCalc; overload;

    public function Value(): Double; overload;

    public function ValueByte(): Byte; overload;

    public function ValueInt16(): Int16; overload;

    public function ValueInt32(): Int32; overload;

    public function ValueInt64(): Int64; overload;

    public function ValueSng(): Single; overload;

    public function ValueDbl(): Double; overload;

    public function ValueStr(): AnsiString; overload;

    public function ValuePtr(): System.Pointer; overload;

    public function ValueAddr(): System.Pointer; overload;

    public function Handle(): System.Pointer; overload;

    public procedure Release();

    public constructor Create; overload;
    public constructor Create(PassedItemHandle: System.Pointer); overload;
    public constructor Create(ItemHandleInstance: Item); overload;
end; 




/// <summary> Class for retrieving arguments passed to a callback, returning a value, and more</summary>
Callback = class public

/// <summary> Handle of the expression part the current callback instance is refering to</summary>
var ExprPartHandle: System.Pointer;

    public function Arg(Index: Int32): Double; overload;

    public function ArgCount(): Int32; overload;

    public function ArgObj(Index: Int32): Item; overload;

    public function ArgAddr(Index: Int32): System.Pointer; overload;

    public function ArgExpr(Index: Int32): ParsedExpr; overload;

    public function ArgInt32(Index: Int32): Int32; overload;

    public function ArgInt64(Index: Int32): Int64; overload;

    public function ArgPtr(Index: Int32): System.Pointer; overload;

    public function ArgStr(Index: Int32): AnsiString; overload;

    public procedure Return(Value: Double); overload;

    public procedure ReturnInt32(Value: Int32); overload;

    public procedure ReturnInt64(Value: Int64); overload;

    public procedure ReturnPtr(Value: System.Pointer); overload;

    public procedure ReturnStr(Value: AnsiString); overload;

    public function ErrorRaise(ErrNumber: ErrorNumberEnum): Int32; overload;

    public function ErrorRaiseMessage(ErrMessage: AnsiString): Int32; overload;

    public function GetuCalc(): uCalc; overload;

    public function Handle(): System.Pointer; overload;

    public function OpItem(): Item; overload;

    public constructor Create; overload;
    public constructor Create(ExpressionHandle: System.Pointer); overload;
    public constructor Create(ExprInstance: Callback); overload;
end; 


/// <summary> Class for working with uCalc data types</summary>
DataType = class public

/// <summary> Handle for uCalc Data Type</summary>
var DataTypeHandle: System.Pointer;

    public function AllocArray(ArraySize: Int32): System.Pointer; overload;

    public function AllocByPtr(ValuePtr: System.Pointer): System.Pointer; overload;

    public function AllocScalar(Value: AnsiString = ''): System.Pointer; overload;

    public function ByteSize(): Int32; overload;

    public function Index(): BuiltInTypesEnum; overload;

    public function GetItem(): Item; overload;

    public function Name(): AnsiString; overload;

    public procedure ReleaseArray(ValuePtr: System.Pointer); overload;

    public procedure ReleaseScalar(ValuePtr: System.Pointer); overload;

    public procedure ResetArray(ValuePtr: System.Pointer); overload;

    public procedure ResetScalar(ValuePtr: System.Pointer); overload;

    public procedure SetArray(Destination: System.Pointer ; Source: System.Pointer ; Index: Int32); overload;

    public procedure SetScalar(Destination: System.Pointer ; Source: System.Pointer); overload;

    public procedure SwapScalarValues(ValuePtrA: System.Pointer ; ValuePtrB: System.Pointer); overload;

    public function ToString(ValuePtr: System.Pointer): AnsiString; overload;

    public function ToString(Value: AnsiString): AnsiString; overload;

    public function Handle(): System.Pointer; overload;
    public constructor Create; overload;
    public constructor Create(PassedDataTypeHandle: System.Pointer); overload;
    public constructor Create(DataTypeHandleInstance: DataType); overload;
end; 


    public function AddErrorHandler(ErrorHandlerAddress: System.Pointer ;  Position: Int32  = 0): Item; overload;

    public function CreateuCalc(): System.Pointer; overload;

    public function DataTypeOfIndex(Index: BuiltInTypesEnum): DataType; overload;

    public function Define(Expression: AnsiString ;  Address: System.Pointer = nil;  ExistingItem: System.Pointer = nil): Item; overload;

    public function DefineConstant(Definition: AnsiString ;  VariableAddress: System.Pointer = nil): Item; overload;

    public function DefineDataType(TypeSize: Int32 ; AllocArrayValue: System.Pointer ; AllocScalar: System.Pointer ; AllocScalarByPtr: System.Pointer ; DeleteArray: System.Pointer ; DeleteScalar: System.Pointer ; ResetScalar: System.Pointer ; ResetArray: System.Pointer ; SetValueArray: System.Pointer ; SetValueScalar: System.Pointer ; SwapValues: System.Pointer ; ToString: System.Pointer): Item; overload;

    public function DefineFunction(Definition: AnsiString ;  FunctionAddress: System.Pointer = nil): Item; overload;

    public function DefineOperator(Definition: AnsiString ; iPrecedence: Int32 ;  Grouping: GroupingEnum = GroupingEnum.GroupLeftToRight;  FunctionAddress: System.Pointer = nil): Item; overload;

    public function DefineSyntax(Definition: AnsiString): Item; overload;

    public function DefineTransform(FromExpr: AnsiString ; ToExpr: AnsiString ;  ItemHandle: System.Pointer = nil): Item; overload;

    public function DefineVariable(Definition: AnsiString ;  VariableAddress: System.Pointer = nil): Item; overload;

    public function ErrorExpression(): AnsiString; overload;

    public function ErrorLocation(): Int32; overload;

    public function ErrorMessage(Index: ErrorNumberEnum = ErrorNumberEnum.Err_None): AnsiString; overload;

    public function ErrorNumber(): ErrorNumberEnum; overload;

    public procedure ErrorResponse(Response: ErrorHandlerResponseEnum); overload;

    public function ErrorSymbol(): AnsiString; overload;

    public procedure ErrorSetExpression(Expression: AnsiString); overload;

    public function Eval(Expression: AnsiString): Double; overload;

    public function EvalStr(Expression: AnsiString): AnsiString; overload;

    public function Expand(Expression: AnsiString): AnsiString; overload;

    public function ExpandSteps(Expression: AnsiString ;  Separator: AnsiString = ''): AnsiString; overload;

    public function FPEnvClearException(Exceptions: Int32  = 0): Int32; overload;

    public function FPEnvGetExceptionMask(): Int32; overload;

    public function FPEnvRaiseException(Exception: FPEnvExceptionEnum = FPEnvExceptionEnum.Invalid): Int32; overload;

    public function FPEnvGetRounding(): FPEnvRoundingEnum; overload;

    public procedure FPEnvSetExceptionMask(Exceptions: Int32); overload;

    public procedure FPEnvSetExceptionUnMask(Exceptions: Int32); overload;

    public function FPEnvSetRounding(Direction: FPEnvRoundingEnum): Int32; overload;

    public procedure FPEnvToggleExceptionMask(Exception: FPEnvExceptionEnum); overload;

    public function HandleOf(ItemName: AnsiString;   Properties: Int64 = 0;   SkipExpansion: Boolean = False): System.Pointer; overload;

    public function GetItemOf(ItemName: AnsiString ;  Properties: Int64  = 0;  SkipExpansion: Boolean = False): Item; overload;

    public function GetItemOfProperty(Property_: ItemIsEnum ; nth: Int32 ;  DataTypeName: AnsiString = ''): Item; overload;



    public function License(LicenseType: LicenseTypeEnum;   L1: Int32 = 0;   L2: Int32 = 0;   L3: Int32 = 0;   L4: Int32 = 0;   L5: Int32 = 0): Int32; overload;

    public function NamespaceChild(nth: Int32 = 0): Item; overload;

    public function NamespaceChildCount(): Int32; overload;

    public function NamespaceParent(Level: Int32 = 0): Item; overload;

    public function NamespaceSymbol(nth: Int32): Item; overload;

    public function NamespaceSymbolCount(): Int32; overload;

    public function Parse(Expression: AnsiString ;  DataType: AnsiString = ''): ParsedExpr; overload;

    public function Precedence(Operator1: AnsiString;   Operator2: AnsiString = ''): Int32; overload;

    public function Properties(p1: ItemIsEnum;   p2: ItemIsEnum = ItemIsEnum.AnyItem;   p3: ItemIsEnum = ItemIsEnum.AnyItem;   p4: ItemIsEnum = ItemIsEnum.AnyItem): Int64; overload;

    public procedure ReleaseuCalc(); overload;

    public procedure ResetOutput(); overload;

    public function Default(): Item; overload;

    public function SetOutput(FunctionAddress: System.Pointer ; Properties: AnsiString): Item; overload;

    public function SetOutput(Definition: AnsiString): Item; overload;

    public function ValueAt(ValuePtr: System.Pointer; DataType: AnsiString): AnsiString; overload;

    public function Handle(): System.Pointer; overload;

    public procedure Release();

    public procedure Reset();

    public constructor Create; overload;
    public constructor Create(uCalcInstance: uCalc); overload;
    public constructor Create(InstHandle: System.Pointer); overload;
    public constructor Create(LicenseType: LicenseTypeEnum;   L1: Int32 = 0;   L2: Int32 = 0;   L3: Int32 = 0;   L4: Int32 = 0;   L5: Int32 = 0); overload;
end; 

var uc: uCalc;
implementation


    /// <summary> Rapidly evaluates a floating point expression that was parsed with Parse or passed By Expression</summary>
/// <returns> The result of the evaluated floating point expression</returns>
/// <remarks><para>Evaluate can be used instead of Eval for faster results in time-critical loops where the same expression
/// is repeatedly re-evaluated, with the only change in each iteration typically being the value(s) of one or
/// more variables.  Evaluate works in pair with Parse, the relatively slower part, which is called first before
/// going into the time-critical loop.  The ParsedExpr object returned by Parse is what calls the Evaluate function.
/// </para><para>
/// Evaluate can also be called by a ParsedExpr object that was passed to a callback By Expression.
/// </para><para>
/// The expression must be one who&#39;s evaluation returns a floating point numeric value.  If the expression evaluates
/// to a non-numeric value (such as a string) or non-scalar numeric value (such as a complex number), or if a numeric
/// type other than the default was explicitely specified with Parse, the value returned will be invalid.  However,
/// an error will not be raised.
/// </para></remarks>
    function uCalc.ParsedExpr.Evaluate(): Double;
    begin
        Result :=  Evaluate__(ExprHandle);
    end;

    /// <summary> Rapidly evaluates expression that was parsed as integer with Parse or passed By Expression</summary>
/// <returns> The result of the evaluated integer expression</returns>
/// <remarks><para>
/// This is similar to Evaluate, but applies only to integer expressions.
/// </para></remarks>
    function uCalc.ParsedExpr.EvaluateInt32(): Int32;
    begin
        Result :=  EvaluateInt32__(ExprHandle);
    end;

    /// <summary> Rapidly evaluates an expression of any data type that was parsed with Parse or passed By Expression returning a string</summary>
/// <returns> The result of an evaluated expression of any data type, returned as a string</returns>
/// <remarks><para>EvaluateStr is similar to Evaluate.  However, EvaluateStr returns a value as a string.  Unlike Evaluate, which can only
/// return a floating point value, EvaluateStr can return a value of any data type.  Non-string values are converted to a string
/// before being returned.
/// </para></remarks>
    function uCalc.ParsedExpr.EvaluateStr(): AnsiString;
    begin
        Result :=  EvaluateStr__(ExprHandle);
    end;

    /// <summary> Rapidly evaluates an expression that was parsed with Parse or passed By Expression and returns a System.Pointer to the result</summary>
/// <returns> Returns a System.Pointer to the result instead of the result itself</returns>
/// <remarks><para>Use this when you are interested in a System.Pointer to the result instead of the result itself.  For instance, given an integer
/// result, you might use a System.Pointer in order to retrieve the hi byte and lo byte values.
/// </para></remarks>
    function uCalc.ParsedExpr.EvaluateVoid(): System.Pointer;
    begin
        Result :=  EvaluateVoid__(ExprHandle);
    end;

    /// <summary> Rapidly executes code that was parsed with Parse or passed By Expression for which a result is not required</summary>
/// <remarks><para>This can be used in cases where the reutrn value is not needed, such as statements that represent code to be executed
/// rather than a math expression.
/// </para></remarks>
    procedure uCalc.ParsedExpr.Execute();
    begin
          Execute__(ExprHandle);
    end;

    /// <summary> (Use Release instead)</summary>
/// <remarks><para>Use Release</para></remarks>
    procedure uCalc.ParsedExpr.ReleaseExpr();
    begin
          ReleaseExpr__(ExprHandle);
    end;

    /// <summary> Returns the handle of the parsed expression</summary>
/// <remarks><para>This numeric value represents the System.Pointer that uCalc uses internally for the ParsedExpr object.
/// If you are iterating through a series of objects, such as with NextOverload, you may check Handle()
/// to see when it becomes NULL (or 0) to know when to stop.</para></remarks>
    function uCalc.ParsedExpr.Handle(): System.Pointer;
    begin
        Result :=  Handle_Overload_ParsedExpr__(ExprHandle);
    end;

     /// <summary> Releases an expression that was parsed with Parse</summary>
/// <remarks><para>Whenever an expression is parsed with Parse, the parsed expression remains in memory until released.
/// When the parsed expression is no longer needed it should be released with Release to preserve memory.
/// Note: An expression parsed with Parse must explicitely be released with Release unless the uCalc
/// object it is in is released.  A parsed expression will not be released automatically when it simply
/// goes out of scope.
/// </para></remarks>
    procedure uCalc.ParsedExpr.Release();
    begin
        ReleaseExpr(); ExprHandle := 0;
    end;

    constructor uCalc.ParsedExpr.Create;
    begin
       ExprHandle := 0;
    end;

    constructor uCalc.ParsedExpr.Create(ExpressionHandle: System.Pointer);
    begin
       ExprHandle := ExpressionHandle;
    end;

    constructor uCalc.ParsedExpr.Create(ParsedExprInstance: ParsedExpr);
    begin
       ExprHandle := ParsedExprInstance.ExprHandle;
    end;

    /// <summary> Returns the number of elements in a uCalc.Item</summary>
/// <remarks><para>Depending on the kind of object, this may represent the number of elements in an array, the number of parameters in a function,
/// the number of operands in an operator, the number of parameters in a syntax construct, the number of parts in a transform,
/// the number of parts in a compound token, or the value explicitlely set in the Count property when calling Define.
/// </para><para>
/// When determining the number of elements, optional parameters in a function count as an element.  If the function
/// has a variable number of arguments, then count is -1.
/// </para><para>
/// The value returned for a syntax construct may not seem obvious, especially if it has optional and/or alternative
/// parts or some other complexity.
/// </para><para>
/// +++ the number of symbols in a uCalc
/// instance,</para></remarks>
    function uCalc.Item.ElementCount(): Int32;
    begin
        Result :=  ElementCount__(ItemHandle);
    end;

    /// <summary> Returns the data type of a uCalc.Item</summary>
/// <remarks><para>The returned data type object can be used to determine the type name, size, etc.</para></remarks>
    function uCalc.Item.GetDataType(): DataType;
    begin
        Result :=  uCalc.DataType.Create(GetDataType__(ItemHandle));
    end;

    /// <summary> Returns the callback address of a defined function (or operator)</summary>
/// <remarks><para>The returned value is the actual System.Pointer address of the function as used by uCalc internally, and not
/// a delegate, even in .NET.  As such, in .NET this value can be used in the more general Define function but
/// not functions like DefineFunction or DefineneOperator.</para></remarks>
    function uCalc.Item.FunctionAddress(): System.Pointer;
    begin
        Result :=  FunctionAddress__(ItemHandle);
    end;

    /// <summary> Determines if a uCalc.Item has the given properties</summary>
/// <param name="ItemProperty"> The property you want to check for</param>
/// <returns> Returns True if the uCalc.Item matches the given property, or False otherwise.</returns>
/// <remarks><para>This helps you determine whether the uCalc.Item is a function, operator, variable, data type,
/// literal value, etc, or has certain other special properties.  Note: Some items may have multiple
/// properties.  For instance the Function property may be set to true for arrays.
/// </para></remarks>
    function uCalc.Item.IsProperty(ItemProperty: ItemIsEnum): Boolean;
begin
  Result :=  IsProperty__(ItemHandle,ItemProperty) <> 0;
    end;

    /// <summary> Returns the name of the uCalc.Item</summary>
/// <remarks><para>Items such as functions, operators, variables, etc generally are given a name when they are defined.
/// That name can be retried with this function.</para></remarks>
    function uCalc.Item.Name(): AnsiString;
    begin
        Result :=  Name_Overload_Item__(ItemHandle);
    end;

    /// <summary> Returns next uCalc.Item object that shares the same name</summary>
/// <returns> Returns next uCalc.Item object that shares the same name, or a NULL (or 0) if there is none</returns>
/// <remarks><para>If several functions, operators, parameters, etc are overloaded -- that is they are defined with different
/// parameters but share the same name --, or if they (or variables) overshadow each other, this function returns
/// the next uCalc.Item object that shares the same name.</para></remarks>
    function uCalc.Item.NextOverload(): Item;
    begin
        Result :=  uCalc.Item.Create(NextOverload__(ItemHandle));
    end;

    /// <summary> Returns the precedence level of the uCalc.Item</summary>
/// <remarks><para>Operators, and some syntax constructs are defined with a precedence level.  When defining a new operator you
/// may sometimes want to give it the same level as a pre-existing operator.</para></remarks>
    function uCalc.Item.Precedence(): Int32;
    begin
        Result :=  Precedence_Overload_Item__(ItemHandle);
    end;

    /// <summary> (Use Release instead)</summary>
/// <remarks><para>Use Release</para></remarks>
    procedure uCalc.Item.ReleaseItem();
    begin
          ReleaseItem__(ItemHandle);
    end;

    /// <summary> Renames an item</summary>
/// <param name="NewName"> The new name of the item</param>
/// <remarks><para>
/// Any defined item that has a name (such as a variable, constant, function, operator, etc) can be renamed with this function.
/// </para></remarks>
    procedure uCalc.Item.Rename(NewName: AnsiString);
    begin
          Rename__(ItemHandle,NewName);
    end;

    /// <summary> Returns the replacement part of a syntax construct defined with DefineSyntax</summary>
/// <returns> Returns the part to the right of &quot;::=&quot; in a syntax construct defined with DefineSyntax</returns>
/// <remarks><para>This allows you to inspect the part to the right of &quot;::=&quot; in a syntax construct defined with DefineSyntax.</para></remarks>
    function uCalc.Item.ReplacementText(): AnsiString;
    begin
        Result :=  ReplacementText__(ItemHandle);
    end;

    /// <summary> Sets the value of a floating point variable +++ maybe SetValue instead of SetVariableValue</summary>
/// <param name="Value"> Double precision floating point value that the variable will be set to</param>
/// <remarks><para>When a floating point variable is defined with DefineVariable, you can later set a value for this variable using
/// SetVariableValue.  Note: If your compiler supports pointers, you can define a uCalc variable that is directly attached
/// to the address of a variable in your source code, and not have to use SetVariableValue.</para></remarks>
    procedure uCalc.Item.SetVariableValue(Value: Double);
    begin
          SetVariableValue__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a variable of any data type by passing a string</summary>
/// <param name="Value"> String containing the value that will be converted and stored in the variable</param>
/// <remarks><para>This lets you set the value of a variable defined with DefineVariable (or Define) regardless of the variable&#39;s data type.
/// The value is passed as a string, and internally converted to the appropriate data type.  Note: Because a conversion takes
/// place with SetVariableValueAny, it is typically slower than calling SetVariableValue for floating point, or one of the
/// other versions of SetVariableValue specific to a data type.</para></remarks>
    procedure uCalc.Item.SetVariableValueAny(Value: AnsiString);
    begin
          SetVariableValueAny__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a Byte variable</summary>
/// <param name="Value"> Byte value that the variable will be set to</param>
/// <remarks><para>This lets you set the value of a Byte variable defined with DefineVariable (or Define)</para></remarks>
    procedure uCalc.Item.SetVariableValueByte(Value: Byte);
    begin
          SetVariableValueByte__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a 16-bit integer variable</summary>
/// <param name="Value"> 16-bit integer value that the variable will be set to</param>
/// <remarks><para>This lets you set the value of a 16-bit integer variable defined with DefineVariable (or Define).  This works for
/// both signed and unsigned 16-bit integers.</para></remarks>
    procedure uCalc.Item.SetVariableValueInt16(Value: Int16);
    begin
          SetVariableValueInt16__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a 32-bit integer variable</summary>
/// <param name="Value"> 32-bit integer value that the variable will be set to</param>
/// <remarks><para>This lets you set the value of a 32-bit integer variable defined with DefineVariable (or Define).  This works for
/// both signed and unsigned 32-bit integers.</para></remarks>
    procedure uCalc.Item.SetVariableValueInt32(Value: Int32);
    begin
          SetVariableValueInt32__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a 64-bit integer variable</summary>
/// <param name="Value"> 64-bit integer value that the variable will be set to</param>
/// <remarks><para>This lets you set the value of a 64-bit integer variable defined with DefineVariable (or Define).  This works for
/// both signed and unsigned 64-bit integers.</para></remarks>
    procedure uCalc.Item.SetVariableValueInt64(Value: Int64);
    begin
          SetVariableValueInt64__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a single precision floating point variable</summary>
/// <param name="Value"> Single precision value that the variable will be set to</param>
/// <remarks><para>This lets you set the value of a single precision variable defined with DefineVariable (or Define).</para></remarks>
    procedure uCalc.Item.SetVariableValueSng(Value: Single);
    begin
          SetVariableValueSng__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a Double precision floating point variable</summary>
/// <param name="Value"> Double precision floating point value that the variable will be set to</param>
/// <remarks><para>SetVariableValueDbl is synonymous with SetVariableValue</para></remarks>
    procedure uCalc.Item.SetVariableValueDbl(Value: Double);
    begin
          SetVariableValueDbl__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a string variable</summary>
/// <param name="Value"> String value that the variable will be set to</param>
/// <remarks><para>This lets you set the value of a string variable defined with DefineVariable (or Define).</para></remarks>
    procedure uCalc.Item.SetVariableValueStr(Value: AnsiString);
    begin
          SetVariableValueStr__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a System.Pointer variable</summary>
/// <param name="Value"> System.Pointer that the variable will be set to</param>
/// <remarks><para>This lets you set the value of a System.Pointer variable defined with DefineVariable (or Define).  Note: This does not
/// change the System.Pointer of the variable.  It simply stores a System.Pointer value in the variable.</para></remarks>
    procedure uCalc.Item.SetVariableValuePtr(Value: System.Pointer);
    begin
          SetVariableValuePtr__(ItemHandle,Value);
    end;

    /// <summary> Sets the value of a variable to a value who&#39;s System.Pointer is passed</summary>
/// <param name="Value"> System.Pointer to the value that the variable will be set to</param>
/// <remarks><para>This lets you set the value of a variable by passing the System.Pointer to the value you want to set the variable to.</para></remarks>
    procedure uCalc.Item.SetVariableValueByPtr(Value: System.Pointer);
    begin
          SetVariableValueByPtr__(ItemHandle,Value);
    end;

    /// <summary> Returns the text data associated with a uCalc.Item</summary>
/// <remarks><para>Certain items, such as a syntax construct defined with DefineSyntax have a text property that can be retrieved
/// with a call to this function.</para></remarks>
    function uCalc.Item.Text(): AnsiString;
    begin
        Result :=  Text__(ItemHandle);
    end;

    /// <summary> Returns the uCalc object that the uCalc.Item belongs to</summary>
/// <remarks><para>A uCalc.Item is defined in the context of a uCalc object.  You can later use this function to determine
/// the uCalc object the defined item belongs to.</para></remarks>
    function uCalc.Item.GetuCalc(): uCalc;
    begin
        Result :=  uCalc.Create(GetuCalc_Overload_Item__(ItemHandle));
    end;

    /// <summary> Returns the value (default Double precision) associated with a uCalc.Item</summary>
/// <remarks><para>Use this to retrieve the Double precision value associated with a variable.</para></remarks>
    function uCalc.Item.Value(): Double;
    begin
        Result :=  Value__(ItemHandle);
    end;

    /// <summary> Returns the value (default Double precision) associated with a uCalc.Item</summary>
/// <remarks><para>Use this to retrieve the Byte value associated with a variable.</para></remarks>
    function uCalc.Item.ValueByte(): Byte;
    begin
        Result :=  ValueByte__(ItemHandle);
    end;

    /// <summary> Returns the 16-bit integer value associated with a uCalc.Item</summary>
/// <remarks><para>Use this to retrieve the Int16 value associated with a variable.</para></remarks>
    function uCalc.Item.ValueInt16(): Int16;
    begin
        Result :=  ValueInt16__(ItemHandle);
    end;

    /// <summary> Returns the 32-bit integer value associated with a uCalc.Item</summary>
/// <remarks><para>Use this to retrieve the Int32 value associated with a variable.</para></remarks>
    function uCalc.Item.ValueInt32(): Int32;
    begin
        Result :=  ValueInt32__(ItemHandle);
    end;

    /// <summary> Returns the 64-bit integer value associated with a uCalc.Item</summary>
/// <remarks><para>Use this to retrieve the Int64 value associated with a variable.</para></remarks>
    function uCalc.Item.ValueInt64(): Int64;
    begin
        Result :=  ValueInt64__(ItemHandle);
    end;

    /// <summary> Returns the Double precision value associated with a uCalc.Item</summary>
/// <remarks><para>Use this to retrieve the Single precision value associated with a variable.</para></remarks>
    function uCalc.Item.ValueSng(): Single;
    begin
        Result :=  ValueSng__(ItemHandle);
    end;

    /// <summary> Returns the Double precision value associated with a uCalc.Item</summary>
/// <remarks><para>Synonymous with Value</para></remarks>
    function uCalc.Item.ValueDbl(): Double;
    begin
        Result :=  ValueDbl__(ItemHandle);
    end;

    /// <summary> Returns the String value associated with a uCalc.Item</summary>
/// <remarks><para>Use this to retrieve the String value associated with a variable.
/// </para><para>
/// ValueStr has a special property beyond the Value function for other data types in that it can return the value
/// of an item regardless of what data type it has.  The value is returned as a string.
/// </para></remarks>
    function uCalc.Item.ValueStr(): AnsiString;
    begin
        Result :=  ValueStr__(ItemHandle);
    end;

    /// <summary> Returns the System.Pointer value associated with a uCalc.Item</summary>
/// <remarks><para>Use this to retrieve the System.Pointer value associated with a variable.
/// Note: If the uCalc.Item is a Variable of type System.Pointer, then ValuePtr returns a System.Pointer value
/// that is stored in that variable, whereas ValueAddr returns the System.Pointer of the variable itself.  In other words
/// ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any value you may
/// have set as the address the variable points to.</para></remarks>
    function uCalc.Item.ValuePtr(): System.Pointer;
    begin
        Result :=  ValuePtr__(ItemHandle);
    end;

    /// <summary> Returns the System.Pointer address of the value associated with a uCalc.Item</summary>
/// <remarks><para>Use this to retrieve the System.Pointer address of the value associated with a variable (instead of
/// the value itself).  Note: If the uCalc.Item is a Variable of type System.Pointer, then ValuePtr returns a System.Pointer value
/// that is stored in that variable, whereas ValueAddr returns the System.Pointer of the variable itself.  In other words
/// ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any given address
/// you may have stored as the value of the variable.</para></remarks>
    function uCalc.Item.ValueAddr(): System.Pointer;
    begin
        Result :=  ValueAddr__(ItemHandle);
    end;

    /// <summary> Returns the handle of the current uCalc.Item object</summary>
    function uCalc.Item.Handle(): System.Pointer;
    begin
        Result :=  Handle_Overload_Item__(ItemHandle);
    end;

     /// <summary> Releases an Item defined with Define or other related function</summary>
/// <remarks><para>When a function, operator, variable, or other item is defined with such functions as
/// Define, DefineFunction, DefineOperator, DefineVariable, these items remain in memory, until they are
/// released with this function.  Items should generally be released when they are no longer needed, in order
/// to preserve memory.
/// </para><para>
/// Note: A uCalc Item object must explicitely be released with Release unless the entire uCalc
/// object it is a part of is is released.  An Item object will not be released automatically when it
/// simply goes out of scope.
/// </para></remarks>
    procedure uCalc.Item.Release();
    begin
        ReleaseItem(); ItemHandle := 0;
    end;

    constructor uCalc.Item.Create;
    begin
       ItemHandle := 0;
    end;

    constructor uCalc.Item.Create(PassedItemHandle: System.Pointer);
    begin
       ItemHandle := PassedItemHandle;
    end;

    constructor uCalc.Item.Create(ItemHandleInstance: Item);
    begin
       ItemHandle := ItemHandleInstance.ItemHandle;
    end;

    /// <summary> Returns the value of an argument passed to a uCalc.Callback function</summary>
/// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
/// <remarks><para>When the name of a callback function is specified in DefineFunction or DefineOperator, that
/// callback function will receive arguments uCalc passes to it through the Arg.  Arg handles values
/// of the default Double floating point data type.  For other data types, ArgInt32, ArgStr, etc instead.
/// </para><para>
/// Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be
/// 1 (not 0), and the second and third will be 2 and 3 respectively.
/// </para><para>
/// If you are defining a binary operator, 1 is the index for the left operand, and 2 for the right operand.
/// For a unary operator, the index is 1.
/// </para></remarks>
    function uCalc.Callback.Arg(Index: Int32): Double;
    begin
        Result :=  Arg__(ExprPartHandle,Index);
    end;

    /// <summary> Returns the number of arguments being passed to a uCalc.Callback function</summary>
/// <returns> 0 if no arg is passed; or a positive integer representing the number of args being passed.</returns>
/// <remarks><para>This is useful for functions that have a variable number of arguments</para></remarks>
    function uCalc.Callback.ArgCount(): Int32;
    begin
        Result :=  ArgCount__(ExprPartHandle);
    end;

    /// <summary> Returns an object instance of the argument being passed to a uCalc.Callback function</summary>
/// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
/// <returns> The returned object can be used (where relevent) just like any other uCalc.Item</returns>
/// <remarks><para>Each argument is a uCalc.Item, just like anything else defined with Define.  With this object,
/// you can do such things as determine the argument&#39;s data type, determine which instance of uCalc
/// it belongs to, retrieve the System.Pointer of the value of the argument, etc.</para></remarks>
    function uCalc.Callback.ArgObj(Index: Int32): Item;
    begin
        Result :=  uCalc.Item.Create(ArgObj__(ExprPartHandle,Index));
    end;

    /// <summary> Returns the address of the argument value that is being passed</summary>
/// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
/// <remarks><para>In addition to Arg which returns arguments that are of the default Double floating point numeric type, there
/// are specialized functions for other datatypes such as ArgInt32, ArgInt64, etc.  However, if the argument is of
/// a data type that does not have a specialized function, you can retrieve the address of the argument value as
/// a System.Pointer, and use that to retrieve the value.</para></remarks>
    function uCalc.Callback.ArgAddr(Index: Int32): System.Pointer;
    begin
        Result :=  ArgAddr__(ExprPartHandle,Index);
    end;

    /// <summary> Returns the object instance of the expression that&#39;s passed as an argument to a uCalc.Callback function+++Example</summary>
/// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
/// <returns> Returns parsed expression object with which you can call functions like Evaluate or Execute</returns>
/// <remarks><para>uCalc allows you to not only pass arguments by value or by reference.  But it also lets you pass an argument
/// by expression.  When an expression is passed By Expression, the expresison is not evaluated before being passed.
/// Instead a handle to the parsed expression is passed, allowing the callback to perform the evaluation.</para></remarks>
    function uCalc.Callback.ArgExpr(Index: Int32): ParsedExpr;
    begin
        Result :=  uCalc.ParsedExpr.Create(ArgExpr__(ExprPartHandle,Index));
    end;

    /// <summary> Returns the Int32 integer argument passed to a uCalc.Callback function</summary>
/// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
/// <remarks><para>If a function parameter(or operator operand) was explicitely defined as Int32, then the callback function can
/// recieve it through ArgInt32.  If the argument is of a different but compatible numeric type, a conversion takes
/// place first before being passed.</para></remarks>
    function uCalc.Callback.ArgInt32(Index: Int32): Int32;
    begin
        Result :=  ArgInt32__(ExprPartHandle,Index);
    end;

    /// <summary> Returns the Int64 integer argument being passed to a uCalc.Callback function</summary>
/// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
/// <remarks><para>If a function parameter(or operator operand) was explicitely defined as Int64, then the callback function can
/// recieve it through ArgInt64.  If the argument is of a different but compatible numeric type, a conversion takes
/// place first before being passed.</para></remarks>
    function uCalc.Callback.ArgInt64(Index: Int32): Int64;
    begin
        Result :=  ArgInt64__(ExprPartHandle,Index);
    end;

    /// <summary> Returns a System.Pointer value being passed as an argument to a uCalc.Callback function</summary>
/// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
/// <remarks><para>If a function parameter(or operator operand) was explicitely defined as a System.Pointer, then the callback function can
/// recieve it through ArgPtr.  Note that ArgPtr returns any address value that is being sent, whereas ArgAddr returns the
/// address of the value being passed.</para></remarks>
    function uCalc.Callback.ArgPtr(Index: Int32): System.Pointer;
    begin
        Result :=  ArgPtr__(ExprPartHandle,Index);
    end;

    /// <summary> Returns the value being passed as argument in to a uCalc.Callback function in form of a string</summary>
/// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
/// <remarks><para>In addition to returning an argument that was passed as a string, ArgStr has the special property of returning an argument that
/// was passed as any data type.  If the argument was not a string, ArgStr converts it to a string before returning its value.</para></remarks>
    function uCalc.Callback.ArgStr(Index: Int32): AnsiString;
    begin
        Result :=  ArgStr__(ExprPartHandle,Index);
    end;

    /// <summary> Sets the return value (default floating point) of a uCalc.Callback function</summary>
/// <param name="Value"> Floating point value that your callback returns</param>
/// <remarks><para>When you define a callback function with DefineFunction and you want to return a floating point number,
/// do it with Return.
/// </para><para>
/// Note: For compilers in which Return is a reserved keyword, use ReturnDbl instead.
/// </para></remarks>
    procedure uCalc.Callback.Return(Value: Double);
    begin
          Return__(ExprPartHandle,Value);
    end;

    /// <summary> Sets the return value of a uCalc.Callback function as an Int32 integer</summary>
/// <param name="Value"> Integer (32-bit) that your callback returns</param>
/// <remarks><para>When you define a callback function with DefineFunction and you want to return a Int32,
/// do it with ReturnInt32.</para></remarks>
    procedure uCalc.Callback.ReturnInt32(Value: Int32);
    begin
          ReturnInt32__(ExprPartHandle,Value);
    end;

    /// <summary> Sets the return value of a uCalc.Callback function as an Int64 integer</summary>
/// <param name="Value"> Integer (64-bit) that your callback returns</param>
/// <remarks><para>When you define a callback function with DefineFunction and you want to return a Int64,
/// do it with ReturnInt64.</para></remarks>
    procedure uCalc.Callback.ReturnInt64(Value: Int64);
    begin
          ReturnInt64__(ExprPartHandle,Value);
    end;

    /// <summary> Sets the return value of a uCalc.Callback function as a System.Pointer</summary>
/// <param name="Value"> System.Pointer value that your callback returns</param>
/// <remarks><para>When you define a callback function with DefineFunction that returns a System.Pointer value,
/// do it with ReturnPtr.</para></remarks>
    procedure uCalc.Callback.ReturnPtr(Value: System.Pointer);
    begin
          ReturnPtr__(ExprPartHandle,Value);
    end;

    /// <summary> Sets the return value of a uCalc.Callback function as a String</summary>
/// <param name="Value"> String value that your callback returns</param>
/// <remarks><para>When you define a callback function with DefineFunction that returns a string,
/// do it with ReturnStr.</para></remarks>
    procedure uCalc.Callback.ReturnStr(Value: AnsiString);
    begin
          ReturnStr__(ExprPartHandle,Value);
    end;

    /// <summary> Raises an error in a uCalc.Callback function</summary>
/// <param name="ErrNumber"> Index number of error to raise</param>
/// <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
/// <remarks><para>Your callback can raise an error by calling this funciton with one of the values of the
/// ErrorNumberEnum enumerator.  End-user function definitions can also raise an error by calling Error().
/// The return value is determined by error handlers that you may have added.  ErrorAbort (value of 0)
/// is returned by default if there are no error handlers.</para></remarks>
    function uCalc.Callback.ErrorRaise(ErrNumber: ErrorNumberEnum): Int32;
    begin
        Result :=  ErrorRaise__(ExprPartHandle,ErrNumber);
    end;

    /// <summary> Raises an error with a user-defined message in a uCalc.Callback function</summary>
/// <param name="ErrMessage"> Text of custom error message to return</param>
/// <returns> Returns ErrorAbort, ErrorResume, or ErrorReRaise.</returns>
/// <remarks><para>Your callback can raise a custom error message (one not listed as a ErrorNumberEnum enumerator)
/// by simply passing the text of the message to this function.</para></remarks>
    function uCalc.Callback.ErrorRaiseMessage(ErrMessage: AnsiString): Int32;
    begin
        Result :=  ErrorRaiseMessage__(ExprPartHandle,ErrMessage);
    end;

    /// <summary> Returns the uCalc instance the current expression part belongs to</summary>
/// <remarks><para>A callback is designed to work with all instances of uCalc.  However, at times you may need to know
/// which particular instance of uCalc the expression part being passed belongs to.</para></remarks>
    function uCalc.Callback.GetuCalc(): uCalc;
    begin
        Result :=  uCalc.Create(GetuCalc_Overload_Callback__(ExprPartHandle));
    end;

    /// <summary> Returns the handle of the sub expression passed to the callback</summary>
/// <remarks><para>This System.Pointer value is the argument received by a callback function associated with a Function Or_  operator
/// defined with DefineFunction or DefineOperator.  Once a uCalc.Callback object is construct with this handle, this
/// handle typically no longer needs to be refered to directly.  However, it&#39;s still avaible should you need it.
/// uCalc interfaces with this handle underneath the hood of the uCalc.Callback object.
/// </para></remarks>
    function uCalc.Callback.Handle(): System.Pointer;
    begin
        Result :=  Handle_Overload_Callback__(ExprPartHandle);
    end;

    /// <summary> Returns the Function Or_  operator Item of the current expression part</summary>
/// <remarks><para>If several functions and/or operators are mapped to the same callback function, you can use this to
/// let you determine which one is currently being called, based on properties you can check.</para></remarks>
    function uCalc.Callback.OpItem(): Item;
    begin
        Result :=  uCalc.Item.Create(OpItem__(ExprPartHandle));
    end;

    constructor uCalc.Callback.Create;
    begin
       ExprPartHandle := 0;
    end;

    constructor uCalc.Callback.Create(ExpressionHandle: System.Pointer);
    begin
       ExprPartHandle := ExpressionHandle;
    end;

    constructor uCalc.Callback.Create(ExprInstance: Callback);
    begin
       ExprPartHandle := ExprInstance.ExprPartHandle;
    end;

    /// <summary> Allocates an array of the given data type</summary>
/// <param name="ArraySize"> Size - in umber of elements - of the array you&#39;re allocating</param>
/// <returns> System.Pointer of the array that was allocated</returns>
/// <remarks><para>This allocates an array of the given type.  Elements of the array are set to the default value
/// of that type.</para></remarks>
    function uCalc.DataType.AllocArray(ArraySize: Int32): System.Pointer;
    begin
        Result :=  AllocArray__(DataTypeHandle,ArraySize);
    end;

    /// <summary> Allocates a scalar of the given type and copies data from the area indicated by the System.Pointer</summary>
/// <param name="ValuePtr"> System.Pointer to the data that is copied to the scalar being defined</param>
/// <returns> System.Pointer of the scalar that was allocated</returns>
    function uCalc.DataType.AllocByPtr(ValuePtr: System.Pointer): System.Pointer;
    begin
        Result :=  AllocByPtr__(DataTypeHandle,ValuePtr);
    end;

    /// <summary> Allocates a scalar of the given data type</summary>
/// <param name="Value"> Expression who&#39;s evaluated value will be copied to the allocated scalar</param>
/// <returns> System.Pointer of the scalar that was allocated</returns>
    function uCalc.DataType.AllocScalar(Value: AnsiString = ''): System.Pointer;
    begin
        Result :=  AllocScalar__(DataTypeHandle,Value);
    end;

    /// <summary> Returns the size (in bytes) of one unit of data for this data type</summary>
    function uCalc.DataType.ByteSize(): Int32;
    begin
        Result :=  ByteSize__(DataTypeHandle);
    end;

    /// <summary> Returns the index of the data type</summary>
/// <remarks><para>The return value is a BuiltInTypesEnum enumerator value that represents the data type of the
/// current object.  For instance, if the data type is a string, the return value would be Type_String.
/// </para></remarks>
    function uCalc.DataType.Index(): BuiltInTypesEnum;
    begin
        Result :=  Index__(DataTypeHandle);
    end;

    /// <summary> Returns the uCalc.Item handle of the data type</summary>
/// <remarks><para>Each data type object can also be represented as a uCalc object by using this return value.</para></remarks>
    function uCalc.DataType.GetItem(): Item;
    begin
        Result :=  uCalc.Item.Create(GetItem__(DataTypeHandle));
    end;

    /// <summary> Returns the name of the data type</summary>
    function uCalc.DataType.Name(): AnsiString;
    begin
        Result :=  Name_Overload_DataType__(DataTypeHandle);
    end;

    /// <summary> Releases an array that was allocated with AllocArray</summary>
/// <param name="ValuePtr"> System.Pointer of uCalc array to be released from memory</param>
/// <remarks><para>This should only be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
    procedure uCalc.DataType.ReleaseArray(ValuePtr: System.Pointer);
    begin
          ReleaseArray__(DataTypeHandle,ValuePtr);
    end;

    /// <summary> Releases a scalar that was allocated with AllocScalar</summary>
/// <param name="ValuePtr"> System.Pointer of uCalc scalar to be released from memory</param>
/// <remarks><para>This should only be used for scalars that were explicitly allocated with uCalc&#39;s AllocScalar.</para></remarks>
    procedure uCalc.DataType.ReleaseScalar(ValuePtr: System.Pointer);
    begin
          ReleaseScalar__(DataTypeHandle,ValuePtr);
    end;

    /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
/// <param name="ValuePtr"> System.Pointer of the uCalc array to be reset</param>
/// <remarks><para>Resets all the elements of the array to the default value for the given data type.  This should only
/// be used for arrays that were explicitly allocated with uCalc&#39;s AllocArray.</para></remarks>
    procedure uCalc.DataType.ResetArray(ValuePtr: System.Pointer);
    begin
          ResetArray__(DataTypeHandle,ValuePtr);
    end;

    /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
/// <param name="ValuePtr"> System.Pointer of the uCalc scalar to be reset</param>
    procedure uCalc.DataType.ResetScalar(ValuePtr: System.Pointer);
    begin
          ResetScalar__(DataTypeHandle,ValuePtr);
    end;

    /// <summary> Sets the value of an array at a given memory location that was allocated by AllocArray</summary>
/// <param name="Destination"> System.Pointer of array (base address) who&#39;s element value is to be set</param>
/// <param name="Source"> System.Pointer to a value of the same data type that will be copied to the element</param>
/// <param name="Index"> Index of the array element who&#39;s value needs to be set</param>
    procedure uCalc.DataType.SetArray(Destination: System.Pointer ; Source: System.Pointer ; Index: Int32);
    begin
          SetArray__(DataTypeHandle,Destination , Source , Index);
    end;

    /// <summary> Sets the value of a scalar at a given memory location that was allocated by AllocScalar</summary>
/// <param name="Destination"> System.Pointer of scalar who&#39;se value is to be set</param>
/// <param name="Source"> System.Pointer to a value of the same data type that will be copied to the element</param>
    procedure uCalc.DataType.SetScalar(Destination: System.Pointer ; Source: System.Pointer);
    begin
          SetScalar__(DataTypeHandle,Destination , Source);
    end;

    /// <summary> swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</summary>
/// <param name="ValuePtrA"> System.Pointer of variable 1</param>
/// <param name="ValuePtrB"> System.Pointer of variable 2</param>
    procedure uCalc.DataType.SwapScalarValues(ValuePtrA: System.Pointer ; ValuePtrB: System.Pointer);
    begin
          SwapScalarValues__(DataTypeHandle,ValuePtrA , ValuePtrB);
    end;

    /// <summary> Retuns a string representing the value at the given address formatted for the given data type</summary>
/// <param name="ValuePtr"> System.Pointer of the data who&#39;s value you want to display</param>
/// <remarks><para>If you have a System.Pointer to data of the given type, you can display this data as a string with this function.  Special formatting
/// associated with a data type is used, if such formatting was defined.  For instance, if you&#39;re dealing with a Boolean, then
/// you might get: True, Yes, On, etc, depending on how the formatting was configured with SetOutput.</para></remarks>
    function uCalc.DataType.ToString(ValuePtr: System.Pointer): AnsiString;
    begin
        Result :=  ToString_Overload_DataType_ByPtr__(DataTypeHandle,ValuePtr);
    end;

    /// <summary> Retuns a formatted string of the given data type representing the value that was passed as text</summary>
/// <param name="Value"> String representing data of any type that is to be represented as data of the given type</param>
/// <remarks><para>If you pass a string, it will be converted to the given data type, and return using the formatting
/// for that data type.</para></remarks>
    function uCalc.DataType.ToString(Value: AnsiString): AnsiString;
    begin
        Result :=  ToString_Overload_DataType_ByStr__(DataTypeHandle,Value);
    end;

    /// <summary> Returns the handle of the data type object</summary>
/// <remarks><para>Generally for internal use</para></remarks>
    function uCalc.DataType.Handle(): System.Pointer;
    begin
        Result :=  Handle_Overload_DataType__(DataTypeHandle);
    end;

    constructor uCalc.DataType.Create;
    begin
       DataTypeHandle := 0;
    end;

    constructor uCalc.DataType.Create(PassedDataTypeHandle: System.Pointer);
    begin
       DataTypeHandle := PassedDataTypeHandle;
    end;

    constructor uCalc.DataType.Create(DataTypeHandleInstance: DataType);
    begin
       DataTypeHandle := DataTypeHandleInstance.DataTypeHandle;
    end;

    /// <summary> Adds an error handler</summary>
/// <param name="ErrorHandlerAddress"> Address of the callback function containing the error handler code</param>
/// <param name="Position"> Position in which to insert the error handler</param>
/// <returns> The error handler can later be removed using this return object</returns>
/// <remarks><para>Use this to set a a callback function that is called whenever there is an error.  Your callback can
/// do things like alert the user of the error, correct the error and resume, keep a log of errors, etc.
/// </para><para>
/// By default, if there are multiple error handlers, they are called starting with the most recently
/// defined one first, ending with the first one to be defined last.  In order to define a handler in a different
/// order, specify the Position value of where it should be inserted; 0 so that it is called first, -1 so it
/// is called last, or a positive integer indicated where it should be inserted.  If the Postion value is out
/// of bounds, it is adjusted into the maximum acceptable value.
/// </para><para>
/// If you set ErrorResponse to ErrorHandlerResponseEnum.ErrorResume, subsequent error handler callbacks will
/// be bypassed and uCalc will resume operations as though no error had taken place.  Note that the condition
/// that caused the error must be corrected before resuming, otherwise uCalc may get stuck in an infinite loop.
/// </para><para>
/// Some functions, such as ErrorExpression, ErrorLocation, and ErrorSymbol produce a meaningful result only
/// during the parsing stage.  They do not cause other problems beyond meaningless data during the evaluation stage.
/// </para><para>
/// For Floating Point Environment errors, such as: Err_Float_Division_By_Zero, Err_Float_Inexact_Result,
/// Err_Float_Invalid_Operation, Err_Float_Overflow, and Err_Float_Underflow, the uCalc error handler callbacks
/// are called only if the corresponding environment bits are not masked.
/// </para><para>
/// </para></remarks>
    function uCalc.AddErrorHandler(ErrorHandlerAddress: System.Pointer ;  Position: Int32  = 0): Item;
    begin
        Result :=  uCalc.Item.Create(AddErrorHandler__(uCalcHandle,ErrorHandlerAddress ,Position));
    end;

    /// <summary> Creates a new uCalc instance</summary>
/// <remarks><para>This is not meant to be called directly.</para></remarks>
    function uCalc.CreateuCalc(): System.Pointer;
    begin
        Result :=  CreateuCalc__(uCalcHandle);
    end;

    /// <summary> Returns a data type object</summary>
/// <param name="Index"> Index associated with the data type</param>
/// <remarks><para>Each instance of uCalc has its own separate handle &amp; data type object for any given data type.  For
/// instance, GetItemOf(&quot;Int64&quot;) in one instance of uCalc does not share the same handle as GetItemOf(&quot;Int64&quot;) in
/// another instance.  This allows the same data type to have a different name and different formatting in
/// different instances.  However, they share the same data type index across instances.  Given the index
/// of a data type, you can directly obtain the proper data type object for a particular uCalc instance,
/// faster than obtaining it by name as text with GetItemOf.</para></remarks>
    function uCalc.DataTypeOfIndex(Index: BuiltInTypesEnum): DataType;
    begin
        Result :=  uCalc.DataType.Create(DataTypeOfIndex__(uCalcHandle,Index));
    end;

    /// <summary> Defines a uCalc.Item object (a constunction, function, operator, variable, etc)</summary>
/// <param name="Expression"> Text containing item to be defined.  Expression must start with a keyword such as Function, Operator, Variable, Token, Syntax, etc., followed by a colon, before the expression</param>
/// <param name="Address"> Address a Function Or_  variable is attached to</param>
/// <param name="ExistingItem"> Address of existing item this definition is associated with</param>
/// <remarks><para>Functions, operators, variables, constants, tokens, syntax constructs, etc., can be defined with
/// this method.  Although there are specialized methods that do the same things, such as DefineFunction,
/// DefineOperator, DefineVariable, DefineConstant, and DefineSyntax, this method may in same cases offer
/// more flexibility.
/// </para><para>
/// The expression must be preceded by a keyword and colon, such as &quot;Variable:&quot;.  For a full list of keywords,
/// see Definition Keywords.
/// </para><para>
/// Sections are separated by ~~ (two tilda signs).
/// </para></remarks>
    function uCalc.Define(Expression: AnsiString ;  Address: System.Pointer = nil;  ExistingItem: System.Pointer = nil): Item;
    begin
        Result :=  uCalc.Item.Create(Define__(uCalcHandle,Expression ,Address,ExistingItem));
    end;

    /// <summary> Defines a constant</summary>
/// <param name="Definition"> Text containing constant to be defined</param>
/// <param name="VariableAddress"> Address the constant is attached to</param>
/// <remarks><para>
/// A constant definition is similar to that of a variable defined with DefineVariable, except that the end-user is not allowed to modify the value of a constant.
/// You are, however, allowed to perform lower-level operations such as renaming a constant, or deleting it and re-defining a new constant or variable with the same
/// name but different value.  If you attach the constant to the address of a variable in your host program, the host program may change the value.  However, when
/// the end-user attempts to assign a value to the constant, an error is raised.
/// </para><para>
/// DefineConstant(&quot;x&quot;) is the same as: Define(&quot;Lock ~~ Variable: x&quot;)
/// </para></remarks>
    function uCalc.DefineConstant(Definition: AnsiString ;  VariableAddress: System.Pointer = nil): Item;
    begin
        Result :=  uCalc.Item.Create(DefineConstant__(uCalcHandle,Definition ,VariableAddress));
    end;

    /// <summary> Defines a data type</summary>
/// <param name="TypeSize"> Number of bytes 1 scalar value of this type takes</param>
/// <param name="AllocArrayValue"> Address of a function that will allocate an array of this type</param>
/// <param name="AllocScalar"> Address of a function that will allocate a scalar of this type.  If a string is passed, this function must convert it to this type and store the value, otherwise the function must store a default value for this type.</param>
/// <param name="AllocScalarByPtr"> Address of a function that takes a System.Pointer to a value of this type, allocates a scalar and stores the copied value in the allocated space.</param>
/// <param name="DeleteArray"> Address of a function that frees a memory block that was allocated with AllocValueArray</param>
/// <param name="DeleteScalar"> Address of a function that frees a memory block that was allocated by AllocScalar or AllocScalarByPtr.</param>
/// <param name="ResetScalar"> Address of a function that resets an allocated scalar back to the default value of this type.</param>
/// <param name="ResetArray"> Address of a function that resets array elements back to the default value of this type.</param>
/// <param name="SetValueArray"> Address of a function that stores an array element value.</param>
/// <param name="SetValueScalar"> Address of a function that stores a scalar value.</param>
/// <param name="SwapValues"> Address of a function that swaps two values of this type.</param>
/// <param name="ToString"> Address of a function that converts this data type to a string.</param>
/// <remarks><para>
/// This allows you to define a data type that functions at the same level as the intrinsic types that come with uCalc.
/// </para></remarks>
    function uCalc.DefineDataType(TypeSize: Int32 ; AllocArrayValue: System.Pointer ; AllocScalar: System.Pointer ; AllocScalarByPtr: System.Pointer ; DeleteArray: System.Pointer ; DeleteScalar: System.Pointer ; ResetScalar: System.Pointer ; ResetArray: System.Pointer ; SetValueArray: System.Pointer ; SetValueScalar: System.Pointer ; SwapValues: System.Pointer ; ToString: System.Pointer): Item;
    begin
        Result :=  uCalc.Item.Create(DefineDataType__(uCalcHandle,TypeSize , AllocArrayValue , AllocScalar , AllocScalarByPtr , DeleteArray , DeleteScalar , ResetScalar , ResetArray , SetValueArray , SetValueScalar , SwapValues , ToString));
    end;

    /// <summary> Defines a function</summary>
/// <param name="Definition"> Text containing function to be defined</param>
/// <param name="FunctionAddress"> Address of callback function this function is attached to</param>
/// <remarks><para>
/// A function definition can either be self-contained, or it may be attached to the address of a callback function in your
/// source code.
/// </para><para>
/// DefineFunction(&quot;f(x) = ...&quot;) is the same as: Define(&quot;Function: ...&quot;)
/// </para></remarks>
    function uCalc.DefineFunction(Definition: AnsiString ;  FunctionAddress: System.Pointer = nil): Item;
    begin
        Result :=  uCalc.Item.Create(DefineFunction__(uCalcHandle,Definition ,FunctionAddress));
    end;

    /// <summary> Defines an operator</summary>
/// <param name="Definition"> Text containing operator to be defined</param>
/// <param name="iPrecedence"> Precedence level of the operator.</param>
/// <param name="Grouping"> Operator associativity; arithmetic operators are typically grouped left-to-right; special operators, such as the equal sign, =, assignment operator are right-to-left</param>
/// <param name="FunctionAddress"> Address of callback function this operator is attached to</param>
/// <remarks><para>
/// Many details for defining an operator are similar to those of defining a function.  In additionn, all operators must be
/// defined with a precedence level.  You may use the Precedence function to obtain precedence levels of existing operators,
/// and select a precedence level in comparison to those operators.  By default operators are grouped left to right.  But you
/// may also set it to right to left.  This means that when adjacent operators have the same precedence level, the operands on
/// the right will be evaluated first.
/// </para><para>
/// DefineOperator(&quot;...&quot;) is the same as: Define(&quot;Operator: ...&quot;)
/// More later+++
/// </para></remarks>
    function uCalc.DefineOperator(Definition: AnsiString ; iPrecedence: Int32 ;  Grouping: GroupingEnum = GroupingEnum.GroupLeftToRight;  FunctionAddress: System.Pointer = nil): Item;
    begin
        Result :=  uCalc.Item.Create(DefineOperator__(uCalcHandle,Definition , iPrecedence ,Grouping,FunctionAddress));
    end;

    /// <summary> Defines a syntax construct</summary>
/// <param name="Definition"> Text containing syntax construct</param>
    function uCalc.DefineSyntax(Definition: AnsiString): Item;
    begin
        Result :=  uCalc.Item.Create(DefineSyntax__(uCalcHandle,Definition));
    end;

    /// <summary> Defines a transform</summary>
/// <param name="FromExpr"> Pattern representing the text that must be transformed</param>
/// <param name="ToExpr"> The text that the matching test is transformed to</param>
/// <param name="ItemHandle"> Handle</param>
    function uCalc.DefineTransform(FromExpr: AnsiString ; ToExpr: AnsiString ;  ItemHandle: System.Pointer = nil): Item;
    begin
        Result :=  uCalc.Item.Create(DefineTransform__(uCalcHandle,FromExpr , ToExpr ,ItemHandle));
    end;

    /// <summary> Defines a variable</summary>
/// <param name="Definition"> Text containing variable to be defined</param>
/// <param name="VariableAddress"> Address of variable in host program this uCalc variable is attached to</param>
/// <remarks><para>
/// A variable consists of ...
/// </para><para>
/// DefineVariable(&quot;...&quot;) is the same as: Define(&quot;Variable: x&quot;)
/// </para></remarks>
    function uCalc.DefineVariable(Definition: AnsiString ;  VariableAddress: System.Pointer = nil): Item;
    begin
        Result :=  uCalc.Item.Create(DefineVariable__(uCalcHandle,Definition ,VariableAddress));
    end;

    /// <summary> Returns the expression associated with an error that was raised</summary>
    function uCalc.ErrorExpression(): AnsiString;
    begin
        Result :=  ErrorExpression__(uCalcHandle);
    end;

    /// <summary> Returns the location of an error that was raised</summary>
    function uCalc.ErrorLocation(): Int32;
    begin
        Result :=  ErrorLocation__(uCalcHandle);
    end;

    /// <summary> Returns a message associated with the error that was raised</summary>
/// <param name="Index"> Index of error message to return</param>
/// <remarks><para>If an error was raised, this returns a message describing the recent error.  If the Index argument is specified,
/// then the error message associated with this Index is returned.  A value must be chosen from ErrorNumberEnum.
/// </para><para>
/// Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
/// </para></remarks>
    function uCalc.ErrorMessage(Index: ErrorNumberEnum = ErrorNumberEnum.Err_None): AnsiString;
    begin
        Result :=  ErrorMessage__(uCalcHandle,Index);
    end;

    /// <summary> Returns the error number of the error that was raised</summary>
/// <remarks><para>
/// Returns the error number of the most recently raised error.
/// </para><para>
/// Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
/// </para></remarks>
    function uCalc.ErrorNumber(): ErrorNumberEnum;
    begin
        Result :=  ErrorNumber__(uCalcHandle);
    end;

    /// <summary> Sets the error response for an error handler</summary>
/// <param name="Response"> Indicates what should happen now that an error is raised</param>
/// <remarks><para>When an error is raised, you have the option of aborting the rest of the option (which is the default option),
/// or you may correct the problem, and then resume the operation.  Or you may perform an action, and than raise the
/// error again for a subsequent error handle to take care of.
/// </para><para>
/// This method can only be called from within a callback of an error handler routine definined with AddErrorHandler.
/// The options to use can be found at ErrorHandlerResponseEnum.
/// </para></remarks>
    procedure uCalc.ErrorResponse(Response: ErrorHandlerResponseEnum);
    begin
          ErrorResponse__(uCalcHandle,Response);
    end;

    /// <summary> Returns the symbol associated with the error that is raised</summary>
    function uCalc.ErrorSymbol(): AnsiString;
    begin
        Result :=  ErrorSymbol__(uCalcHandle);
    end;

    /// <summary> Sets a new expression after an error is raised</summary>
/// <param name="Expression"> Error expression</param>
    procedure uCalc.ErrorSetExpression(Expression: AnsiString);
    begin
          ErrorSetExpression__(uCalcHandle,Expression);
    end;

    /// <summary> Parses and evaluates an expression in one step</summary>
/// <param name="Expression"> Expression to be evaluated</param>
/// <remarks><para>This takes a math expression and returns a numeric result.</para></remarks>
    function uCalc.Eval(Expression: AnsiString): Double;
    begin
        Result :=  Eval__(uCalcHandle,Expression);
    end;

    /// <summary> Parses and evaluates an expression of any data type in one step, returning the result as a string</summary>
/// <param name="Expression"> Expression to be evaluated</param>
/// <remarks><para>This is similar to Eval, however, it can return non-numeric results, such as strings.</para></remarks>
    function uCalc.EvalStr(Expression: AnsiString): AnsiString;
    begin
        Result :=  EvalStr__(uCalcHandle,Expression);
    end;

    /// <summary> Expands an expression</summary>
/// <param name="Expression"> Expression to be expanded</param>
    function uCalc.Expand(Expression: AnsiString): AnsiString;
    begin
        Result :=  Expand__(uCalcHandle,Expression);
    end;

    /// <summary> Returns each step leading to an expression expansion</summary>
/// <param name="Expression"> Expression to be expanded</param>
/// <param name="Separator"> Separator used between steps</param>
    function uCalc.ExpandSteps(Expression: AnsiString ;  Separator: AnsiString = ''): AnsiString;
    begin
        Result :=  ExpandSteps__(uCalcHandle,Expression ,Separator);
    end;

    /// <summary> Clears floating point exception(s) that were raised</summary>
/// <param name="Exceptions"> Exceptions</param>
    function uCalc.FPEnvClearException(Exceptions: Int32  = 0): Int32;
    begin
        Result :=  FPEnvClearException__(uCalcHandle,Exceptions);
    end;

    /// <summary> Returns the floating point environment error mask</summary>
    function uCalc.FPEnvGetExceptionMask(): Int32;
    begin
        Result :=  FPEnvGetExceptionMask__(uCalcHandle);
    end;

    /// <summary> Raises a floating point exception in a uCalc.Callback function</summary>
/// <param name="Exception"> Bitmask value consisting of one or more exception flags to raise</param>
/// <returns> Returns a 0 if all the specified execptions were raised successfully, otherwise a non-zero value is returned.</returns>
/// <remarks><para>This attempts to raise the exception(s) specfied by the Exception parameter.  This is based on the C++
/// feraiseexcept() function.  If you are using C++, you may use the pre-defined macro values of 8,
/// 1, 16, 4, 2, 31.  If you are using another compiler, you may
/// use the matching members of the FPEnvExceptionEnum enumerator.
/// </para></remarks>
    function uCalc.FPEnvRaiseException(Exception: FPEnvExceptionEnum = FPEnvExceptionEnum.Invalid): Int32;
    begin
        Result :=  FPEnvRaiseException__(uCalcHandle,Exception);
    end;

    /// <summary> Gets the current floating point environment rounding option</summary>
    function uCalc.FPEnvGetRounding(): FPEnvRoundingEnum;
    begin
        Result :=  FPEnvGetRounding__(uCalcHandle);
    end;

    /// <summary> Sets the current floating point environment mask</summary>
    procedure uCalc.FPEnvSetExceptionMask(Exceptions: Int32);
    begin
          FPEnvSetExceptionMask__(uCalcHandle,Exceptions);
    end;

    /// <summary> Unmasks the current floating point environment mask</summary>
    procedure uCalc.FPEnvSetExceptionUnMask(Exceptions: Int32);
    begin
          FPEnvSetExceptionUnMask__(uCalcHandle,Exceptions);
    end;

    /// <summary> Sets the floating point environment rounding option</summary>
    function uCalc.FPEnvSetRounding(Direction: FPEnvRoundingEnum): Int32;
    begin
        Result :=  FPEnvSetRounding__(uCalcHandle,Direction);
    end;

    /// <summary> Toggles the floating point environment mask</summary>
    procedure uCalc.FPEnvToggleExceptionMask(Exception: FPEnvExceptionEnum);
    begin
          FPEnvToggleExceptionMask__(uCalcHandle,Exception);
    end;

    /// <summary> Returns the handle of a uCalc.Item given a name</summary>
    function uCalc.HandleOf(ItemName: AnsiString;   Properties: Int64 = 0;   SkipExpansion: Boolean = False): System.Pointer;
    begin
        Result :=  HandleOf__(uCalcHandle,ItemName, Properties, Ord(SkipExpansion));
    end;

    /// <summary> Returns the uCalc.Item object associated with the given name</summary>
/// <param name="ItemName"> Name of uCalc.Item that you want to get</param>
/// <param name="Properties"> Properties to help pinpoint the right Item, in case several items share the same name</param>
/// <param name="SkipExpansion"> Some items are define as a syntax construct alias of another item.  By default the symbol is expanded first.  However, use this option if the alias is the object you&#39;re actually after.</param>
/// <remarks><para>
/// If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
/// </para><para>
/// In case multiple items share the same name, you can narrow it down using properties.  For instance, the + operator has a
/// binary and unary versions.  You can specify Prefix if you wanted the unary one.
/// </para><para>
/// Note: A ItemIsEnum member should not be passed directly as the second argument.  Instead Properties, with one or more
/// ItemIsEnum members should be passed.
/// </para></remarks>
    function uCalc.GetItemOf(ItemName: AnsiString ;  Properties: Int64  = 0;  SkipExpansion: Boolean = False): Item;
    begin
        Result :=  uCalc.Item.Create(GetItemOf__(uCalcHandle,ItemName ,Properties,Ord(SkipExpansion)));
    end;

    /// <summary> Returns the nth uCalc.Item object that matches a property</summary>
/// <param name="Property_"> Property of the uCalc.Item object you&#39;re after</param>
/// <param name="nth"> The nth object</param>
/// <param name="DataTypeName"> The data type of the object</param>
/// <remarks><para>
/// This can be used to cycle through defined items.
/// If there is no match, an item whose IsProperty ItemIsEnum.NotFound value is true is returned.
/// </para></remarks>
    function uCalc.GetItemOfProperty(Property_: ItemIsEnum ; nth: Int32 ;  DataTypeName: AnsiString = ''): Item;
    begin
        Result :=  uCalc.Item.Create(GetItemOfProperty__(uCalcHandle,Property_ , nth ,DataTypeName));
    end;

    /// <summary> Sets the license code</summary>
    function uCalc.License(LicenseType: LicenseTypeEnum;   L1: Int32 = 0;   L2: Int32 = 0;   L3: Int32 = 0;   L4: Int32 = 0;   L5: Int32 = 0): Int32;
    begin
        Result :=  License__(uCalcHandle,LicenseType, L1, L2, L3, L4, L5);
    end;

    /// <summary> ???</summary>
    function uCalc.NamespaceChild(nth: Int32 = 0): Item;
    begin
        Result :=  uCalc.Item.Create(NamespaceChild__(uCalcHandle,nth));
    end;

    /// <summary> ???</summary>
    function uCalc.NamespaceChildCount(): Int32;
    begin
        Result :=  NamespaceChildCount__(uCalcHandle);
    end;

    /// <summary> ???</summary>
    function uCalc.NamespaceParent(Level: Int32 = 0): Item;
    begin
        Result :=  uCalc.Item.Create(NamespaceParent__(uCalcHandle,Level));
    end;

    /// <summary> ???</summary>
    function uCalc.NamespaceSymbol(nth: Int32): Item;
    begin
        Result :=  uCalc.Item.Create(NamespaceSymbol__(uCalcHandle,nth));
    end;

    /// <summary> ???</summary>
    function uCalc.NamespaceSymbolCount(): Int32;
    begin
        Result :=  NamespaceSymbolCount__(uCalcHandle);
    end;

    /// <summary> Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
/// <param name="Expression"> Expression to be parsed</param>
/// <param name="DataType"> Data type of return value</param>
    function uCalc.Parse(Expression: AnsiString ;  DataType: AnsiString = ''): ParsedExpr;
    begin
        Result :=  uCalc.ParsedExpr.Create(Parse__(uCalcHandle,Expression ,DataType));
    end;

    /// <summary> Returns the precedence level of an operator or the level between two operators</summary>
    function uCalc.Precedence(Operator1: AnsiString;   Operator2: AnsiString = ''): Int32;
    begin
        Result :=  Precedence_Overload_uCalc__(uCalcHandle,Operator1, Operator2);
    end;

    /// <summary> Combines properties into a numeric value that can be passed to GetItemOf</summary>
/// <remarks><para>
/// ItemIsEnum properties are numbered sequentially (0, 1, 2, 3, 4, ...).  However the Properties argument
/// of the GetItemOf function requires properties in the 2^x sequence(1; 2; 4; 8; 16; ...).  So you
/// must pass one or more ItemIsEnum to Properties before passing it to GetItemOf.
/// </para></remarks>
    function uCalc.Properties(p1: ItemIsEnum;   p2: ItemIsEnum = ItemIsEnum.AnyItem;   p3: ItemIsEnum = ItemIsEnum.AnyItem;   p4: ItemIsEnum = ItemIsEnum.AnyItem): Int64;
    begin
        Result :=  Properties__(uCalcHandle,p1, p2, p3, p4);
    end;

    /// <summary> (Use Release instead)</summary>
/// <remarks><para>Use Release</para></remarks>
    procedure uCalc.ReleaseuCalc();
    begin
          ReleaseuCalc__(uCalcHandle);
    end;

    /// <summary> Removes any output decoration</summary>
    procedure uCalc.ResetOutput();
    begin
          ResetOutput__(uCalcHandle);
    end;

    /// <summary> Returns default settings</summary>
/// <remarks><para>The Item returned contains the default settings for the current uCalc instance.  To define new settings,
/// use Define(&quot;Default ~~ ...&quot;).</para></remarks>
    function uCalc.Default(): Item;
    begin
        Result :=  uCalc.Item.Create(Default__(uCalcHandle));
    end;

    /// <summary> Allows a callback to decorate the output of EvalStr</summary>
/// <param name="FunctionAddress"> Address of callback function that formats the output</param>
/// <param name="Properties"> Name of datatype whose output will be formated and/or ranking among other output formats; leave blank to affect all data types and use default ranking.</param>
    function uCalc.SetOutput(FunctionAddress: System.Pointer ; Properties: AnsiString): Item;
    begin
        Result :=  uCalc.Item.Create(SetOutput_Overload_uCalc_Callback__(uCalcHandle,FunctionAddress , Properties));
    end;

    /// <summary> Formats the appearance of results returned by EvalStr or EvaluateStr</summary>
/// <param name="Definition"> Format definition for the output</param>
/// <remarks><para>
/// This function lets you modify the output returned by EvalStr or EvaluateStr.  The definition may contain one or more
/// of the following:
/// </para><para>
/// Datatype:  If you specify a data type, then the modification will affect only the evaluation result of expressions
/// that evaluate to the specified data type.
/// </para><para>
/// InsertAt:  By default, if you define multiple outputs, then the one defined most recently is applied first, followed by
/// the other outputs in reverse order in which they were defined, ending with the output that was defined first.  You may
/// specify a different insertion point than the default with this parameter.
/// </para><para>
/// Expression:  This represents what the modified output should look like.  The expression consists of an identifier
/// (any temporary variable name), followed by an equal sign, followed by a string expression representing the value
/// that will be returned.  The variable holds the original value that would have been returned.
/// </para></remarks>
    function uCalc.SetOutput(Definition: AnsiString): Item;
    begin
        Result :=  uCalc.Item.Create(SetOutput_Overload_uCalc_NoCallback__(uCalcHandle,Definition));
    end;

    /// <summary> Returns the value located at the specified System.Pointer address</summary>
    function uCalc.ValueAt(ValuePtr: System.Pointer; DataType: AnsiString): AnsiString;
    begin
        Result :=  ValueAt__(uCalcHandle,ValuePtr, DataType);
    end;

    /// <summary> Returns the handle of the uCalc object</summary>
/// <remarks><para>Generally for internal use</para></remarks>
    function uCalc.Handle(): System.Pointer;
    begin
        Result :=  Handle_Overload_uCalc__(uCalcHandle);
    end;

     /// <summary> Releases a uCalc instance</summary>
/// <remarks><para>When a uCalc instance is no longer needed, it can be released with this function.  This releases all items
/// associated with the uCalc instance.
/// </para><para>
/// Note: The uCalc object named uc, which is instantiated at startup (except in C#) should not be explicitely released.
/// In C#, if there is only 1 uCalc object left, do not explicitely release it.
/// </para><para>
/// Note: A uCalc object must explicitely be released with Release for it to be released.  A uCalc object is not
/// automatically released from memory when the object goes out of scope.</para></remarks>
    procedure uCalc.Release();
    begin
        ReleaseuCalc(); uCalcHandle := 0;
    end;

    
    procedure uCalc.Reset();
    begin
        ReleaseuCalc(); uCalcHandle := CreateuCalc();
    end;

    constructor uCalc.Create;
    begin
       uCalcHandle := 0; uCalcHandle := CreateuCalc();
    end;

    constructor uCalc.Create(uCalcInstance: uCalc);
    begin
       uCalcHandle := uCalcInstance.uCalcHandle;
    end;

    constructor uCalc.Create(InstHandle: System.Pointer);
    begin
       uCalcHandle := InstHandle;
    end;

    constructor uCalc.Create(LicenseType: LicenseTypeEnum;   L1: Int32 = 0;   L2: Int32 = 0;   L3: Int32 = 0;   L4: Int32 = 0;   L5: Int32 = 0);
    begin
       License(LicenseType, L1, L2, L3, L4, L5); uCalcHandle := CreateuCalc();
    end; 





initialization
   uc := uCalc.Create;
end.
